@@ -1,6 +1,6 @@
 <?php
 /*
- * Copyright (C) 2023 Xibo Signage Ltd
+ * Copyright (C) 2024 Xibo Signage Ltd
  *
  * Xibo - Digital Signage - https://xibosignage.com
  *
@@ -117,7 +117,11 @@ class AlphaVantageConnector implements ConnectorInterface
                 $event->getDataProvider()->setCacheTtl($this->getSetting('cachePeriod', 3600));
             } catch (\Exception $exception) {
                 $this->getLogger()->error('onDataRequest: Failed to get results. e = ' . $exception->getMessage());
-                $dataProvider->addError(__('Unable to contact the AlphaVantage API'));
+                if ($exception instanceof InvalidArgumentException) {
+                    $dataProvider->addError($exception->getMessage());
+                } else {
+                    $dataProvider->addError(__('Unable to contact the AlphaVantage API'));
+                }
             }
         }
     }
@@ -231,7 +235,7 @@ class AlphaVantageConnector implements ConnectorInterface
 
         if ($items == '') {
             $this->getLogger()->error('Missing Items for Stocks Module with WidgetId ' . $dataProvider->getWidgetId());
-            throw new InvalidArgumentException(__('Missing Items for Stocks Module'), 'items');
+            throw new InvalidArgumentException(__('Add some stock symbols'), 'items');
         }
 
         // Parse items out into an array
@@ -296,10 +300,11 @@ class AlphaVantageConnector implements ConnectorInterface
     protected function getStockQuote(string $symbol, ?int $isPaidPlan): array
     {
         try {
-            $cache = $this->getPool()->getItem('/widget/stock/'.md5($symbol));
+            $cache = $this->getPool()->getItem('/widget/stock/api_'.md5($symbol));
             $cache->setInvalidationMethod(Invalidation::SLEEP, 5000, 15);
 
             $data = $cache->get();
+
             if ($cache->isMiss()) {
                 $this->getLogger()->debug('AlphaVantage Connector : getStockQuote is served from the API.');
 
@@ -405,6 +410,18 @@ class AlphaVantageConnector implements ConnectorInterface
         // Parse items out into an array
         $items = array_map('trim', explode(',', $items));
 
+        // Ensure base isn't also in the items list (Currencies)
+        if (in_array($base, $items)) {
+            $this->getLogger()->error(
+                'Invalid Currencies: Base "' . $base . '" also included in Items for ' .
+                'Currencies Module with WidgetId ' . $dataProvider->getWidgetId()
+            );
+            throw new InvalidArgumentException(
+                __('Base currency must not be included in the Currencies list. Please remove it and try again.'),
+                'items'
+            );
+        }
+
         // Each item we want is a call to the results API
         try {
             foreach ($items as $currency) {
