@@ -27,6 +27,7 @@ use Psr\Log\LoggerInterface;
 use Psr\Log\NullLogger;
 use Stash\Interfaces\PoolInterface;
 use Xibo\Service\JwtServiceInterface;
+use Xibo\Service\PlayerActionServiceInterface;
 
 /**
  * Connector trait to assist with basic scaffolding and utility methods.
@@ -55,6 +56,9 @@ trait ConnectorTrait
     /** @var JwtServiceInterface */
     private $jwtService;
 
+    /** @var PlayerActionServiceInterface */
+    private $playerActionService;
+
     /**
      * @param \Psr\Log\LoggerInterface $logger
      * @return \Xibo\Connector\ConnectorInterface
@@ -155,6 +159,17 @@ trait ConnectorTrait
         return $this->jwtService;
     }
 
+    public function usePlayerActionService(PlayerActionServiceInterface $playerActionService): ConnectorInterface
+    {
+        $this->playerActionService = $playerActionService;
+        return $this;
+    }
+
+    protected function getPlayerActionService(): PlayerActionServiceInterface
+    {
+        return $this->playerActionService;
+    }
+
     public function setFactories($container): ConnectorInterface
     {
         return $this;
