@@ -1,6 +1,6 @@
 <?php
 /*
- * Copyright (C) 2023 Xibo Signage Ltd
+ * Copyright (C) 2024 Xibo Signage Ltd
  *
  * Xibo - Digital Signage - https://xibosignage.com
  *
@@ -31,7 +31,9 @@ class ProviderDetails implements \JsonSerializable
     public $message;
     public $link;
     public $logoUrl;
+    public $iconUrl;
     public $backgroundColor;
+    public $mediaTypes;
 
     public function jsonSerialize(): array
     {
@@ -40,7 +42,9 @@ class ProviderDetails implements \JsonSerializable
             'message' => $this->message,
             'link' => $this->link,
             'logoUrl' => $this->logoUrl,
-            'backgroundColor' => $this->backgroundColor
+            'iconUrl' => $this->iconUrl,
+            'backgroundColor' => $this->backgroundColor,
+            'mediaTypes' => $this->mediaTypes
         ];
     }
 }
