@@ -1,8 +1,8 @@
 <?php
-/**
- * Copyright (C) 2022 Xibo Signage Ltd
+/*
+ * Copyright (C) 2025 Xibo Signage Ltd
  *
- * Xibo - Digital Signage - http://www.xibo.org.uk
+ * Xibo - Digital Signage - https://xibosignage.com
  *
  * This file is part of Xibo.
  *
@@ -33,12 +33,15 @@ use Xibo\Entity\ApplicationScope;
 use Xibo\Factory\ApplicationFactory;
 use Xibo\Factory\ApplicationRedirectUriFactory;
 use Xibo\Factory\ApplicationScopeFactory;
+use Xibo\Factory\ConnectorFactory;
 use Xibo\Factory\UserFactory;
 use Xibo\Helper\DateFormatHelper;
 use Xibo\Helper\Session;
 use Xibo\OAuth\AuthCodeRepository;
 use Xibo\OAuth\RefreshTokenRepository;
 use Xibo\Support\Exception\AccessDeniedException;
+use Xibo\Support\Exception\ControllerNotImplemented;
+use Xibo\Support\Exception\GeneralException;
 use Xibo\Support\Exception\InvalidArgumentException;
 
 /**
@@ -71,6 +74,9 @@ class Applications extends Base
     /** @var PoolInterface */
     private $pool;
 
+    /** @var \Xibo\Factory\ConnectorFactory */
+    private $connectorFactory;
+
     /**
      * Set common dependencies.
      * @param Session $session
@@ -78,15 +84,25 @@ class Applications extends Base
      * @param ApplicationRedirectUriFactory $applicationRedirectUriFactory
      * @param $applicationScopeFactory
      * @param UserFactory $userFactory
+     * @param $pool
+     * @param \Xibo\Factory\ConnectorFactory $connectorFactory
      */
-    public function __construct($session, $applicationFactory, $applicationRedirectUriFactory, $applicationScopeFactory, $userFactory, $pool)
-    {
+    public function __construct(
+        $session,
+        $applicationFactory,
+        $applicationRedirectUriFactory,
+        $applicationScopeFactory,
+        $userFactory,
+        $pool,
+        ConnectorFactory $connectorFactory
+    ) {
         $this->session = $session;
         $this->applicationFactory = $applicationFactory;
         $this->applicationRedirectUriFactory = $applicationRedirectUriFactory;
         $this->applicationScopeFactory = $applicationScopeFactory;
         $this->userFactory = $userFactory;
         $this->pool = $pool;
+        $this->connectorFactory = $connectorFactory;
     }
 
     /**
@@ -99,7 +115,27 @@ class Applications extends Base
      */
     public function displayPage(Request $request, Response $response)
     {
+        // Load all connectors and output any javascript.
+        $connectorJavaScript = [];
+        foreach ($this->connectorFactory->query(['isVisible' => 1]) as $connector) {
+            try {
+                // Create a connector, add in platform settings and register it with the dispatcher.
+                $connectorObject = $this->connectorFactory->create($connector);
+
+                $settingsFormJavaScript = $connectorObject->getSettingsFormJavaScript();
+                if (!empty($settingsFormJavaScript)) {
+                    $connectorJavaScript[] = $settingsFormJavaScript;
+                }
+            } catch (\Exception $exception) {
+                // Log and ignore.
+                $this->getLog()->error('Incorrectly configured connector. e=' . $exception->getMessage());
+            }
+        }
+
         $this->getState()->template = 'applications-page';
+        $this->getState()->setData([
+            'connectorJavaScript' => $connectorJavaScript,
+        ]);
 
         return $this->render($request, $response);
     }
@@ -117,7 +153,13 @@ class Applications extends Base
         $this->getState()->template = 'grid';
         $sanitizedParams = $this->getSanitizer($request->getParams());
 
-        $applications = $this->applicationFactory->query($this->gridRenderSort($sanitizedParams), $this->gridRenderFilter([], $sanitizedParams));
+        $applications = $this->applicationFactory->query(
+            $this->gridRenderSort($sanitizedParams),
+            $this->gridRenderFilter(
+                ['name' => $sanitizedParams->getString('name')],
+                $sanitizedParams
+            )
+        );
 
         foreach ($applications as $application) {
             if ($this->isApi($request)) {
@@ -131,18 +173,17 @@ class Applications extends Base
             $application->buttons = [];
 
             if ($application->userId == $this->getUser()->userId || $this->getUser()->getUserTypeId() == 1) {
-
                 // Edit
                 $application->buttons[] = [
                     'id' => 'application_edit_button',
-                    'url' => $this->urlFor($request,'application.edit.form', ['id' => $application->key]),
+                    'url' => $this->urlFor($request, 'application.edit.form', ['id' => $application->key]),
                     'text' => __('Edit')
                 ];
 
                 // Delete
                 $application->buttons[] = [
                     'id' => 'application_delete_button',
-                    'url' => $this->urlFor($request,'application.delete.form', ['id' => $application->key]),
+                    'url' => $this->urlFor($request, 'application.delete.form', ['id' => $application->key]),
                     'text' => __('Delete')
                 ];
             }
@@ -185,23 +226,29 @@ class Applications extends Base
         // if we have scopes in the request, make sure we only add the valid ones.
         // the default scope is all, if it's not set on the Application, $scopes will still be empty here.
         if ($authScopes !== null) {
-            $validScopes = $this->applicationScopeFactory->finalizeScopes($authScopes, $authParams->getGrantTypeId(), $client);
+            $validScopes = $this->applicationScopeFactory->finalizeScopes(
+                $authScopes,
+                $authParams->getGrantTypeId(),
+                $client
+            );
+
             // get all the valid scopes by their ID, we need to do this to present more details on the authorize form.
             foreach ($validScopes as $scope) {
                 $scopes[] = $this->applicationScopeFactory->getById($scope->getIdentifier());
             }
-        }
 
-        // if the request does not contain any valid scopes, default to all scopes assigned to this Application.
-        if (count($scopes) <= 0) {
-            $validScopes = $client->getScopes();
-            foreach ($validScopes as $scope) {
-                $scopes[] = $this->applicationScopeFactory->getById($scope->getIdentifier());
+            if (count($scopes) <= 0) {
+                throw new InvalidArgumentException(
+                    __('This application has not requested access to anything.'),
+                    'authParams'
+                );
             }
+
+            // update scopes in auth request in session to scopes we actually present for approval
+            $authParams->setScopes($validScopes);
         }
 
-        // update scopes in auth request in session to scopes we actually present for approval
-        $authParams->setScopes($validScopes);
+        // Reasert  the auth params.
         $this->session->set('authParams', $authParams);
 
         // Get, show page
@@ -255,9 +302,6 @@ class Applications extends Base
             new \DateInterval('PT1H')
         );
 
-        // Default scope
-        $server->setDefaultScope('all');
-
         // get oauth User Entity and set the UserId to the current web userId
         $authRequest->setUser($this->getUser());
 
@@ -271,6 +315,16 @@ class Applications extends Base
                 Carbon::now()->format(DateFormatHelper::getSystemFormat()),
                 $request->getAttribute('ip_address')
             );
+
+            $this->getLog()->audit(
+                'Auth',
+                0,
+                'Application access approved',
+                [
+                    'Application identifier ends with' => substr($authRequest->getClient()->getIdentifier(), -8),
+                    'Application Name' => $authRequest->getClient()->getName()
+                ]
+            );
         } else {
             $authRequest->setAuthorizationApproved(false);
         }
@@ -298,9 +352,6 @@ class Applications extends Base
     public function addForm(Request $request, Response $response)
     {
         $this->getState()->template = 'applications-form-add';
-        $this->getState()->setData([
-            'help' => $this->getHelp()->link('Services', 'Register')
-        ]);
 
         return $this->render($request, $response);
     }
@@ -340,7 +391,7 @@ class Applications extends Base
                 }
             }
 
-            $scope->selected = $found ? 1 : 0;
+            $scope->setUnmatchedProperty('selected', $found ? 1 : 0);
         }
 
         // Render the view
@@ -348,8 +399,6 @@ class Applications extends Base
         $this->getState()->setData([
             'client' => $client,
             'scopes' => $scopes,
-            'users' => $this->userFactory->query(),
-            'help' => $this->getHelp()->link('Services', 'Register')
         ]);
 
         return $this->render($request, $response);
@@ -378,7 +427,6 @@ class Applications extends Base
         $this->getState()->template = 'applications-form-delete';
         $this->getState()->setData([
             'client' => $client,
-            'help' => $this->getHelp()->link('Services', 'Register')
         ]);
 
         return $this->render($request, $response);
@@ -416,73 +464,6 @@ class Applications extends Base
         return $this->render($request, $response);
     }
 
-    /**
-     * Form to register a new application for Advertisement.
-     * @param Request $request
-     * @param Response $response
-     * @return \Psr\Http\Message\ResponseInterface|Response
-     * @throws \Xibo\Support\Exception\ControllerNotImplemented
-     * @throws \Xibo\Support\Exception\GeneralException
-     */
-    public function addDoohForm(Request $request, Response $response)
-    {
-        $this->getState()->template = 'applications-form-add-dooh';
-        $this->getState()->setData([
-            'help' => $this->getHelp()->link('Services', 'Register')
-        ]);
-
-        return $this->render($request, $response);
-    }
-
-    /**
-     * Register a new application with OAuth
-     * @param Request $request
-     * @param Response $response
-     * @return \Psr\Http\Message\ResponseInterface|Response
-     * @throws InvalidArgumentException
-     * @throws \Xibo\Support\Exception\ControllerNotImplemented
-     * @throws \Xibo\Support\Exception\GeneralException
-     * @throws \Xibo\Support\Exception\NotFoundException
-     */
-    public function addDooh(Request $request, Response $response)
-    {
-        $sanitizedParams = $this->getSanitizer($request->getParams());
-        $application = $this->applicationFactory->create();
-        $application->name = $sanitizedParams->getString('name');
-        $application->userId = $sanitizedParams->getInt('userId');
-
-        if ($application->name == '') {
-            throw new InvalidArgumentException(__('Please enter Application name'), 'name');
-        }
-
-        if ($application->userId == null) {
-            throw new InvalidArgumentException(__('Please select user'), 'userId');
-        }
-
-        if ($this->userFactory->getById($application->userId)->userTypeId != 4) {
-            throw new InvalidArgumentException(__('Invalid user type'), 'userTypeId');
-        }
-
-        // The dooh application is always confidential, and always client credentials
-        $application->clientCredentials = 1;
-        $application->isConfidential = 1;
-
-        // Add the all scope
-        $application->assignScope($this->applicationScopeFactory->getById('all'));
-
-        // Save
-        $application->save();
-
-        // Return
-        $this->getState()->hydrate([
-            'message' => sprintf(__('Added %s'), $application->name),
-            'data' => $application,
-            'id' => $application->key
-        ]);
-
-        return $this->render($request, $response);
-    }
-
     /**
      * Edit Application
      * @param Request $request
@@ -620,6 +601,16 @@ class Applications extends Base
         return $this->render($request, $response);
     }
 
+    /**
+     * @param Request $request
+     * @param Response $response
+     * @param $id
+     * @param $userId
+     * @return \Psr\Http\Message\ResponseInterface|Response
+     * @throws ControllerNotImplemented
+     * @throws GeneralException
+     * @throws InvalidArgumentException
+     */
     public function revokeAccess(Request $request, Response $response, $id, $userId)
     {
         if ($userId === null) {
@@ -632,11 +623,25 @@ class Applications extends Base
 
         $client = $this->applicationFactory->getClientEntity($id);
 
+        if ($this->getUser()->userId != $userId) {
+            throw new InvalidArgumentException(__('Access denied: You do not own this authorization.'));
+        }
+
         // remove record in lk table
         $this->applicationFactory->revokeAuthorised($userId, $client->key);
         // clear cache for this clientId/userId pair, this is how we know the application is no longer approved
         $this->pool->getItem('C_' . $client->key . '/' . $userId)->clear();
 
+        $this->getLog()->audit(
+            'Auth',
+            0,
+            'Application access revoked',
+            [
+                'Application identifier ends with' => substr($client->key, -8),
+                'Application Name' => $client->getName()
+            ]
+        );
+
         $this->getState()->hydrate([
             'httpStatus' => 204,
             'message' => sprintf(__('Access to %s revoked'), $client->name)
