@@ -253,6 +253,7 @@ class Campaign extends Base
             'layoutId' => $parsedParams->getInt('layoutId'),
             'logicalOperator' => $parsedParams->getString('logicalOperator'),
             'logicalOperatorName' => $parsedParams->getString('logicalOperatorName'),
+            'excludeMedia' => $parsedParams->getInt('excludeMedia'),
         ];
 
         $embed = ($parsedParams->getString('embed') !== null) ? explode(',', $parsedParams->getString('embed')) : [];
@@ -527,7 +528,7 @@ class Campaign extends Base
      *  @SWG\Parameter(
      *      name="targetType",
      *      in="formData",
-     *      description="For ad campaigns, how do we measure the target? plays|budget",
+     *      description="For ad campaigns, how do we measure the target? plays|budget|imp",
      *      type="string",
      *      required=false
      *   ),
@@ -774,7 +775,7 @@ class Campaign extends Base
      *  @SWG\Parameter(
      *      name="targetType",
      *      in="formData",
-     *      description="For ad campaigns, how do we measure the target? plays|budget",
+     *      description="For ad campaigns, how do we measure the target? plays|budget|imp",
      *      type="string",
      *      required=false
      *   ),
@@ -862,6 +863,7 @@ class Campaign extends Base
 
         $campaign->campaign = $parsedRequestParams->getString('name');
         $campaign->folderId = $parsedRequestParams->getInt('folderId', ['default' => $campaign->folderId]);
+        $campaign->modifiedBy = $this->getUser()->getId();
 
         if ($campaign->hasPropertyChanged('folderId')) {
             if ($campaign->folderId === 1) {
