@@ -1,6 +1,6 @@
 <?php
 /*
- * Copyright (C) 2023 Xibo Signage Ltd
+ * Copyright (C) 2024 Xibo Signage Ltd
  *
  * Xibo - Digital Signage - https://xibosignage.com
  *
@@ -69,6 +69,41 @@ class Folder extends Base
      *  tags={"folder"},
      *  summary="Search Folders",
      *  description="Returns JSON representation of the Folder tree",
+     *  @SWG\Parameter(
+     *         name="folderId",
+     *         in="path",
+     *         description="Show usage details for the specified Folder Id",
+     *         type="integer",
+     *         required=false
+     *     ),
+     *  @SWG\Parameter(
+     *        name="gridView",
+     *        in="query",
+     *        description="Flag (0, 1), Show Folders in a standard grid response",
+     *        type="integer",
+     *        required=false
+     *     ),
+     *  @SWG\Parameter(
+     *       name="folderId",
+     *       in="query",
+     *       description="Use with gridView, Filter by Folder Id",
+     *       type="integer",
+     *       required=false
+     *    ),
+     *   @SWG\Parameter(
+     *       name="folderName",
+     *       in="query",
+     *       description="Use with gridView, Filter by Folder name",
+     *       type="string",
+     *       required=false
+     *    ),
+     *   @SWG\Parameter(
+     *       name="exactFolderName",
+     *       in="query",
+     *       description="Use with gridView, Filter by exact Folder name match",
+     *       type="integer",
+     *       required=false
+     *    ),
      *  @SWG\Response(
      *      response=200,
      *      description="successful operation",
@@ -86,8 +121,28 @@ class Folder extends Base
     public function grid(Request $request, Response $response, $folderId = null)
     {
         $params = $this->getSanitizer($request->getParams());
-        // Should we return information for a specific folder?
-        if ($folderId !== null) {
+        if ($params->getInt('gridView') === 1) {
+            $folders = $this->folderFactory->query($this->gridRenderSort($params), $this->gridRenderFilter([
+                'folderName' => $params->getString('folderName'),
+                'folderId' => $params->getInt('folderId'),
+                'exactFolderName' => $params->getInt('exactFolderName'),
+            ], $params));
+
+            $this->getState()->template = 'grid';
+            $this->getState()->recordsTotal = $this->folderFactory->countLast();
+            $this->getState()->setData($folders);
+
+            return $this->render($request, $response);
+        } else if ($params->getString('folderName') !== null) {
+            // Search all folders by name
+            $folders = $this->folderFactory->query($this->gridRenderSort($params), $this->gridRenderFilter([
+                'folderName' => $params->getString('folderName'),
+                'exactFolderName' => $params->getInt('exactFolderName'),
+            ], $params));
+
+            return $response->withJson($folders);
+        } else if ($folderId !== null) {
+            // Should we return information for a specific folder?
             $folder = $this->folderFactory->getById($folderId);
 
             $this->decorateWithButtons($folder);
@@ -99,15 +154,14 @@ class Folder extends Base
         } else {
             // Show a tree view of all folders.
             $rootFolder = $this->folderFactory->getById(1);
-            $rootFolder->a_attr['title'] = __('Right click a Folder for further Options');
 
             // homeFolderId,
             // do we show tree for current user
             // or a specified user?
             $homeFolderId = ($params->getInt('homeFolderId') !== null)
-                ? $params->getInt('homeFolderId')
-                : $this->getUser()->homeFolderId;
-
+                    ? $params->getInt('homeFolderId')
+                    : $this->getUser()->homeFolderId;
+            
             $this->buildTreeView($rootFolder, $homeFolderId);
             return $response->withJson([$rootFolder]);
         }
@@ -130,7 +184,11 @@ class Folder extends Base
             $folder->type = 'home';
         }
 
-        $children = array_filter(explode(',', $folder->children));
+        if (!empty($folder->children)) {
+            $children = array_filter(explode(',', $folder->children));
+        } else {
+            $children = [];
+        }
         $childrenDetails = [];
 
         foreach ($children as $childId) {
@@ -143,7 +201,7 @@ class Folder extends Base
 
                 if (!$this->getUser()->checkViewable($child)) {
                     $child->text = __('Private Folder');
-                    $child->li_attr['disabled'] = true;
+                    $child->type = 'disabled';
                 }
 
                 if ($homeFolderId === $child->id) {
@@ -331,11 +389,51 @@ class Folder extends Base
             throw new AccessDeniedException();
         }
 
+        if ($folder->isHome()) {
+            throw new InvalidArgumentException(
+                __('Cannot remove Folder set as home Folder for a user'),
+                'folderId',
+                __('Change home Folder for Users using this Folder before deleting')
+            );
+        }
+
+        if ($folder->id == $this->getConfig()->getSetting('DISPLAY_DEFAULT_FOLDER')) {
+            throw new InvalidArgumentException(
+                __('Cannot remove Folder set as default Folder for new Displays'),
+                'folderId',
+                __('Change Default Folder for new Displays before deleting')
+            );
+        }
+
+        // Check if the folder is in use
+        $this->folderFactory->decorateWithUsage($folder);
+        $usage = $folder->getUnmatchedProperty('usage');
+
+        // Prevent deletion if the folder has any usage
+        if (!empty($usage)) {
+            $usageDetails = [];
+
+            // Loop through usage data and construct the formatted message
+            foreach ($usage as $item) {
+                $usageDetails[] = $item['type'] . ' (' . $item['count'] . ')';
+            }
+
+            throw new InvalidArgumentException(
+                __('Cannot remove Folder with content: ' . implode(', ', $usageDetails)),
+                'folderId',
+                __('Reassign objects from this Folder before deleting.')
+            );
+        }
+
         try {
             $folder->delete();
         } catch (\Exception $exception) {
             $this->getLog()->debug('Folder delete failed with message: ' . $exception->getMessage());
-            throw new InvalidArgumentException(__('Cannot remove Folder with content'), 'folderId', __('Reassign objects from this Folder before deleting.'));
+            throw new InvalidArgumentException(
+                __('Cannot remove Folder with content'),
+                'folderId',
+                __('Reassign objects from this Folder before deleting.')
+            );
         }
 
         // Return
@@ -362,7 +460,7 @@ class Folder extends Base
         return $response->withJson($folder->buttons);
     }
 
-    private function decorateWithButtons($folder)
+    private function decorateWithButtons(\Xibo\Entity\Folder $folder)
     {
         $user = $this->getUser();
 
