@@ -1,8 +1,8 @@
 <?php
-/**
- * Copyright (C) 2021 Xibo Signage Ltd
+/*
+ * Copyright (C) 2023 Xibo Signage Ltd
  *
- * Xibo - Digital Signage - http://www.xibo.org.uk
+ * Xibo - Digital Signage - https://xibosignage.com
  *
  * This file is part of Xibo.
  *
@@ -32,37 +32,20 @@ use Xibo\Support\Exception\GeneralException;
 use Xibo\Support\Exception\InvalidArgumentException;
 use Xibo\Support\Exception\NotFoundException;
 
+/**
+ * Menu Board Controller
+ */
 class MenuBoard extends Base
 {
-    /**
-     * @var MenuBoardFactory
-     */
-    private $menuBoardFactory;
-
-    /**
-     * @var UserFactory
-     */
-    private $userFactory;
-
-    /**
-     * @var FolderFactory
-     */
-    private $folderFactory;
-
     /**
      * Set common dependencies.
      * @param MenuBoardFactory $menuBoardFactory
-     * @param UserFactory $userFactory
      * @param FolderFactory $folderFactory
      */
     public function __construct(
-        $menuBoardFactory,
-        $userFactory,
-        $folderFactory
+        private readonly MenuBoardFactory $menuBoardFactory,
+        private readonly FolderFactory $folderFactory
     ) {
-        $this->menuBoardFactory = $menuBoardFactory;
-        $this->userFactory = $userFactory;
-        $this->folderFactory = $folderFactory;
     }
 
     /**
@@ -77,9 +60,7 @@ class MenuBoard extends Base
     {
         // Call to render the template
         $this->getState()->template = 'menuboard-page';
-        $this->getState()->setData([
-            'users' => $this->userFactory->query()
-        ]);
+
         return $this->render($request, $response);
     }
 
@@ -150,7 +131,8 @@ class MenuBoard extends Base
             'userId' => $parsedParams->getInt('userId'),
             'name' => $parsedParams->getString('name'),
             'code' => $parsedParams->getString('code'),
-            'folderId' => $parsedParams->getInt('folderId')
+            'folderId' => $parsedParams->getInt('folderId'),
+            'logicalOperatorName' => $parsedParams->getString('logicalOperatorName'),
         ];
 
         $menuBoards = $this->menuBoardFactory->query(
@@ -166,7 +148,7 @@ class MenuBoard extends Base
             $menuBoard->includeProperty('buttons');
             $menuBoard->buttons = [];
 
-            if ($this->getUser()->featureEnabled('menuboard.modify') && $this->getUser()->checkEditable($menuBoard)) {
+            if ($this->getUser()->featureEnabled('menuBoard.modify') && $this->getUser()->checkEditable($menuBoard)) {
                 $menuBoard->buttons[] = [
                     'id' => 'menuBoard_button_viewcategories',
                     'url' => $this->urlFor($request, 'menuBoard.category.view', ['id' => $menuBoard->menuId]),
@@ -202,7 +184,7 @@ class MenuBoard extends Base
                 }
             }
 
-            if ($this->getUser()->featureEnabled('menuboard.modify') && $this->getUser()->checkPermissionsModifyable($menuBoard)) {
+            if ($this->getUser()->featureEnabled('menuBoard.modify') && $this->getUser()->checkPermissionsModifyable($menuBoard)) {
                 $menuBoard->buttons[] = ['divider' => true];
 
                 // Share button
@@ -230,7 +212,7 @@ class MenuBoard extends Base
                 ];
             }
 
-            if ($this->getUser()->featureEnabled('menuboard.modify')
+            if ($this->getUser()->featureEnabled('menuBoard.modify')
                 && $this->getUser()->checkDeleteable($menuBoard)
             ) {
                 $menuBoard->buttons[] = ['divider' => true];
@@ -260,10 +242,6 @@ class MenuBoard extends Base
     public function addForm(Request $request, Response $response): Response
     {
         $this->getState()->template = 'menuboard-form-add';
-        $this->getState()->setData([
-            'help' => $this->getHelp()->link('MenuBoard', 'Add')
-        ]);
-
         return $this->render($request, $response);
     }
 
@@ -330,7 +308,19 @@ class MenuBoard extends Base
         $code = $sanitizedParams->getString('code');
         $folderId = $sanitizedParams->getInt('folderId');
 
-        $menuBoard = $this->menuBoardFactory->create($name, $description, $code, $folderId);
+        if ($folderId === 1) {
+            $this->checkRootFolderAllowSave();
+        }
+
+        if (empty($folderId) || !$this->getUser()->featureEnabled('folder.view')) {
+            $folderId = $this->getUser()->homeFolderId;
+        }
+
+        $folder = $this->folderFactory->getById($folderId, 0);
+
+        $menuBoard = $this->menuBoardFactory->create($name, $description, $code);
+        $menuBoard->folderId = $folder->getId();
+        $menuBoard->permissionsFolderId = $folder->getPermissionFolderIdOrThis();
         $menuBoard->save();
 
         // Return
@@ -441,6 +431,9 @@ class MenuBoard extends Base
         $menuBoard->folderId = $sanitizedParams->getInt('folderId', ['default' => $menuBoard->folderId]);
 
         if ($menuBoard->hasPropertyChanged('folderId')) {
+            if ($menuBoard->folderId === 1) {
+                $this->checkRootFolderAllowSave();
+            }
             $folder = $this->folderFactory->getById($menuBoard->folderId);
             $menuBoard->permissionsFolderId = ($folder->getPermissionFolderId() == null) ? $folder->id : $folder->getPermissionFolderId();
         }
@@ -614,6 +607,10 @@ class MenuBoard extends Base
 
         $folderId = $this->getSanitizer($request->getParams())->getInt('folderId');
 
+        if ($folderId === 1) {
+            $this->checkRootFolderAllowSave();
+        }
+
         $menuBoard->folderId = $folderId;
         $folder = $this->folderFactory->getById($menuBoard->folderId);
         $menuBoard->permissionsFolderId = ($folder->getPermissionFolderId() == null) ? $folder->id : $folder->getPermissionFolderId();
