@@ -1,6 +1,6 @@
 <?php
 /*
- * Copyright (C) 2023 Xibo Signage Ltd
+ * Copyright (C) 2024 Xibo Signage Ltd
  *
  * Xibo - Digital Signage - https://xibosignage.com
  *
@@ -22,6 +22,7 @@
 
 namespace Xibo\Controller;
 
+use Carbon\Carbon;
 use Slim\Http\Response as Response;
 use Slim\Http\ServerRequest as Request;
 use Xibo\Factory\DisplayFactory;
@@ -36,10 +37,12 @@ use Xibo\Factory\TagFactory;
 use Xibo\Factory\UserFactory;
 use Xibo\Factory\UserGroupFactory;
 use Xibo\Factory\WidgetFactory;
+use Xibo\Helper\DateFormatHelper;
 use Xibo\Support\Exception\AccessDeniedException;
 use Xibo\Support\Exception\GeneralException;
 use Xibo\Support\Exception\InvalidArgumentException;
 use Xibo\Support\Exception\NotFoundException;
+use Xibo\Widget\SubPlaylistItem;
 
 /**
  * Class Playlist
@@ -323,6 +326,9 @@ class Playlist extends Base
                         // Augment with deletable flag
                         $widget->setUnmatchedProperty('isDeletable', $this->getUser()->checkDeleteable($widget));
 
+                        // Augment with viewable flag
+                        $widget->setUnmatchedProperty('isViewable', $this->getUser()->checkViewable($widget));
+
                         // Augment with permissions flag
                         $widget->setUnmatchedProperty(
                             'isPermissionsModifiable',
@@ -697,16 +703,17 @@ class Playlist extends Base
             $playlist->updateTagLinks($tags);
         }
 
-        // Do we have a tag or name filter?
+        // Do we have a tag, name or folder filter?
         $nameFilter = $sanitizedParams->getString('filterMediaName');
         $nameFilterLogicalOperator = $sanitizedParams->getString('logicalOperatorName');
         $tagFilter = $this->getUser()->featureEnabled('tag.tagging') ? $sanitizedParams->getString('filterMediaTag') : null;
         $logicalOperator = $this->getUser()->featureEnabled('tag.tagging') ? $sanitizedParams->getString('logicalOperator') : 'OR';
         $exactTags = $this->getUser()->featureEnabled('tag.tagging') ? $sanitizedParams->getCheckbox('exactTags') : 0;
+        $folderIdFilter = $this->getUser()->featureEnabled('folder.view') ? $sanitizedParams->getInt('filterFolderId') : null;
 
         // Capture these as dynamic filter criteria
         if ($playlist->isDynamic === 1) {
-            if (empty($nameFilter) && empty($tagFilter)) {
+            if (empty($nameFilter) && empty($tagFilter) && empty($folderIdFilter)) {
                 throw new InvalidArgumentException(__('No filters have been set for this dynamic Playlist, please click the Filters tab to define'));
             }
             $playlist->filterMediaName = $nameFilter;
@@ -716,18 +723,24 @@ class Playlist extends Base
                 $playlist->filterExactTags = $exactTags;
                 $playlist->filterMediaTagsLogicalOperator = $logicalOperator;
             }
+
+            if ($this->getUser()->featureEnabled('folder.view')) {
+                $playlist->filterFolderId = $folderIdFilter;
+            }
+
             $playlist->maxNumberOfItems = $sanitizedParams->getInt('maxNumberOfItems', ['default' => $this->getConfig()->getSetting('DEFAULT_DYNAMIC_PLAYLIST_MAXNUMBER')]);
         }
 
         $playlist->save();
 
         // Should we assign any existing media
-        if (!empty($nameFilter) || !empty($tagFilter)) {
+        if (!empty($nameFilter) || !empty($tagFilter) || !empty($folderIdFilter)) {
             $media = $this->mediaFactory->query(
                 null,
                 [
                     'name' => $nameFilter,
                     'tags' => $tagFilter,
+                    'folderId' => $folderIdFilter,
                     'assignable' => 1,
                     'exactTags' => $exactTags,
                     'logicalOperator' => $logicalOperator,
@@ -954,17 +967,26 @@ class Playlist extends Base
         // Do we have a tag or name filter?
         // Capture these as dynamic filter criteria
         if ($playlist->isDynamic === 1) {
-            if (empty($sanitizedParams->getString('filterMediaName')) && empty($sanitizedParams->getString('filterMediaTag'))) {
+            $filterMediaName = $sanitizedParams->getString('filterMediaName');
+            $filterMediaTag = $sanitizedParams->getString('filterMediaTag');
+            $filterFolderId = $sanitizedParams->getString('filterFolderId');
+
+            if (empty($filterMediaName) && empty($filterMediaTag) && empty($filterFolderId)) {
                 throw new InvalidArgumentException(__('No filters have been set for this dynamic Playlist, please click the Filters tab to define'));
             }
-            $playlist->filterMediaName = $sanitizedParams->getString('filterMediaName');
+            $playlist->filterMediaName = $filterMediaName;
             $playlist->filterMediaNameLogicalOperator = $sanitizedParams->getString('logicalOperatorName');
 
             if ($this->getUser()->featureEnabled('tag.tagging')) {
-                $playlist->filterMediaTags = $sanitizedParams->getString('filterMediaTag');
+                $playlist->filterMediaTags = $filterMediaTag;
                 $playlist->filterExactTags = $sanitizedParams->getCheckbox('exactTags');
                 $playlist->filterMediaTagsLogicalOperator = $sanitizedParams->getString('logicalOperator');
             }
+
+            if ($this->getUser()->featureEnabled('folder.view')) {
+                $playlist->filterFolderId = $filterFolderId;
+            }
+
             $playlist->maxNumberOfItems = $sanitizedParams->getInt('maxNumberOfItems');
         }
 
@@ -1225,10 +1247,17 @@ class Playlist extends Base
             throw new AccessDeniedException();
         }
 
+        // Get a list of timezones
+        $timeZones = [];
+        foreach (DateFormatHelper::timezoneList() as $key => $value) {
+            $timeZones[] = ['id' => $key, 'value' => $value];
+        }
+
         // Pass to view
         $this->getState()->template = 'playlist-form-timeline';
         $this->getState()->setData([
             'playlist' => $playlist,
+            'timeZones' => $timeZones,
         ]);
 
         return $this->render($request, $response);
@@ -1313,8 +1342,9 @@ class Playlist extends Base
         // Expect a list of mediaIds
         $media = $sanitizedParams->getIntArray('media');
 
-        if (count($media) <= 0)
+        if (empty($media)) {
             throw new InvalidArgumentException(__('Please provide Media to Assign'), 'media');
+        }
 
         // Optional Duration
         $duration = ($sanitizedParams->getInt('duration'));
@@ -1359,11 +1389,11 @@ class Playlist extends Base
             if ($duration !== null || $sanitizedParams->getCheckbox('useDuration') == 1) {
                 $widget->useDuration = 1;
                 $widget->duration = $itemDuration;
+                $widget->calculateDuration($module);
+            } else {
+                $widget->calculatedDuration = $itemDuration;
             }
 
-            // Calculate the duration
-            $widget->calculateDuration($module);
-
             // Assign the widget to the playlist
             $playlist->assignWidget($widget, $displayOrder);
 
@@ -1705,7 +1735,8 @@ class Playlist extends Base
                     'id' => 'layout_button_preview',
                     'external' => true,
                     'url' => '#',
-                    'onclick' => 'createMiniLayoutPreview("' . $this->urlFor($request, 'layout.preview', ['id' => $layout->layoutId]) . '");',
+                    'onclick' => 'createMiniLayoutPreview',
+                    'onclickParam' => $this->urlFor($request, 'layout.preview', ['id' => $layout->layoutId]),
                     'text' => __('Preview Layout')
                 );
             }
@@ -1929,4 +1960,154 @@ class Playlist extends Base
     {
         return $this->layoutFactory->getLinkedFullScreenLayout('playlist', $playlist->playlistId)?->campaignId;
     }
+
+    /**
+     * Convert Layout editor playlist to global playlist.
+     * Assign this Playlist to the original regionPlaylist via sub-playlist Widget.
+     * @SWG\Post(
+     *   path="/playlist/{id}/convert",
+     *   operationId="convert",
+     *   tags={"playlist"},
+     *   summary="Playlist Convert",
+     *   description="Create a global playlist from inline editor Playlist.
+     * Assign created Playlist via sub-playlist Widget to region Playlist.",
+     *   @SWG\Parameter(
+     *       name="playlistId",
+     *       in="path",
+     *       description="The Playlist ID",
+     *       type="integer",
+     *       required=true
+     *    ),
+     *   @SWG\Parameter(
+     *       name="name",
+     *       in="formData",
+     *       description="Optional name for the global Playlist.",
+     *       type="string",
+     *       required=false
+     *    ),
+     *   @SWG\Response(
+     *       response=201,
+     *       description="successful operation"
+     *   )
+     *  )
+     * @param Request $request
+     * @param Response $response
+     * @param $id
+     * @return Response
+     * @throws AccessDeniedException
+     * @throws GeneralException
+     * @throws InvalidArgumentException
+     * @throws NotFoundException
+     */
+    public function convert(Request $request, Response $response, $id): Response
+    {
+        $params = $this->getSanitizer($request->getParams());
+
+        // get region playlist
+        $regionPlaylist = $this->playlistFactory->getById($id);
+
+        // check if it is region playlist
+        if (!$regionPlaylist->isRegionPlaylist()) {
+            throw new InvalidArgumentException(__('Not a Region Playlist'), 'playlistId');
+        }
+
+        // get the region
+        $region = $this->regionFactory->getById($regionPlaylist->regionId);
+
+        // make sure this is playlist type region
+        if ($region->type !== 'playlist') {
+            throw new InvalidArgumentException(__('Not a Playlist'), 'playlistId');
+        }
+
+        // get Layout
+        $layout = $this->layoutFactory->getByRegionId($regionPlaylist->regionId);
+
+        // check permissions
+        if (!$this->getUser()->checkEditable($layout)) {
+            throw new AccessDeniedException();
+        }
+
+        // check if it is a draft
+        if (!$layout->isEditable()) {
+            throw new InvalidArgumentException(
+                __('This Layout is not a Draft, please checkout.'),
+                'layoutId'
+            );
+        }
+
+        $regionPlaylist->load();
+
+        // clone region playlist to a new Playlist object
+        $playlist = clone $regionPlaylist;
+        $name = $params->getString(
+            'name',
+            ['default' => sprintf(__('Untitled %s'), Carbon::now()->format(DateFormatHelper::getSystemFormat()))]
+        );
+
+        $playlist->name = empty($playlist->name) ? $name : $playlist->name;
+        $playlist->setOwner($this->getUser()->userId);
+
+        if ($playlist->enableStat == null) {
+            $playlist->enableStat = $this->getConfig()->getSetting('PLAYLIST_STATS_ENABLED_DEFAULT');
+        }
+
+        // Save the new playlist
+        $playlist->save();
+        $playlist->updateDuration();
+
+        // Clone the closure table for the original playlist
+        $regionPlaylist->cloneClosureTable($playlist->getId());
+
+        // remove widgets on the region Playlist
+        foreach ($regionPlaylist->widgets as $widget) {
+            $widget->delete();
+        }
+        $regionPlaylist->widgets = [];
+
+        $module = $this->moduleFactory->getByType('subplaylist');
+
+        // create a new sub-playlist Widget
+        $widget = $this->widgetFactory->create(
+            $this->getUser()->userId,
+            $regionPlaylist->playlistId,
+            'subplaylist',
+            $playlist->duration,
+            $module->schemaVersion
+        );
+
+        // save, simulate add
+        $widget->save();
+
+        // prepare sub-playlist item
+        $item = new SubPlaylistItem();
+        $item->rowNo = 1;
+        $item->playlistId = $playlist->playlistId;
+        $item->spotFill = 'repeat';
+        $item->spotLength =  '';
+        $item->spots = '';
+
+        $playlistItems[] = $item;
+
+        // update Widget subPlaylists option
+        $widget->setOptionValue('subPlaylists', 'attrib', json_encode($playlistItems));
+
+        // Calculate the duration
+        $widget->calculateDuration($module);
+
+        // Assign the sub-playlist widget to the region playlist
+        $regionPlaylist->assignWidget($widget);
+        // Save the region playlist
+        $regionPlaylist->save();
+
+        // build Layout xlf
+        $layout->xlfToDisk(['notify' => true, 'exceptionOnError' => true, 'exceptionOnEmptyRegion' => false]);
+
+        // Success
+        $this->getState()->hydrate([
+            'httpStatus' => 201,
+            'message' => __('Conversion Successful'),
+        ]);
+
+        return $this->render($request, $response);
+    }
 }
