@@ -1,8 +1,8 @@
 <?php
 /*
- * Copyright (c) 2022 Xibo Signage Ltd
+ * Copyright (C) 2023 Xibo Signage Ltd
  *
- * Xibo - Digital Signage - http://www.xibo.org.uk
+ * Xibo - Digital Signage - https://xibosignage.com
  *
  * This file is part of Xibo.
  *
@@ -25,6 +25,10 @@ namespace Xibo\Controller;
 use Slim\Http\Response as Response;
 use Slim\Http\ServerRequest as Request;
 use Xibo\Event\DisplayGroupLoadEvent;
+use Xibo\Event\TagAddEvent;
+use Xibo\Event\TagDeleteEvent;
+use Xibo\Event\TagEditEvent;
+use Xibo\Event\TriggerTaskEvent;
 use Xibo\Factory\CampaignFactory;
 use Xibo\Factory\DisplayFactory;
 use Xibo\Factory\DisplayGroupFactory;
@@ -202,7 +206,8 @@ class Tag extends Base
             'isSystem' => $sanitizedQueryParams->getCheckbox('isSystem'),
             'isRequired' => $sanitizedQueryParams->getCheckbox('isRequired'),
             'haveOptions' => $sanitizedQueryParams->getCheckbox('haveOptions'),
-            'allTags' => $sanitizedQueryParams->getInt('allTags')
+            'allTags' => $sanitizedQueryParams->getInt('allTags'),
+            'logicalOperatorName' => $sanitizedQueryParams->getString('logicalOperatorName'),
         ];
 
         $tags = $this->tagFactory->query($this->gridRenderSort($sanitizedQueryParams), $this->gridRenderFilter($filter, $sanitizedQueryParams));
@@ -211,11 +216,6 @@ class Tag extends Base
             /* @var \Xibo\Entity\Tag $tag */
 
             if ($this->isApi($request)) {
-                $tag->excludeProperty('layouts');
-                $tag->excludeProperty('playlists');
-                $tag->excludeProperty('campaigns');
-                $tag->excludeProperty('medias');
-                $tag->excludeProperty('displayGroups');
                 continue;
             }
 
@@ -248,6 +248,12 @@ class Tag extends Base
                     ]
                 ];
             }
+
+            $tag->buttons[] = [
+                'id' => 'tag_button_usage',
+                'url' => $this->urlFor($request, 'tag.usage.form', ['id' => $tag->tagId]),
+                'text' => __('Usage')
+            ];
         }
 
         $this->getState()->template = 'grid';
@@ -268,10 +274,6 @@ class Tag extends Base
     public function addForm(Request $request, Response $response)
     {
         $this->getState()->template = 'tag-form-add';
-        $this->getState()->setData([
-            'help' => $this->getHelp()->link('Tags', 'Add')
-        ]);
-
         return $this->render($request, $response);
     }
 
@@ -349,6 +351,10 @@ class Tag extends Base
 
         $tag->save();
 
+        // dispatch Tag add event
+        $event = new TagAddEvent($tag->tagId);
+        $this->getDispatcher()->dispatch($event, $event::$NAME);
+
         // Return
         $this->getState()->hydrate([
             'httpStatus' => 201,
@@ -422,12 +428,44 @@ class Tag extends Base
         $this->getState()->setData([
             'tag' => $tag,
             'options' => $tagOptions,
-            'help' => $this->getHelp()->link('Tags', 'Add')
         ]);
 
         return $this->render($request, $response);
     }
 
+    public function usageForm(Request $request, Response $response, $id)
+    {
+        $tag = $this->tagFactory->getById($id);
+
+        $this->getState()->template = 'tag-usage-form';
+        $this->getState()->setData([
+            'tag' => $tag
+        ]);
+
+        return $this->render($request, $response);
+    }
+
+    public function usage(Request $request, Response $response, $id)
+    {
+        $sanitizedParams = $this->getSanitizer($request->getParams());
+        $sanitizedQueryParams = $this->getSanitizer($request->getQueryParams());
+
+        $filter = [
+            'tagId' => $id,
+        ];
+
+        $entries = $this->tagFactory->getAllLinks(
+            $this->gridRenderSort($sanitizedParams),
+            $this->gridRenderFilter($filter, $sanitizedQueryParams)
+        );
+
+        $this->getState()->template = 'grid';
+        $this->getState()->recordsTotal = $this->tagFactory->countLast();
+        $this->getState()->setData($entries);
+
+        return $this->render($request, $response);
+    }
+
     /**
      * Edit a Tag
      *
@@ -450,7 +488,6 @@ class Tag extends Base
 
         $sanitizedParams = $this->getSanitizer($request->getParams());
         $tag = $this->tagFactory->getById($id);
-        $tag->load();
 
         if ($tag->isSystem === 1) {
             throw new AccessDeniedException(__('Access denied System tags cannot be edited'));
@@ -463,6 +500,7 @@ class Tag extends Base
 
         $values = [];
 
+        $oldTag = $tag->tag;
         $tag->tag = $sanitizedParams->getString('name');
         $tag->isRequired = $sanitizedParams->getCheckbox('isRequired');
         $optionValues = $sanitizedParams->getString('options');
@@ -499,6 +537,10 @@ class Tag extends Base
 
         $tag->save();
 
+        // dispatch Tag edit event
+        $event = new TagEditEvent($tag->tagId, $oldTag, $tag->tag);
+        $this->getDispatcher()->dispatch($event, $event::$NAME);
+
         // Return
         $this->getState()->hydrate([
             'httpStatus' => 201,
@@ -527,7 +569,6 @@ class Tag extends Base
         $this->getState()->template = 'tag-form-delete';
         $this->getState()->setData([
             'tag' => $tag,
-            'help' => $this->getHelp()->link('Tag', 'Delete')
         ]);
 
         return $this->render($request, $response);
@@ -574,55 +615,15 @@ class Tag extends Base
         }
 
         $tag = $this->tagFactory->getById($id);
-        $tag->load();
 
         if ($tag->isSystem === 1) {
             throw new AccessDeniedException(__('Access denied System tags cannot be deleted'));
         }
 
-        // get all the linked items to the tag we want to delete
-        $linkedLayoutsIds = $tag->layouts;
-        $linkedDisplayGroupsIds = $tag->displayGroups;
-        $linkedCampaignsIds = $tag->campaigns;
-        $linkedPlaylistsIds = $tag->playlists;
-        $linkedMediaIds = $tag->medias;
-
-        // go through each linked layout and unassign the tag
-        foreach($linkedLayoutsIds as $layoutId => $value) {
-            $layout = $this->layoutFactory->getById($layoutId);
-            $tag->unassignLayout($layoutId);
-            $layout->save();
-        }
-
-        // go through each linked displayGroup and unassign the tag
-        foreach ($linkedDisplayGroupsIds as $displayGroupId => $value) {
-            $displayGroup = $this->displayGroupFactory->getById($displayGroupId);
-            $tag->unassignDisplayGroup($displayGroupId);
-            $displayGroup->save();
-        }
-
-        // go through each linked campaign and unassign the tag
-        foreach ($linkedCampaignsIds as $campaignId => $value) {
-            $campaign = $this->campaignFactory->getById($campaignId);
-            $tag->unassignCampaign($campaignId);
-            $campaign->save();
-        }
-
-        // go through each linked playlist and unassign the tag
-        foreach ($linkedPlaylistsIds as $playlistId => $value) {
-            $playlist = $this->playlistFactory->getById($playlistId);
-            $tag->unassignPlaylist($playlistId);
-            $playlist->save();
-        }
-
-        // go through each linked media and unassign the tag
-        foreach($linkedMediaIds as $mediaId => $value) {
-            $media = $this->mediaFactory->getById($mediaId);
-            $tag->unassignMedia($mediaId);
-            $media->save();
-        }
-
-        // finally call delete tag, which also removes the links from lktag tables
+        // Dispatch delete event, remove this tag links in all lktag tables.
+        $event = new TagDeleteEvent($tag->tagId);
+        $this->getDispatcher()->dispatch($event, $event::$NAME);
+        // tag delete, remove the record from tag table
         $tag->delete();
 
         // Return
@@ -739,23 +740,16 @@ class Tag extends Base
                     $entity->assignTag($tag);
                 }
 
-                $entity->save();
+                $entity->save(['isTagEdit' => true]);
             }
 
             // Once we're done, and if we're a Display entity, we need to calculate the dynamic display groups
             if ($targetType === 'display') {
-                foreach ($this->displayGroupFactory->getByIsDynamic(1) as $group) {
-                    $this->getDispatcher()->dispatch(new DisplayGroupLoadEvent($group), DisplayGroupLoadEvent::$NAME);
-                    $group->save([
-                        'validate' => false,
-                        'saveGroup' => false,
-                        'saveTags' => false,
-                        'manageLinks' => false,
-                        'manageDisplayLinks' => true,
-                        'manageDynamicDisplayLinks' => true,
-                        'allowNotify' => true
-                    ]);
-                }
+                // Background update.
+                $this->getDispatcher()->dispatch(
+                    new TriggerTaskEvent('\Xibo\XTR\MaintenanceRegularTask', 'DYNAMIC_DISPLAY_GROUP_ASSESSED'),
+                    TriggerTaskEvent::$NAME
+                );
             }
         } else {
             $this->getLog()->debug('Tags were not changed');
@@ -769,4 +763,4 @@ class Tag extends Base
 
         return $this->render($request, $response);
     }
-}
\ No newline at end of file
+}
