@@ -1,8 +1,8 @@
 <?php
 /*
- * Copyright (c) 2022 Xibo Signage Ltd
+ * Copyright (C) 2025 Xibo Signage Ltd
  *
- * Xibo - Digital Signage - http://www.xibo.org.uk
+ * Xibo - Digital Signage - https://xibosignage.com
  *
  * This file is part of Xibo.
  *
@@ -22,11 +22,13 @@
 namespace Xibo\Controller;
 
 use Parsedown;
+use Psr\Http\Message\ResponseInterface;
 use Slim\Http\Response as Response;
 use Slim\Http\ServerRequest as Request;
 use Xibo\Entity\SearchResult;
 use Xibo\Entity\SearchResults;
 use Xibo\Event\TemplateProviderEvent;
+use Xibo\Event\TemplateProviderListEvent;
 use Xibo\Factory\LayoutFactory;
 use Xibo\Factory\TagFactory;
 use Xibo\Support\Exception\AccessDeniedException;
@@ -113,7 +115,9 @@ class Template extends Base
     {
         $sanitizedQueryParams = $this->getSanitizer($request->getQueryParams());
         // Embed?
-        $embed = ($sanitizedQueryParams->getString('embed') != null) ? explode(',', $sanitizedQueryParams->getString('embed')) : [];
+        $embed = ($sanitizedQueryParams->getString('embed') != null)
+            ? explode(',', $sanitizedQueryParams->getString('embed'))
+            : [];
 
         $templates = $this->layoutFactory->query($this->gridRenderSort($sanitizedQueryParams), $this->gridRenderFilter([
             'excludeTemplates' => 0,
@@ -146,24 +150,34 @@ class Template extends Base
             $template->includeProperty('buttons');
 
             // Thumbnail
-            $template->thumbnail = '';
+            $template->setUnmatchedProperty('thumbnail', '');
             if (file_exists($template->getThumbnailUri())) {
-                $template->thumbnail = $this->urlFor($request, 'layout.download.thumbnail', ['id' => $template->layoutId]);
+                $template->setUnmatchedProperty(
+                    'thumbnail',
+                    $this->urlFor($request, 'layout.download.thumbnail', ['id' => $template->layoutId])
+                );
             }
 
             // Parse down for description
-            $template->descriptionWithMarkup = Parsedown::instance()->text($template->description);
+            $template->setUnmatchedProperty(
+                'descriptionWithMarkup',
+                Parsedown::instance()->setSafeMode(true)->text($template->description),
+            );
 
             if ($this->getUser()->featureEnabled('template.modify')
                 && $this->getUser()->checkEditable($template)
             ) {
                 // Design Button
-                $template->buttons[] = array(
+                $template->buttons[] = [
                     'id' => 'layout_button_design',
                     'linkType' => '_self', 'external' => true,
-                    'url' => $this->urlFor($request, 'layout.designer', array('id' => $template->layoutId)),
+                    'url' => $this->urlFor(
+                        $request,
+                        'layout.designer',
+                        ['id' => $template->layoutId]
+                    ) . '?isTemplateEditor=1',
                     'text' => __('Alter Template')
-                );
+                ];
 
                 if ($template->isEditable()) {
                     $template->buttons[] = ['divider' => true];
@@ -191,7 +205,14 @@ class Template extends Base
                         'text' => __('Checkout'),
                         'dataAttributes' => [
                             ['name' => 'auto-submit', 'value' => true],
-                            ['name' => 'commit-url', 'value' => $this->urlFor($request, 'layout.checkout', ['id' => $template->layoutId])],
+                            [
+                                'name' => 'commit-url',
+                                'value' => $this->urlFor(
+                                    $request,
+                                    'layout.checkout',
+                                    ['id' => $template->layoutId]
+                                )
+                            ],
                             ['name' => 'commit-method', 'value' => 'PUT']
                         ]
                     );
@@ -214,7 +235,14 @@ class Template extends Base
                         'text' => __('Select Folder'),
                         'multi-select' => true,
                         'dataAttributes' => [
-                            ['name' => 'commit-url', 'value' => $this->urlFor($request, 'campaign.selectfolder', ['id' => $template->campaignId])],
+                            [
+                                'name' => 'commit-url',
+                                'value' => $this->urlFor(
+                                    $request,
+                                    'campaign.selectfolder',
+                                    ['id' => $template->campaignId]
+                                )
+                            ],
                             ['name' => 'commit-method', 'value' => 'put'],
                             ['name' => 'id', 'value' => 'campaign_button_selectfolder'],
                             ['name' => 'text', 'value' => __('Move to Folder')],
@@ -242,7 +270,14 @@ class Template extends Base
                     'text' => __('Delete'),
                     'multi-select' => true,
                     'dataAttributes' => [
-                        ['name' => 'commit-url', 'value' => $this->urlFor($request, 'layout.delete', ['id' => $template->layoutId])],
+                        [
+                            'name' => 'commit-url',
+                            'value' => $this->urlFor(
+                                $request,
+                                'layout.delete',
+                                ['id' => $template->layoutId]
+                            )
+                        ],
                         ['name' => 'commit-method', 'value' => 'delete'],
                         ['name' => 'id', 'value' => 'layout_button_delete'],
                         ['name' => 'text', 'value' => __('Delete')],
@@ -260,18 +295,36 @@ class Template extends Base
                 // Permissions button
                 $template->buttons[] = [
                     'id' => 'layout_button_permissions',
-                    'url' => $this->urlFor($request, 'user.permissions.form', ['entity' => 'Campaign', 'id' => $template->campaignId]) . '?nameOverride=' . __('Template'),
+                    'url' => $this->urlFor(
+                        $request,
+                        'user.permissions.form',
+                        ['entity' => 'Campaign', 'id' => $template->campaignId]
+                    ) . '?nameOverride=' . __('Template'),
                     'text' => __('Share'),
                     'multi-select' => true,
                     'dataAttributes' => [
-                        ['name' => 'commit-url', 'value' => $this->urlFor($request, 'user.permissions.multi', ['entity' => 'Campaign', 'id' => $template->campaignId])],
+                        [
+                            'name' => 'commit-url',
+                            'value' => $this->urlFor(
+                                $request,
+                                'user.permissions.multi',
+                                ['entity' => 'Campaign', 'id' => $template->campaignId]
+                            )
+                        ],
                         ['name' => 'commit-method', 'value' => 'post'],
                         ['name' => 'id', 'value' => 'layout_button_permissions'],
                         ['name' => 'text', 'value' => __('Share')],
                         ['name' => 'rowtitle', 'value' => $template->layout],
                         ['name' => 'sort-group', 'value' => 2],
                         ['name' => 'custom-handler', 'value' => 'XiboMultiSelectPermissionsFormOpen'],
-                        ['name' => 'custom-handler-url', 'value' => $this->urlFor($request, 'user.permissions.multi.form', ['entity' => 'Campaign'])],
+                        [
+                            'name' => 'custom-handler-url',
+                            'value' => $this->urlFor(
+                                $request,
+                                'user.permissions.multi.form',
+                                ['entity' => 'Campaign']
+                            )
+                        ],
                         ['name' => 'content-id-name', 'value' => 'campaignId']
                     ]
                 ];
@@ -332,6 +385,7 @@ class Template extends Base
                 'excludeTemplates' => 0,
                 'layout' => $sanitizedQueryParams->getString('template'),
                 'folderId' => $sanitizedQueryParams->getInt('folderId'),
+                'orientation' => $sanitizedQueryParams->getString('orientation', ['defaultOnEmptyString' => true]),
                 'publishedStatusId' => 1
             ], $sanitizedQueryParams));
 
@@ -344,7 +398,7 @@ class Template extends Base
                 // Handle the description
                 $searchResult->description = '';
                 if (!empty($template->description)) {
-                    $searchResult->description = Parsedown::instance()->line($template->description);
+                    $searchResult->description = Parsedown::instance()->setSafeMode(true)->line($template->description);
                 }
                 $searchResult->orientation = $template->orientation;
                 $searchResult->width = $template->width;
@@ -379,7 +433,8 @@ class Template extends Base
                 $searchResults,
                 $sanitizedQueryParams->getInt('start', ['default' => 0]),
                 $sanitizedQueryParams->getInt('length', ['default' => 15]),
-                $sanitizedQueryParams->getString('template')
+                $sanitizedQueryParams->getString('template'),
+                $sanitizedQueryParams->getString('orientation'),
             );
 
             $this->getLog()->debug('Dispatching event. ' . $event->getName());
@@ -417,7 +472,6 @@ class Template extends Base
         $this->getState()->template = 'template-form-add-from-layout';
         $this->getState()->setData([
             'layout' => $layout,
-            'help' => $this->getHelp()->link('Template', 'Add')
         ]);
 
         return $this->render($request, $response);
@@ -503,7 +557,7 @@ class Template extends Base
         } else {
             $tags = [];
         }
-        $tags[] = $this->tagFactory->getByTag('template');
+        $tags[] = $this->tagFactory->tagFromString('template');
 
         $layout = $this->layoutFactory->createFromResolution($resolutionId,
             $this->getUser()->userId,
@@ -602,7 +656,7 @@ class Template extends Base
      *  )
      * )
      */
-    function addFromLayout(Request $request, Response $response, $id)
+    public function addFromLayout(Request $request, Response $response, $id): Response
     {
         // Get the layout
         $layout = $this->layoutFactory->getById($id);
@@ -631,11 +685,11 @@ class Template extends Base
 
         $layout->layout = $sanitizedParams->getString('name');
         if ($this->getUser()->featureEnabled('tag.tagging')) {
-            $layout->tags = $this->tagFactory->tagsFromString($sanitizedParams->getString('tags'));
+            $layout->updateTagLinks($this->tagFactory->tagsFromString($sanitizedParams->getString('tags')));
         } else {
             $layout->tags = [];
         }
-        $layout->tags[] = $this->tagFactory->getByTag('template');
+        $layout->assignTag($this->tagFactory->tagFromString('template'));
 
         $layout->description = $sanitizedParams->getString('description');
         $layout->folderId = $sanitizedParams->getInt('folderId');
@@ -644,6 +698,23 @@ class Template extends Base
             $this->checkRootFolderAllowSave();
         }
 
+        // When saving a layout as a template, we should not include the empty canva region as that requires
+        // a widget to be inside it.
+        // https://github.com/xibosignage/xibo/issues/3574
+        if (!$includeWidgets) {
+            $this->getLog()->debug('addFromLayout: widgets have not been included, checking for empty regions');
+
+            $regionsWithWidgets = [];
+            foreach ($layout->regions as $region) {
+                if ($region->type === 'canvas') {
+                    $this->getLog()->debug('addFromLayout: Canvas region excluded from export');
+                } else {
+                    $regionsWithWidgets[] = $region;
+                }
+            }
+            $layout->regions = $regionsWithWidgets;
+        }
+
         $layout->setOwner($this->getUser()->userId, true);
         $layout->save();
 
@@ -682,7 +753,6 @@ class Template extends Base
         $this->getState()->template = 'template-form-add';
         $this->getState()->setData([
             'resolutions' => $this->resolutionFactory->query(['resolution']),
-            'help' => $this->getHelp()->link('Layout', 'Add')
         ]);
 
         return $this->render($request, $response);
@@ -712,9 +782,25 @@ class Template extends Base
         $this->getState()->template = 'template-form-edit';
         $this->getState()->setData([
             'layout' => $template,
-            'help' => $this->getHelp()->link('Template', 'Edit')
         ]);
 
         return $this->render($request, $response);
     }
+
+    /**
+     * Get list of Template providers with their details.
+     *
+     * @param Request $request
+     * @param Response $response
+     * @return Response|ResponseInterface
+     */
+    public function providersList(Request $request, Response $response): Response|\Psr\Http\Message\ResponseInterface
+    {
+        $event = new TemplateProviderListEvent();
+        $this->getDispatcher()->dispatch($event, $event->getName());
+
+        $providers = $event->getProviders();
+
+        return $response->withJson($providers);
+    }
 }
