@@ -1,6 +1,6 @@
 <?php
 /*
- * Copyright (C) 2023 Xibo Signage Ltd
+ * Copyright (C) 2025 Xibo Signage Ltd
  *
  * Xibo - Digital Signage - https://xibosignage.com
  *
@@ -26,6 +26,7 @@ use Slim\Http\Response as Response;
 use Slim\Http\ServerRequest as Request;
 use Xibo\Entity\Permission;
 use Xibo\Event\LayoutOwnerChangeEvent;
+use Xibo\Event\LayoutSharingChangeEvent;
 use Xibo\Event\ParsePermissionEntityEvent;
 use Xibo\Event\UserDeleteEvent;
 use Xibo\Factory\ApplicationFactory;
@@ -279,6 +280,8 @@ class User extends Base
             'userId' => $sanitizedParams->getInt('userId'),
             'userTypeId' => $sanitizedParams->getInt('userTypeId'),
             'userName' => $sanitizedParams->getString('userName'),
+            'firstName' => $sanitizedParams->getString('firstName'),
+            'lastName' => $sanitizedParams->getString('lastName'),
             'useRegexForName' => $sanitizedParams->getCheckbox('useRegexForName'),
             'retired' => $sanitizedParams->getInt('retired'),
             'logicalOperatorName' => $sanitizedParams->getString('logicalOperatorName'),
@@ -433,7 +436,8 @@ class User extends Base
      *      name="homePageId",
      *      in="formData",
      *      description="The homepage to use for this User",
-     *      type="integer",
+     *      type="string",
+     *      enum={"statusdashboard.view", "icondashboard.view", "mediamanager.view", "playlistdashboard.view"},
      *      required=true
      *   ),
      *  @SWG\Parameter(
@@ -575,7 +579,7 @@ class User extends Base
 
         // Are user home folders enabled? If not, use the default.
         if ($this->getUser()->featureEnabled('folder.userHome')) {
-            $user->homeFolderId = $sanitizedParams->getInt('homeFolderId');
+            $user->homeFolderId = $sanitizedParams->getInt('homeFolderId', ['default' => 1]);
         } else {
             $user->homeFolderId = 1;
         }
@@ -678,7 +682,8 @@ class User extends Base
      *      name="homePageId",
      *      in="formData",
      *      description="The homepage to use for this User",
-     *      type="integer",
+     *      type="string",
+     *      enum={"statusdashboard.view", "icondashboard.view", "mediamanager.view", "playlistdashboard.view"},
      *      required=true
      *   ),
      *  @SWG\Parameter(
@@ -830,17 +835,30 @@ class User extends Base
 
         // Are user home folders enabled? Don't change unless they are.
         if ($this->getUser()->featureEnabled('folder.userHome')) {
-            $user->homeFolderId = $sanitizedParams->getInt('homeFolderId');
+            $user->homeFolderId = $sanitizedParams->getInt('homeFolderId', ['default' => 1]);
         }
 
+        // Some configuration is only avaialble to super admins.
         if ($this->getUser()->isSuperAdmin()) {
             $user->userTypeId = $sanitizedParams->getInt('userTypeId');
             if ($user->retired === 1) {
                 $user->isSystemNotification = 0;
                 $user->isDisplayNotification = 0;
+                $user->isDataSetNotification = 0;
+                $user->isCustomNotification = 0;
+                $user->isLayoutNotification = 0;
+                $user->isLibraryNotification = 0;
+                $user->isReportNotification = 0;
+                $user->isScheduleNotification = 0;
             } else {
                 $user->isSystemNotification = $sanitizedParams->getCheckbox('isSystemNotification');
                 $user->isDisplayNotification = $sanitizedParams->getCheckbox('isDisplayNotification');
+                $user->isDataSetNotification = $sanitizedParams->getCheckbox('isDataSetNotification');
+                $user->isCustomNotification = $sanitizedParams->getCheckbox('isCustomNotification');
+                $user->isLayoutNotification = $sanitizedParams->getCheckbox('isLayoutNotification');
+                $user->isLibraryNotification = $sanitizedParams->getCheckbox('isLibraryNotification');
+                $user->isReportNotification = $sanitizedParams->getCheckbox('isReportNotification');
+                $user->isScheduleNotification = $sanitizedParams->getCheckbox('isScheduleNotification');
             }
         }
 
@@ -863,7 +881,10 @@ class User extends Base
         // Handle enabled features for the homepage.
         $homepage = $this->userGroupFactory->getHomepageByName($user->homePageId);
         if (!empty($homepage->feature) && !$user->featureEnabled($homepage->feature)) {
-            throw new InvalidArgumentException(__('User does not have the enabled Feature for this Dashboard'), 'homePageId');
+            throw new InvalidArgumentException(
+                __('User does not have the enabled Feature for this Dashboard'),
+                'homePageId'
+            );
         }
 
         $this->getLog()->debug('Homepage validated.');
@@ -899,6 +920,10 @@ class User extends Base
 
         $this->getLog()->debug('User saved, about to return.');
 
+        // Re-fetch the user before returning to ensure all fields are populated,
+        // especially those omitted in the edit request.
+        $user = $this->userFactory->getById($id);
+
         // Return
         $this->getState()->hydrate([
             'message' => sprintf(__('Edited %s'), $user->userName),
@@ -1481,17 +1506,16 @@ class User extends Base
      * Force User Password Change
      * @param Request $request
      * @param Response $response
-     * @return \Psr\Http\Message\ResponseInterface|Response
-     * @throws GeneralException
-     * @throws \Xibo\Support\Exception\ControllerNotImplemented
+     * @return \Slim\Http\Response
+     * @throws \Xibo\Support\Exception\GeneralException
      */
-    public function forceChangePasswordPage(Request $request, Response $response)
+    public function forceChangePasswordPage(Request $request, Response $response): Response
     {
         $user = $this->getUser();
 
         // if the flag to force change password is not set to 1 then redirect to the Homepage
         if ($user->isPasswordChangeRequired != 1) {
-            $response->withRedirect('home');
+            return $response->withRedirect($this->urlFor($request, 'home'));
         }
 
         $this->getState()->template = 'user-force-change-password-page';
@@ -1503,25 +1527,30 @@ class User extends Base
      * Force change my Password
      * @param Request $request
      * @param Response $response
-     * @return \Psr\Http\Message\ResponseInterface|Response
-     * @throws GeneralException
-     * @throws InvalidArgumentException
-     * @throws \Xibo\Support\Exception\ControllerNotImplemented
-     * @throws \Xibo\Support\Exception\DuplicateEntityException
+     * @return \Slim\Http\Response
+     * @throws \Xibo\Support\Exception\GeneralException
      */
-    public function forceChangePassword(Request $request, Response $response)
+    public function forceChangePassword(Request $request, Response $response): Response
     {
-        // Save the user
         $user = $this->getUser();
+
+        // This is only valid if the user has that option set on their account
+        if ($user->isPasswordChangeRequired != 1) {
+            throw new AccessDeniedException();
+        }
+
+        // Save the user
         $sanitizedParams = $this->getSanitizer($request->getParams());
         $newPassword = $sanitizedParams->getString('newPassword');
         $retypeNewPassword = $sanitizedParams->getString('retypeNewPassword');
 
-        if ($newPassword == null || $retypeNewPassword == '')
+        if ($newPassword == null || $retypeNewPassword == '') {
             throw new InvalidArgumentException(__('Please enter the password'), 'password');
+        }
 
-        if ($newPassword != $retypeNewPassword)
+        if ($newPassword != $retypeNewPassword) {
             throw new InvalidArgumentException(__('Passwords do not match'), 'password');
+        }
 
         // Make sure that the new password doesn't verify against the existing hash
         try {
@@ -1874,6 +1903,13 @@ class User extends Base
             throw new InvalidArgumentException(__('You cannot share the root folder'), 'id');
         }
 
+        if ($object->permissionsClass() === 'Xibo\Entity\Region' && $object->type === 'canvas') {
+            throw new InvalidArgumentException(
+                __('You cannot share the Canvas on a Layout, share the layout instead.'),
+                'type',
+            );
+        }
+
         $sanitizedParams = $this->getSanitizer($request->getParams());
 
         // Get all current permissions
@@ -1908,13 +1944,40 @@ class User extends Base
             if ($object->permissionsClass() == 'Xibo\Entity\Campaign') {
                 $this->getLog()->debug('Changing owner on child Layout');
 
-                $this->getDispatcher()->dispatch(LayoutOwnerChangeEvent::$NAME, new LayoutOwnerChangeEvent($object->getId(), $ownerId));
+                $this->getDispatcher()->dispatch(
+                    new LayoutOwnerChangeEvent($object->getId(), $ownerId),
+                    LayoutOwnerChangeEvent::$NAME,
+                );
             }
         }
 
         if ($object->permissionsClass() === 'Xibo\Entity\Folder') {
             /** @var $object \Xibo\Entity\Folder */
             $object->managePermissions();
+        } else if ($object->permissionsClass() === 'Xibo\Entity\Campaign') {
+            // Update any Canvas Regions to have the same permissions.
+            $event = new LayoutSharingChangeEvent($object->getId());
+            $this->getDispatcher()->dispatch($event, LayoutSharingChangeEvent::$NAME);
+
+            foreach ($event->getCanvasRegionIds() as $canvasRegionId) {
+                $this->getLog()->debug('permissions: canvas region detected, cascading permissions');
+                $permissions = $this->permissionFactory->getAllByObjectId(
+                    $this->getUser(),
+                    'Xibo\Entity\Region',
+                    $canvasRegionId,
+                );
+                $this->updatePermissions($permissions, $groupIds);
+            }
+        } else if ($object->permissionsClass() === 'Xibo\Entity\Region') {
+            /** @var $object \Xibo\Entity\Region */
+            // The regions own playlist should always have the same permissions.
+            $permissions = $this->permissionFactory->getAllByObjectId(
+                $this->getUser(),
+                'Xibo\Entity\Playlist',
+                $object->getPlaylist()->playlistId
+            );
+
+            $this->updatePermissions($permissions, $groupIds);
         }
 
         // Return
@@ -2118,7 +2181,12 @@ class User extends Base
         $requestedPreference =  $request->getQueryParam('preference');
 
         if (!empty($requestedPreference)) {
-            $this->getState()->setData($this->getUser()->getOption($requestedPreference));
+            try {
+                $option = $this->getUser()->getOption($requestedPreference);
+            } catch (NotFoundException $exception) {
+                $option = [];
+            }
+            $this->getState()->setData($option);
         } else {
             $this->getState()->setData($this->getUser()->getUserOptions());
         }
