@@ -1,8 +1,8 @@
 <?php
 /*
- * Copyright (c) 2022 Xibo Signage Ltd
+ * Copyright (C) 2023 Xibo Signage Ltd
  *
- * Xibo - Digital Signage - http://www.xibo.org.uk
+ * Xibo - Digital Signage - https://xibosignage.com
  *
  * This file is part of Xibo.
  *
@@ -20,10 +20,10 @@
  * along with Xibo.  If not, see <http://www.gnu.org/licenses/>.
  */
 namespace Xibo\Controller;
+
 use Slim\Http\Response as Response;
 use Slim\Http\ServerRequest as Request;
 use Xibo\Entity\Permission;
-use Xibo\Entity\User;
 use Xibo\Factory\PermissionFactory;
 use Xibo\Factory\UserFactory;
 use Xibo\Factory\UserGroupFactory;
@@ -123,7 +123,10 @@ class UserGroup extends Base
         $filterBy = [
             'groupId' => $sanitizedQueryParams->getInt('userGroupId'),
             'group' => $sanitizedQueryParams->getString('userGroup'),
-            'isUserSpecific' => 0
+            'useRegexForName' => $sanitizedQueryParams->getCheckbox('useRegexForName'),
+            'logicalOperatorName' => $sanitizedQueryParams->getString('logicalOperatorName'),
+            'isUserSpecific' => 0,
+            'userIdMember' => $sanitizedQueryParams->getInt('userIdMember'),
         ];
 
         $groups = $this->userGroupFactory->query(
@@ -134,19 +137,23 @@ class UserGroup extends Base
         foreach ($groups as $group) {
             /* @var \Xibo\Entity\UserGroup $group */
 
-            $group->libraryQuotaFormatted = ByteFormatter::format($group->libraryQuota * 1024);
+            $group->setUnmatchedProperty(
+                'libraryQuotaFormatted',
+                ByteFormatter::format($group->libraryQuota * 1024)
+            );
 
-            if ($this->isApi($request))
+            if ($this->isApi($request)) {
                 continue;
+            }
 
             // we only want to show certain buttons, depending on the user logged in
             if ($this->getUser()->featureEnabled('usergroup.modify')
-                && $this->isEditable($group)
+                && $this->getUser()->checkEditable($group)
             ) {
                 // Edit
                 $group->buttons[] = array(
                     'id' => 'usergroup_button_edit',
-                    'url' => $this->urlFor($request,'group.edit.form', ['id' => $group->groupId]),
+                    'url' => $this->urlFor($request, 'group.edit.form', ['id' => $group->groupId]),
                     'text' => __('Edit')
                 );
 
@@ -154,7 +161,7 @@ class UserGroup extends Base
                     // Delete
                     $group->buttons[] = array(
                         'id' => 'usergroup_button_delete',
-                        'url' => $this->urlFor($request,'group.delete.form', ['id' => $group->groupId]),
+                        'url' => $this->urlFor($request, 'group.delete.form', ['id' => $group->groupId]),
                         'text' => __('Delete')
                     );
 
@@ -163,7 +170,7 @@ class UserGroup extends Base
                     // Copy
                     $group->buttons[] = array(
                         'id' => 'usergroup_button_copy',
-                        'url' => $this->urlFor($request,'group.copy.form', ['id' => $group->groupId]),
+                        'url' => $this->urlFor($request, 'group.copy.form', ['id' => $group->groupId]),
                         'text' => __('Copy')
                     );
 
@@ -173,7 +180,7 @@ class UserGroup extends Base
                 // Members
                 $group->buttons[] = array(
                     'id' => 'usergroup_button_members',
-                    'url' => $this->urlFor($request,'group.members.form', ['id' => $group->groupId]),
+                    'url' => $this->urlFor($request, 'group.members.form', ['id' => $group->groupId]),
                     'text' => __('Members')
                 );
 
@@ -182,7 +189,7 @@ class UserGroup extends Base
                     $group->buttons[] = ['divider' => true];
                     $group->buttons[] = array(
                         'id' => 'usergroup_button_page_security',
-                        'url' => $this->urlFor($request,'group.acl.form', ['id' => $group->groupId]),
+                        'url' => $this->urlFor($request, 'group.acl.form', ['id' => $group->groupId]),
                         'text' => __('Features'),
                         'title' => __('Turn Features on/off for this User')
                     );
@@ -208,12 +215,6 @@ class UserGroup extends Base
     function addForm(Request $request, Response $response)
     {
         $this->getState()->template = 'usergroup-form-add';
-        $this->getState()->setData([
-            'help' => [
-                'add' => $this->getHelp()->link('UserGroup', 'Add')
-            ]
-        ]);
-
         return $this->render($request, $response);
     }
 
@@ -232,16 +233,13 @@ class UserGroup extends Base
     {
         $group = $this->userGroupFactory->getById($id);
 
-        if (!$this->isEditable($group)) {
+        if (!$this->getUser()->checkEditable($group)) {
             throw new AccessDeniedException();
         }
 
         $this->getState()->template = 'usergroup-form-edit';
         $this->getState()->setData([
             'group' => $group,
-            'help' => [
-                'add' => $this->getHelp()->link('UserGroup', 'Edit')
-            ]
         ]);
 
         return $this->render($request, $response);
@@ -262,16 +260,13 @@ class UserGroup extends Base
     {
         $group = $this->userGroupFactory->getById($id);
 
-        if (!$this->isEditable($group)) {
+        if (!$this->getUser()->checkEditable($group)) {
             throw new AccessDeniedException();
         }
 
         $this->getState()->template = 'usergroup-form-delete';
         $this->getState()->setData([
             'group' => $group,
-            'help' => [
-                'delete' => $this->getHelp()->link('UserGroup', 'Delete')
-            ]
         ]);
 
         return $this->render($request, $response);
@@ -316,10 +311,53 @@ class UserGroup extends Base
      *  @SWG\Parameter(
      *      name="isDisplayNotification",
      *      in="formData",
-     *      description="Flag (0, 1), should members of this Group receive Display notifications for Displays they have permissions to see",
+     *      description="Flag (0, 1), should members of this Group receive Display notifications
+     * for Displays they have permissions to see",
      *      type="integer",
      *      required=false
      *   ),
+     * @SWG\Parameter(
+     *        name="isDataSetNotification",
+     *        in="formData",
+     *        description="Flag (0, 1), should members of this Group receive DataSet notification emails?",
+     *        type="integer",
+     *        required=false
+     *     ),
+     * @SWG\Parameter(
+     *        name="isLayoutNotification",
+     *        in="formData",
+     *        description="Flag (0, 1), should members of this Group receive Layout notification emails?",
+     *        type="integer",
+     *        required=false
+     *     ),
+     * @SWG\Parameter(
+     *        name="isLibraryNotification",
+     *        in="formData",
+     *        description="Flag (0, 1), should members of this Group receive Library notification emails?",
+     *        type="integer",
+     *        required=false
+     *     ),
+     * @SWG\Parameter(
+     *        name="isReportNotification",
+     *        in="formData",
+     *        description="Flag (0, 1), should members of this Group receive Report notification emails?",
+     *        type="integer",
+     *        required=false
+     *     ),
+     * @SWG\Parameter(
+     *         name="isScheduleNotification",
+     *         in="formData",
+     *         description="Flag (0, 1), should members of this Group receive Schedule notification emails?",
+     *         type="integer",
+     *         required=false
+     *      ),
+     * @SWG\Parameter(
+     *         name="isCustomNotification",
+     *         in="formData",
+     *         description="Flag (0, 1), should members of this Group receive Custom notification emails?",
+     *         type="integer",
+     *         required=false
+     *      ),
      *  @SWG\Parameter(
      *      name="isShownForAddUser",
      *      in="formData",
@@ -352,7 +390,7 @@ class UserGroup extends Base
      * @throws \Xibo\Support\Exception\GeneralException
      * @throws \Xibo\Support\Exception\InvalidArgumentException
      */
-    function add(Request $request, Response $response)
+    public function add(Request $request, Response $response)
     {
         $sanitizedParams = $this->getSanitizer($request->getParams());
 
@@ -370,6 +408,12 @@ class UserGroup extends Base
         if ($this->getUser()->userTypeId == 1) {
             $group->isSystemNotification = $sanitizedParams->getCheckbox('isSystemNotification');
             $group->isDisplayNotification = $sanitizedParams->getCheckbox('isDisplayNotification');
+            $group->isDataSetNotification = $sanitizedParams->getCheckbox('isDataSetNotification');
+            $group->isCustomNotification = $sanitizedParams->getCheckbox('isCustomNotification');
+            $group->isLayoutNotification = $sanitizedParams->getCheckbox('isLayoutNotification');
+            $group->isLibraryNotification = $sanitizedParams->getCheckbox('isLibraryNotification');
+            $group->isReportNotification = $sanitizedParams->getCheckbox('isReportNotification');
+            $group->isScheduleNotification = $sanitizedParams->getCheckbox('isScheduleNotification');
             $group->isShownForAddUser = $sanitizedParams->getCheckbox('isShownForAddUser');
             $group->defaultHomepageId = $sanitizedParams->getString('defaultHomepageId');
         }
@@ -439,10 +483,53 @@ class UserGroup extends Base
      *  @SWG\Parameter(
      *      name="isDisplayNotification",
      *      in="formData",
-     *      description="Flag (0, 1), should members of this Group receive Display notifications for Displays they have permissions to see",
+     *      description="Flag (0, 1), should members of this Group receive Display notifications
+     * for Displays they have permissions to see",
      *      type="integer",
      *      required=false
      *   ),
+     * @SWG\Parameter(
+     *       name="isDataSetNotification",
+     *       in="formData",
+     *       description="Flag (0, 1), should members of this Group receive DataSet notification emails?",
+     *       type="integer",
+     *       required=false
+     *    ),
+     * @SWG\Parameter(
+     *       name="isLayoutNotification",
+     *       in="formData",
+     *       description="Flag (0, 1), should members of this Group receive Layout notification emails?",
+     *       type="integer",
+     *       required=false
+     *    ),
+     * @SWG\Parameter(
+     *       name="isLibraryNotification",
+     *       in="formData",
+     *       description="Flag (0, 1), should members of this Group receive Library notification emails?",
+     *       type="integer",
+     *       required=false
+     *    ),
+     * @SWG\Parameter(
+     *       name="isReportNotification",
+     *       in="formData",
+     *       description="Flag (0, 1), should members of this Group receive Report notification emails?",
+     *       type="integer",
+     *       required=false
+     *    ),
+     * @SWG\Parameter(
+     *        name="isScheduleNotification",
+     *        in="formData",
+     *        description="Flag (0, 1), should members of this Group receive Schedule notification emails?",
+     *        type="integer",
+     *        required=false
+     *     ),
+     * @SWG\Parameter(
+     *        name="isCustomNotification",
+     *        in="formData",
+     *        description="Flag (0, 1), should members of this Group receive Custom notification emails?",
+     *        type="integer",
+     *        required=false
+     *     ),
      *  @SWG\Parameter(
      *      name="isShownForAddUser",
      *      in="formData",
@@ -477,7 +564,7 @@ class UserGroup extends Base
      * @throws \Xibo\Support\Exception\InvalidArgumentException
      * @throws \Xibo\Support\Exception\NotFoundException
      */
-    function edit(Request $request, Response $response, $id)
+    public function edit(Request $request, Response $response, $id)
     {
         // Check permissions
         if (!$this->getUser()->isSuperAdmin() && !$this->getUser()->isGroupAdmin()) {
@@ -488,7 +575,7 @@ class UserGroup extends Base
 
         $group = $this->userGroupFactory->getById($id);
 
-        if (!$this->isEditable($group)) {
+        if (!$this->getUser()->checkEditable($group)) {
             throw new AccessDeniedException();
         }
 
@@ -501,12 +588,21 @@ class UserGroup extends Base
         if ($this->getUser()->userTypeId == 1) {
             $group->isSystemNotification = $sanitizedParams->getCheckbox('isSystemNotification');
             $group->isDisplayNotification = $sanitizedParams->getCheckbox('isDisplayNotification');
+            $group->isDataSetNotification = $sanitizedParams->getCheckbox('isDataSetNotification');
+            $group->isCustomNotification = $sanitizedParams->getCheckbox('isCustomNotification');
+            $group->isLayoutNotification = $sanitizedParams->getCheckbox('isLayoutNotification');
+            $group->isLibraryNotification = $sanitizedParams->getCheckbox('isLibraryNotification');
+            $group->isReportNotification = $sanitizedParams->getCheckbox('isReportNotification');
+            $group->isScheduleNotification = $sanitizedParams->getCheckbox('isScheduleNotification');
             $group->isShownForAddUser = $sanitizedParams->getCheckbox('isShownForAddUser');
             $group->defaultHomepageId = $sanitizedParams->getString('defaultHomepageId');
 
             // if we have homepage set assign matching feature if it does not already exist
             if (!empty($group->defaultHomepageId)
-                && !in_array($this->userGroupFactory->getHomepageByName($group->defaultHomepageId)->feature, $group->features)
+                && !in_array(
+                    $this->userGroupFactory->getHomepageByName($group->defaultHomepageId)->feature,
+                    $group->features
+                )
                 && $group->defaultHomepageId !== 'icondashboard.view'
             ) {
                 $group->features[] = $this->userGroupFactory->getHomepageByName($group->defaultHomepageId)->feature;
@@ -556,7 +652,7 @@ class UserGroup extends Base
      *  )
      * )
      */
-    function delete(Request $request, Response $response, $id)
+    public function delete(Request $request, Response $response, $id)
     {
         // Check permissions
         if (!$this->getUser()->isSuperAdmin()) {
@@ -565,7 +661,7 @@ class UserGroup extends Base
 
         $group = $this->userGroupFactory->getById($id);
 
-        if (!$this->isEditable($group)) {
+        if (!$this->getUser()->checkEditable($group)) {
             throw new AccessDeniedException();
         }
 
@@ -612,7 +708,6 @@ class UserGroup extends Base
             'features' => $group->features,
             'inheritedFeatures' => $inheritedFeatures,
             'userGroupFactory' => $this->userGroupFactory,
-            'help' => $this->getHelp()->link('User', 'Acl')
         ];
 
         $this->getState()->template = 'usergroup-form-acl';
@@ -679,46 +774,19 @@ class UserGroup extends Base
     {
         $group = $this->userGroupFactory->getById($id);
 
-        if (!$this->isEditable($group)) {
+        if (!$this->getUser()->checkEditable($group)) {
             throw new AccessDeniedException();
         }
 
         // Users in group
         $usersAssigned = $this->userFactory->query(null, ['groupIds' => [$id]]);
 
-        // Users not in group
-        $allUsers = $this->userFactory->query();
-
-        // The available users are all users except users already in assigned users
-        $checkboxes = [];
-
-        foreach ($allUsers as $user) {
-            /* @var User $user */
-            // Check to see if it exists in $usersAssigned
-            $exists = false;
-            foreach ($usersAssigned as $userAssigned) {
-                /* @var User $userAssigned */
-                if ($userAssigned->userId == $user->userId) {
-                    $exists = true;
-                    break;
-                }
-            }
-
-            // Store this checkbox
-            $checkbox = array(
-                'id' => $user->userId,
-                'name' => $user->userName,
-                'value_checked' => (($exists) ? 'checked' : '')
-            );
-
-            $checkboxes[] = $checkbox;
-        }
-
         $this->getState()->template = 'usergroup-form-members';
         $this->getState()->setData([
             'group' => $group,
-            'checkboxes' => $checkboxes,
-            'help' =>  $this->getHelp()->link('UserGroup', 'Members')
+            'extra' => [
+                'usersAssigned' => $usersAssigned
+            ],
         ]);
 
         return $this->render($request, $response);
@@ -773,7 +841,8 @@ class UserGroup extends Base
         $sanitizedParams = $this->getSanitizer($request->getParams());
 
         $group = $this->userGroupFactory->getById($id);
-        if (!$this->isEditable($group)) {
+
+        if (!$this->getUser()->checkEditable($group)) {
             throw new AccessDeniedException();
         }
 
@@ -877,7 +946,7 @@ class UserGroup extends Base
         $group = $this->userGroupFactory->getById($id);
         $sanitizedParams = $this->getSanitizer($request->getParams());
 
-        if (!$this->isEditable($group)) {
+        if (!$this->getUser()->checkEditable($group)) {
             throw new AccessDeniedException();
         }
 
@@ -913,7 +982,7 @@ class UserGroup extends Base
     {
         $group = $this->userGroupFactory->getById($id);
 
-        if (!$this->isEditable($group)) {
+        if (!$this->getUser()->checkEditable($group)) {
             throw new AccessDeniedException();
         }
 
@@ -989,7 +1058,7 @@ class UserGroup extends Base
         $sanitizedParams = $this->getSanitizer($request->getParams());
 
         // Check we have permission to view this group
-        if (!$this->isEditable($group)) {
+        if (!$this->getUser()->checkEditable($group)) {
             throw new AccessDeniedException();
         }
 
@@ -1025,14 +1094,4 @@ class UserGroup extends Base
 
         return $this->render($request, $response);
     }
-
-    /**
-     * @param \Xibo\Entity\UserGroup $group
-     * @return bool
-     */
-    private function isEditable($group)
-    {
-        return $this->getUser()->isSuperAdmin()
-            || ($this->getUser()->isGroupAdmin() && count(array_intersect($this->getUser()->groups, [$group])));
-    }
 }
