@@ -1,8 +1,8 @@
 <?php
 /*
- * Copyright (c) 2022 Xibo Signage Ltd
+ * Copyright (C) 2025 Xibo Signage Ltd
  *
- * Xibo - Digital Signage - http://www.xibo.org.uk
+ * Xibo - Digital Signage - https://xibosignage.com
  *
  * This file is part of Xibo.
  *
@@ -24,6 +24,9 @@ namespace Xibo\Dependencies;
 
 use Psr\Container\ContainerInterface;
 
+/**
+ * Helper class to add factories to DI.
+ */
 class Factories
 {
     /**
@@ -40,6 +43,11 @@ class Factories
                 $repository->useBaseDependenciesService($c->get('RepositoryBaseDependenciesService'));
                 return $repository;
             },
+            'apiRequestsFactory' => function (ContainerInterface $c) {
+                $repository = new \Xibo\Factory\ApplicationRequestsFactory();
+                $repository->useBaseDependenciesService($c->get('RepositoryBaseDependenciesService'));
+                return $repository;
+            },
             'applicationFactory' => function (ContainerInterface $c) {
                 $repository = new \Xibo\Factory\ApplicationFactory(
                     $c->get('user'),
@@ -65,7 +73,9 @@ class Factories
                 return $repository;
             },
             'bandwidthFactory' => function (ContainerInterface $c) {
-                $repository = new \Xibo\Factory\BandwidthFactory();
+                $repository = new \Xibo\Factory\BandwidthFactory(
+                    $c->get('pool'),
+                );
                 $repository->useBaseDependenciesService($c->get('RepositoryBaseDependenciesService'));
                 return $repository;
             },
@@ -75,8 +85,7 @@ class Factories
                     $c->get('userFactory'),
                     $c->get('permissionFactory'),
                     $c->get('scheduleFactory'),
-                    $c->get('displayNotifyService'),
-                    $c->get('tagFactory')
+                    $c->get('displayNotifyService')
                 );
                 $repository->useBaseDependenciesService($c->get('RepositoryBaseDependenciesService'));
                 return $repository;
@@ -93,7 +102,9 @@ class Factories
                 $repository = new \Xibo\Factory\ConnectorFactory(
                     $c->get('pool'),
                     $c->get('configService'),
-                    $c->get('jwtService')
+                    $c->get('jwtService'),
+                    $c->get('playerActionService'),
+                    $c
                 );
                 $repository->useBaseDependenciesService($c->get('RepositoryBaseDependenciesService'));
                 return $repository;
@@ -167,12 +178,16 @@ class Factories
                 $repository = new \Xibo\Factory\DisplayGroupFactory(
                     $c->get('user'),
                     $c->get('userFactory'),
-                    $c->get('permissionFactory'),
-                    $c->get('tagFactory')
+                    $c->get('permissionFactory')
                 );
                 $repository->useBaseDependenciesService($c->get('RepositoryBaseDependenciesService'));
                 return $repository;
             },
+            'displayTypeFactory' => function (ContainerInterface $c) {
+                $repository = new \Xibo\Factory\DisplayTypeFactory();
+                $repository->useBaseDependenciesService($c->get('RepositoryBaseDependenciesService'));
+                return $repository;
+            },
             'displayProfileFactory' => function (ContainerInterface $c) {
                 $repository = new \Xibo\Factory\DisplayProfileFactory(
                     $c->get('configService'),
@@ -190,8 +205,10 @@ class Factories
                 $repository->useBaseDependenciesService($c->get('RepositoryBaseDependenciesService'));
                 return $repository;
             },
-            'helpFactory' => function (ContainerInterface $c) {
-                $repository = new \Xibo\Factory\HelpFactory();
+            'fontFactory' => function (ContainerInterface $c) {
+                $repository = new \Xibo\Factory\FontFactory(
+                    $c->get('configService')
+                );
                 $repository->useBaseDependenciesService($c->get('RepositoryBaseDependenciesService'));
                 return $repository;
             },
@@ -206,13 +223,16 @@ class Factories
                     $c->get('campaignFactory'),
                     $c->get('mediaFactory'),
                     $c->get('moduleFactory'),
+                    $c->get('moduleTemplateFactory'),
                     $c->get('resolutionFactory'),
                     $c->get('widgetFactory'),
                     $c->get('widgetOptionFactory'),
                     $c->get('playlistFactory'),
                     $c->get('widgetAudioFactory'),
                     $c->get('actionFactory'),
-                    $c->get('folderFactory')
+                    $c->get('folderFactory'),
+                    $c->get('fontFactory'),
+                    $c->get('widgetDataFactory'),
                 );
                 $repository->useBaseDependenciesService($c->get('RepositoryBaseDependenciesService'));
 
@@ -233,7 +253,6 @@ class Factories
                     $c->get('userFactory'),
                     $c->get('configService'),
                     $c->get('permissionFactory'),
-                    $c->get('tagFactory'),
                     $c->get('playlistFactory')
                 );
                 $repository->useBaseDependenciesService($c->get('RepositoryBaseDependenciesService'));
@@ -266,28 +285,30 @@ class Factories
             },
             'moduleFactory' => function (ContainerInterface $c) {
                 $repository = new \Xibo\Factory\ModuleFactory(
-                    $c->get('user'),
-                    $c->get('userFactory'),
-                    $c->get('moduleService'),
-                    $c->get('widgetFactory'),
-                    $c->get('regionFactory'),
-                    $c->get('playlistFactory'),
-                    $c->get('mediaFactory'),
-                    $c->get('dataSetFactory'),
-                    $c->get('dataSetColumnFactory'),
-                    $c->get('transitionFactory'),
-                    $c->get('displayFactory'),
-                    $c->get('commandFactory'),
-                    $c->get('scheduleFactory'),
-                    $c->get('permissionFactory'),
-                    $c->get('userGroupFactory'),
-                    $c->get('menuBoardFactory'),
-                    $c->get('menuBoardCategoryFactory'),
-                    $c->get('notificationFactory'),
+                    $c->get('configService')->getSetting('LIBRARY_LOCATION') . 'widget',
+                    $c->get('pool'),
                     $c->get('view'),
-                    $c->get('httpCache')
+                    $c->get('configService')
                 );
-                $repository->useBaseDependenciesService($c->get('RepositoryBaseDependenciesService'));
+                $repository
+                    ->setAclDependencies(
+                        $c->get('user'),
+                        $c->get('userFactory')
+                    )
+                    ->useBaseDependenciesService($c->get('RepositoryBaseDependenciesService'));
+                return $repository;
+            },
+            'moduleTemplateFactory' => function (ContainerInterface $c) {
+                $repository = new \Xibo\Factory\ModuleTemplateFactory(
+                    $c->get('pool'),
+                    $c->get('view'),
+                );
+                $repository
+                    ->setAclDependencies(
+                        $c->get('user'),
+                        $c->get('userFactory')
+                    )
+                    ->useBaseDependenciesService($c->get('RepositoryBaseDependenciesService'));
                 return $repository;
             },
             'notificationFactory' => function (ContainerInterface $c) {
@@ -314,8 +335,7 @@ class Factories
                 $repository = new \Xibo\Factory\PlayerVersionFactory(
                     $c->get('user'),
                     $c->get('userFactory'),
-                    $c->get('configService'),
-                    $c->get('mediaFactory')
+                    $c->get('configService')
                 );
                 $repository->useBaseDependenciesService($c->get('RepositoryBaseDependenciesService'));
                 return $repository;
@@ -326,8 +346,7 @@ class Factories
                     $c->get('user'),
                     $c->get('userFactory'),
                     $c->get('permissionFactory'),
-                    $c->get('widgetFactory'),
-                    $c->get('tagFactory')
+                    $c->get('widgetFactory')
                 );
                 $repository->useBaseDependenciesService($c->get('RepositoryBaseDependenciesService'));
                 return $repository;
@@ -385,7 +404,8 @@ class Factories
                     $c->get('userFactory'),
                     $c->get('scheduleReminderFactory'),
                     $c->get('scheduleExclusionFactory'),
-                    $c->get('user')
+                    $c->get('user'),
+                    $c->get('scheduleCriteriaFactory')
                 );
                 $repository->useBaseDependenciesService($c->get('RepositoryBaseDependenciesService'));
                 return $repository;
@@ -404,11 +424,27 @@ class Factories
                 $repository->useBaseDependenciesService($c->get('RepositoryBaseDependenciesService'));
                 return $repository;
             },
+            'scheduleCriteriaFactory' => function (ContainerInterface $c) {
+                $repository = new \Xibo\Factory\ScheduleCriteriaFactory();
+                $repository->useBaseDependenciesService($c->get('RepositoryBaseDependenciesService'));
+                return $repository;
+            },
             'sessionFactory' => function (ContainerInterface $c) {
                 $repository = new \Xibo\Factory\SessionFactory();
                 $repository->useBaseDependenciesService($c->get('RepositoryBaseDependenciesService'));
                 return $repository;
             },
+            'syncGroupFactory' => function (ContainerInterface $c) {
+                $repository = new \Xibo\Factory\SyncGroupFactory(
+                    $c->get('user'),
+                    $c->get('userFactory'),
+                    $c->get('permissionFactory'),
+                    $c->get('displayFactory'),
+                    $c->get('scheduleFactory')
+                );
+                $repository->useBaseDependenciesService($c->get('RepositoryBaseDependenciesService'));
+                return $repository;
+            },
             'tagFactory' => function (ContainerInterface $c) {
                 $repository = new \Xibo\Factory\TagFactory();
                 $repository->useBaseDependenciesService($c->get('RepositoryBaseDependenciesService'));
@@ -469,7 +505,8 @@ class Factories
                     $c->get('widgetAudioFactory'),
                     $c->get('permissionFactory'),
                     $c->get('displayNotifyService'),
-                    $c->get('actionFactory')
+                    $c->get('actionFactory'),
+                    $c->get('moduleTemplateFactory')
                 );
                 $repository->useBaseDependenciesService($c->get('RepositoryBaseDependenciesService'));
                 return $repository;
@@ -489,6 +526,11 @@ class Factories
                 $repository->useBaseDependenciesService($c->get('RepositoryBaseDependenciesService'));
                 return $repository;
             },
+            'widgetDataFactory' => function (ContainerInterface $c) {
+                $repository = new \Xibo\Factory\WidgetDataFactory();
+                $repository->useBaseDependenciesService($c->get('RepositoryBaseDependenciesService'));
+                return $repository;
+            },
         ];
     }
 }
