@@ -126,7 +126,7 @@ class Campaign implements \JsonSerializable
     public $listPlayOrder;
 
     /**
-     * @SWG\Property(description="For an ad campaign, what's the target type, plays|budget")
+     * @SWG\Property(description="For an ad campaign, what's the target type, plays|budget|imp")
      * @var string
      */
     public $targetType;
@@ -397,7 +397,7 @@ class Campaign implements \JsonSerializable
 
             if ($this->targetType === 'budget') {
                 $progress->progressTarget = ($this->spend / $this->target) * 100;
-            } else if ($this->targetType === 'impressions') {
+            } else if ($this->targetType === 'imp') {
                 $progress->progressTarget = ($this->impressions / $this->target) * 100;
             } else {
                 $progress->progressTarget = ($this->plays / $this->target) * 100;
@@ -485,7 +485,7 @@ class Campaign implements \JsonSerializable
         }
 
         if ($this->type === 'ad') {
-            if (!in_array($this->targetType, ['plays', 'budget', 'impressions'])) {
+            if (!in_array($this->targetType, ['plays', 'budget', 'imp'])) {
                 throw new InvalidArgumentException(__('Invalid target type'), 'targetType');
             }
 
@@ -525,11 +525,17 @@ class Campaign implements \JsonSerializable
             'validate' => true,
             'notify' => true,
             'collectNow' => true,
-            'saveTags' => true
+            'saveTags' => true,
+            'isTagEdit' => false
         ], $options);
 
         $this->getLog()->debug('Saving ' . $this);
 
+        // Manually load display group IDs when editing only campaign tags.
+        if ($options['isTagEdit']) {
+            $this->displayGroupIds = $this->loadDisplayGroupIds();
+        }
+
         if ($options['validate']) {
             $this->validate();
         }
@@ -818,7 +824,8 @@ class Campaign implements \JsonSerializable
                     startDt = :startDt,
                     endDt = :endDt,
                     folderId = :folderId,
-                    permissionsFolderId = :permissionsFolderId
+                    permissionsFolderId = :permissionsFolderId,
+                    modifiedBy = :modifiedBy
              WHERE campaignID = :campaignId
         ', [
             'campaignId' => $this->campaignId,
@@ -838,6 +845,7 @@ class Campaign implements \JsonSerializable
             'ref5' => empty($this->ref5) ? null : $this->ref5,
             'folderId' => $this->folderId,
             'permissionsFolderId' => $this->permissionsFolderId,
+            'modifiedBy' => $this->modifiedBy,
         ]);
     }
 
