@@ -1,8 +1,8 @@
 <?php
 /*
- * Copyright (c) 2022 Xibo Signage Ltd
+ * Copyright (C) 2025 Xibo Signage Ltd
  *
- * Xibo - Digital Signage - http://www.xibo.org.uk
+ * Xibo - Digital Signage - https://xibosignage.com
  *
  * This file is part of Xibo.
  *
@@ -21,18 +21,17 @@
  */
 namespace Xibo\Entity;
 
-
 use Carbon\Carbon;
 use Respect\Validation\Validator as v;
 use Stash\Interfaces\PoolInterface;
 use Xibo\Event\DisplayGroupLoadEvent;
+use Xibo\Event\TriggerTaskEvent;
 use Xibo\Factory\DisplayFactory;
 use Xibo\Factory\DisplayGroupFactory;
 use Xibo\Factory\DisplayProfileFactory;
 use Xibo\Factory\FolderFactory;
 use Xibo\Factory\LayoutFactory;
 use Xibo\Helper\DateFormatHelper;
-use Xibo\Helper\Profiler;
 use Xibo\Service\ConfigServiceInterface;
 use Xibo\Service\LogServiceInterface;
 use Xibo\Storage\StorageServiceInterface;
@@ -44,7 +43,6 @@ use Xibo\Support\Exception\NotFoundException;
 /**
  * Class Display
  * @package Xibo\Entity
- * @property $isCmsTransferInProgress Is a transfer to another CMS in progress?
  *
  * @SWG\Definition()
  */
@@ -55,6 +53,7 @@ class Display implements \JsonSerializable
     public static $STATUS_PENDING = 3;
 
     use EntityTrait;
+    use TagLinkTrait;
 
     /**
      * @SWG\Property(description="The ID of this Display")
@@ -62,6 +61,102 @@ class Display implements \JsonSerializable
      */
     public $displayId;
 
+    /**
+     * @SWG\Property(description="The Display Type ID of this Display")
+     * @var int
+     */
+    public $displayTypeId;
+
+    /**
+     * @SWG\Property(description="The Venue ID of this Display")
+     * @var int
+     */
+    public $venueId;
+
+    /**
+     * @SWG\Property(description="The Location Address of this Display")
+     * @var string
+     */
+    public $address;
+
+    /**
+     * @SWG\Property(description="Is this Display mobile?")
+     * @var int
+     */
+    public $isMobile;
+
+    /**
+     * @SWG\Property(description="The Languages supported in this display location")
+     * @var string
+     */
+    public $languages;
+
+    /**
+     * @SWG\Property(description="The type of this Display")
+     * @var string
+     */
+    public $displayType;
+
+    /**
+     * @SWG\Property(description="The screen size of this Display")
+     * @var int
+     */
+    public $screenSize;
+
+    /**
+     * @SWG\Property(description="Is this Display Outdoor?")
+     * @var int
+     */
+    public $isOutdoor;
+
+    /**
+     * @SWG\Property(description="The custom ID (an Id of any external system) of this Display")
+     * @var string
+     */
+    public $customId;
+
+    /**
+     * @SWG\Property(description="The Cost Per Play of this Display")
+     * @var double
+     */
+    public $costPerPlay;
+
+    /**
+     * @SWG\Property(description="The Impressions Per Play of this Display")
+     * @var double
+     */
+    public $impressionsPerPlay;
+
+    /**
+     * @SWG\Property(description="Optional Reference 1")
+     * @var string
+     */
+    public $ref1;
+
+    /**
+     * @SWG\Property(description="Optional Reference 2")
+     * @var string
+     */
+    public $ref2;
+
+    /**
+     * @SWG\Property(description="Optional Reference 3")
+     * @var string
+     */
+    public $ref3;
+
+    /**
+     * @SWG\Property(description="Optional Reference 4")
+     * @var string
+     */
+    public $ref4;
+
+    /**
+     * @SWG\Property(description="Optional Reference 5")
+     * @var string
+     */
+    public $ref5;
+
     /**
      * @SWG\Property(description="Flag indicating whether this Display is recording Auditing Information from XMDS")
      * @var int
@@ -310,10 +405,10 @@ class Display implements \JsonSerializable
     public $timeZone;
 
     /**
-     * @SWG\Property(description="Tags associated with this Display")
-     * @var Tag[]
+     * @SWG\Property(description="Tags associated with this Display, array of TagLink objects")
+     * @var TagLink[]
      */
-    public $tags;
+    public $tags = [];
 
     /**
      * @SWG\Property(description="The configuration options that will overwrite Display Profile Config")
@@ -405,6 +500,48 @@ class Display implements \JsonSerializable
      */
     public $countFaults;
 
+    /**
+     * @SWG\Property(description="LAN IP Address, if available on the Player")
+     * @var string
+     */
+    public $lanIpAddress;
+
+    /**
+     * @SWG\Property(description="The Display Group ID this Display is synced to")
+     * @var int
+     */
+    public $syncGroupId;
+
+    /**
+     * @SWG\Property(description="The OS version of the Display")
+     * @var string
+     */
+    public $osVersion;
+
+    /**
+     * @SWG\Property(description="The SDK version of the Display")
+     * @var string
+     */
+    public $osSdk;
+
+    /**
+     * @SWG\Property(description="The manufacturer of the Display")
+     * @var string
+     */
+    public $manufacturer;
+
+    /**
+     * @SWG\Property(description="The brand of the Display")
+     * @var string
+     */
+    public $brand;
+
+    /**
+     * @SWG\Property(description="The model of the Display")
+     * @var string
+     */
+    public $model;
+
     /** @var array The configuration from the Display Profile  */
     private $profileConfig;
 
@@ -422,7 +559,7 @@ class Display implements \JsonSerializable
      */
     private $commands = null;
 
-    public static $saveOptionsMinimum = ['validate' => false, 'audit' => false];
+    public static $saveOptionsMinimum = ['validate' => false, 'audit' => false, 'setModifiedDt' => false];
 
     /**
      * @var ConfigServiceInterface
@@ -474,9 +611,6 @@ class Display implements \JsonSerializable
         $this->displayProfileFactory = $displayProfileFactory;
         $this->displayFactory = $displayFactory;
         $this->folderFactory = $folderFactory;
-
-        // Initialise extra validation rules
-        v::with('Xibo\\Validation\\Rules\\');
     }
 
     /**
@@ -521,11 +655,11 @@ class Display implements \JsonSerializable
 
     /**
      * @return \Xibo\Entity\DisplayProfile
+     * @throws \Xibo\Support\Exception\NotFoundException
      */
-    public function getDisplayProfile()
+    public function getDisplayProfile(): DisplayProfile
     {
         if ($this->_displayProfile === null) {
-
             try {
                 if ($this->displayProfileId == 0) {
                     // Load the default profile
@@ -534,30 +668,81 @@ class Display implements \JsonSerializable
                     // Load the specified profile
                     $displayProfile = $this->displayProfileFactory->getById($this->displayProfileId);
                 }
-            } catch (NotFoundException $e) {
-                $this->getLog()->error('Cannot get display profile');
-                $this->getLog()->debug($e->getTraceAsString());
+            } catch (NotFoundException) {
+                $this->getLog()->error('getDisplayProfile: Cannot get display profile, '
+                    . $this->clientType . ' not found.');
 
                 $displayProfile = $this->displayProfileFactory->getUnknownProfile($this->clientType);
             }
 
             // Set our display profile
             $this->_displayProfile = $displayProfile;
-
         }
 
         return $this->_displayProfile;
     }
 
+    /**
+     * @return array
+     */
+    public function getLanguages()
+    {
+        return empty($this->languages) ? [] : explode(',', $this->languages);
+    }
+
+    /**
+     * @return bool true is this display is a PWA
+     */
+    public function isPwa(): bool
+    {
+        return $this->clientType === 'chromeOS';
+    }
+
     /**
      * Is this display auditing?
      * return bool
      */
-    public function isAuditing()
+    public function isAuditing(): bool
     {
-        $this->getLog()->debug(sprintf('Testing whether this display is auditing. %d vs %d.', $this->auditingUntil, Carbon::now()->format('U')));
+        $this->getLog()->debug(sprintf(
+            'Testing whether this display is auditing. %d vs %d.',
+            $this->auditingUntil,
+            Carbon::now()->format('U')
+        ));
+
         // Test $this->auditingUntil against the current date.
-        return ($this->auditingUntil >= Carbon::now()->format('U'));
+        return (!empty($this->auditingUntil) && $this->auditingUntil >= Carbon::now()->format('U'));
+    }
+
+    /**
+     * Does this display has elevated log level?
+     * @return bool
+     * @throws NotFoundException
+     */
+    public function isElevatedLogging(): bool
+    {
+        $elevatedUntil = $this->getSetting('elevateLogsUntil', 0);
+
+        $this->getLog()->debug(sprintf(
+            'Testing whether this display has elevated log level. %d vs %d.',
+            $elevatedUntil,
+            Carbon::now()->format('U')
+        ));
+
+        return (!empty($elevatedUntil) && $elevatedUntil >= Carbon::now()->format('U'));
+    }
+
+    /**
+     * Get current log level for this Display
+     * @return string
+     * @throws NotFoundException
+     */
+    public function getLogLevel(): string
+    {
+        $restingLogLevel = $this->getSetting('logLevel', 'error');
+        $isElevated = $this->isElevatedLogging();
+
+        return $isElevated ? 'audit' : $restingLogLevel;
     }
 
     /**
@@ -576,30 +761,49 @@ class Display implements \JsonSerializable
      */
     public function validate()
     {
-        if (!v::stringType()->notEmpty()->validate($this->display))
+        if (!v::stringType()->notEmpty()->validate($this->display)) {
             throw new InvalidArgumentException(__('Can not have a display without a name'), 'name');
+        }
 
-        if (!v::stringType()->notEmpty()->validate($this->license))
+        if (!v::stringType()->notEmpty()->validate($this->license)) {
             throw new InvalidArgumentException(__('Can not have a display without a hardware key'), 'license');
+        }
 
-        if ($this->wakeOnLanEnabled == 1 && $this->wakeOnLanTime == '')
-            throw new InvalidArgumentException(__('Wake on Lan is enabled, but you have not specified a time to wake the display'), 'wakeonlan');
-
+        if ($this->wakeOnLanEnabled == 1 && $this->wakeOnLanTime == '') {
+            throw new InvalidArgumentException(
+                __('Wake on Lan is enabled, but you have not specified a time to wake the display'),
+                'wakeonlan'
+            );
+        }
         // Broadcast Address
-        if ($this->broadCastAddress != '' && !v::ip()->validate($this->broadCastAddress))
-            throw new InvalidArgumentException(__('BroadCast Address is not a valid IP Address'), 'broadCastAddress');
+        if ($this->broadCastAddress != '' && !v::ip()->validate($this->broadCastAddress)) {
+            throw new InvalidArgumentException(
+                __('BroadCast Address is not a valid IP Address'),
+                'broadCastAddress'
+            );
+        }
 
         // CIDR
-        if (!empty($this->cidr) && !v::numeric()->between(0, 32)->validate($this->cidr))
-            throw new InvalidArgumentException(__('CIDR subnet mask is not a number within the range of 0 to 32.'), 'cidr');
+        if (!empty($this->cidr) && !v::numeric()->between(0, 32)->validate($this->cidr)) {
+            throw new InvalidArgumentException(
+                __('CIDR subnet mask is not a number within the range of 0 to 32.'),
+                'cidr'
+            );
+        }
 
         // secureOn
         if ($this->secureOn != '') {
             $this->secureOn = strtoupper($this->secureOn);
-            $this->secureOn = str_replace(":", "-", $this->secureOn);
+            $this->secureOn = str_replace(':', '-', $this->secureOn);
 
-            if ((!preg_match("/([A-F0-9]{2}[-]){5}([0-9A-F]){2}/", $this->secureOn)) || (strlen($this->secureOn) != 17))
-                throw new InvalidArgumentException(__('Pattern of secureOn-password is not "xx-xx-xx-xx-xx-xx" (x = digit or CAPITAL letter)'), 'secureOn');
+            if ((!preg_match('/([A-F0-9]{2}[-]){5}([0-9A-F]){2}/', $this->secureOn))
+                || (strlen($this->secureOn) != 17)
+            ) {
+                throw new InvalidArgumentException(
+                    __('Pattern of secureOn-password is not "xx-xx-xx-xx-xx-xx" (x = digit or CAPITAL letter)'),
+                    'secureOn'
+                );
+            }
         }
 
         // Mac Address Changes
@@ -610,14 +814,43 @@ class Display implements \JsonSerializable
         }
 
         // Lat/Long
-        if (!empty($this->longitude) && !v::longitude()->validate($this->longitude))
+        if (!empty($this->longitude) && !v::longitude()->validate($this->longitude)) {
             throw new InvalidArgumentException(__('The longitude entered is not valid.'), 'longitude');
+        }
 
-        if (!empty($this->latitude) && !v::latitude()->validate($this->latitude))
+        if (!empty($this->latitude) && !v::latitude()->validate($this->latitude)) {
             throw new InvalidArgumentException(__('The latitude entered is not valid.'), 'latitude');
+        }
 
         if ($this->bandwidthLimit !== null && !v::intType()->min(0)->validate($this->bandwidthLimit)) {
-            throw new InvalidArgumentException(__('Bandwidth limit must be a whole number greater than 0.'), 'bandwidthLimit');
+            throw new InvalidArgumentException(
+                __('Bandwidth limit must be a whole number greater than 0.'),
+                'bandwidthLimit'
+            );
+        }
+
+        // do we have default Layout set?
+        if (empty($this->defaultLayoutId)) {
+            // do we have global default Layout ?
+            $globalDefaultLayoutId = $this->config->getSetting('DEFAULT_LAYOUT');
+            if (!empty($globalDefaultLayoutId)) {
+                $this->getLog()->debug(
+                    'No default Layout set on Display ID '
+                    . $this->displayId
+                    . ' falling back to global default Layout.'
+                );
+                $this->defaultLayoutId = $globalDefaultLayoutId;
+                $this->notify();
+            } else {
+                // we have no Default Layout and no global Default Layout
+                $this->getLog()->error(
+                    'No global default Layout set and no default Layout set for Display ID ' . $this->displayId
+                );
+                throw new InvalidArgumentException(
+                    __('Please set a Default Layout directly on this Display or in CMS Administrator Settings'),
+                    'defaultLayoutId'
+                );
+            }
         }
     }
 
@@ -687,11 +920,10 @@ class Display implements \JsonSerializable
         $options = array_merge([
             'validate' => true,
             'audit' => true,
-            'checkDisplaySlotAvailability' => true
+            'checkDisplaySlotAvailability' => true,
+            'setModifiedDt' => true,
         ], $options);
 
-        $allowNotify = true;
-
         if ($options['validate']) {
             $this->validate();
         }
@@ -710,11 +942,8 @@ class Display implements \JsonSerializable
 
         if ($this->displayId == null || $this->displayId == 0) {
             $this->add();
-
-            // Never notify on add (there is little point, we've only just added).
-            $allowNotify = false;
         } else {
-            $this->edit();
+            $this->edit($options);
         }
 
         if ($options['audit'] && $this->getChangedProperties() != []) {
@@ -723,22 +952,11 @@ class Display implements \JsonSerializable
 
         // Trigger an update of all dynamic DisplayGroups?
         if ($this->hasPropertyChanged('display') || $this->hasPropertyChanged('tags')) {
-            // TODO: Can we trigger a task to run which does this in the background?
-            Profiler::start('Display::save::dynamic', $this->getLog());
-            foreach ($this->displayGroupFactory->getByIsDynamic(1) as $group) {
-                $this->getDispatcher()->dispatch(new DisplayGroupLoadEvent($group), DisplayGroupLoadEvent::$NAME);
-                $group->load();
-                $group->save([
-                    'validate' => false,
-                    'saveGroup' => false,
-                    'saveTags' => false,
-                    'manageLinks' => false,
-                    'manageDisplayLinks' => true,
-                    'manageDynamicDisplayLinks' => true,
-                    'allowNotify' => $allowNotify
-                ]);
-            }
-            Profiler::end('Display::save::dynamic', $this->getLog());
+            // Background update.
+            $this->getDispatcher()->dispatch(
+                new TriggerTaskEvent('\Xibo\XTR\MaintenanceRegularTask', 'DYNAMIC_DISPLAY_GROUP_ASSESSED'),
+                TriggerTaskEvent::$NAME
+            );
         }
     }
 
@@ -750,28 +968,42 @@ class Display implements \JsonSerializable
     {
         $this->load();
 
-        // Delete incidential references
-        $this->getStore()->update('DELETE FROM `requiredfile` WHERE displayId = :displayId', ['displayId' => $this->displayId]);
+        // Delete references
+        $this->getStore()->update('DELETE FROM `display_media` WHERE displayId = :displayId', [
+            'displayId' => $this->displayId
+        ]);
+        $this->getStore()->update('DELETE FROM `requiredfile` WHERE displayId = :displayId', [
+            'displayId' => $this->displayId
+        ]);
+        $this->getStore()->update('DELETE FROM `player_faults` WHERE displayId = :displayId', [
+            'displayId' => $this->displayId
+        ]);
+        $this->getStore()->update('DELETE FROM `schedule_sync` WHERE displayId = :displayId', [
+            'displayId' => $this->displayId
+        ]);
 
         // Remove our display from any groups it is assigned to
         foreach ($this->displayGroups as $displayGroup) {
-            /* @var DisplayGroup $displayGroup */
-            $this->getDispatcher()->dispatch(DisplayGroupLoadEvent::$NAME, new DisplayGroupLoadEvent($displayGroup));
+            $this->getDispatcher()->dispatch(new DisplayGroupLoadEvent($displayGroup), DisplayGroupLoadEvent::$NAME);
             $displayGroup->load();
             $displayGroup->unassignDisplay($this);
-            $displayGroup->save(['validate' => false, 'manageDynamicDisplayLinks' => false]);
+            $displayGroup->save(['validate' => false, 'manageDynamicDisplayLinks' => false, 'allowNotify' => false]);
         }
 
         // Delete our display specific group
         $displayGroup = $this->displayGroupFactory->getById($this->displayGroupId);
-        $this->getDispatcher()->dispatch(DisplayGroupLoadEvent::$NAME, new DisplayGroupLoadEvent($displayGroup));
+        $this->getDispatcher()->dispatch(new DisplayGroupLoadEvent($displayGroup), DisplayGroupLoadEvent::$NAME);
         $displayGroup->delete();
 
         // Delete the display
-        $this->getStore()->update('DELETE FROM `player_faults` WHERE displayId = :displayId', ['displayId' => $this->displayId]);
-        $this->getStore()->update('DELETE FROM `display` WHERE displayId = :displayId', ['displayId' => $this->displayId]);
+        $this->getStore()->update('DELETE FROM `display` WHERE displayId = :displayId', [
+            'displayId' => $this->displayId
+        ]);
 
-        $this->getLog()->audit('Display', $this->displayId, 'Display Deleted', ['displayId' => $this->displayId]);
+        $this->getLog()->audit('Display', $this->displayId, 'Display Deleted', [
+            'displayId' => $this->displayId,
+            'display' => $this->display,
+        ]);
     }
 
     /**
@@ -781,8 +1013,8 @@ class Display implements \JsonSerializable
     private function add()
     {
         $this->displayId = $this->getStore()->insert('
-            INSERT INTO display (display, auditingUntil, defaultlayoutid, license, licensed, lastAccessed, inc_schedule, email_alert, alert_timeout, clientAddress, xmrChannel, xmrPubKey, lastCommandSuccess, macAddress, lastChanged, lastWakeOnLanCommandSent, client_type, client_version, client_code, overrideConfig, newCmsAddress, newCmsKey, commercialLicence)
-              VALUES (:display, :auditingUntil, :defaultlayoutid, :license, :licensed, :lastAccessed, :inc_schedule, :email_alert, :alert_timeout, :clientAddress, :xmrChannel, :xmrPubKey, :lastCommandSuccess, :macAddress, :lastChanged, :lastWakeOnLanCommandSent, :clientType, :clientVersion, :clientCode, :overrideConfig, :newCmsAddress, :newCmsKey, :commercialLicence)
+            INSERT INTO display (display, auditingUntil, defaultlayoutid, license, licensed, lastAccessed, inc_schedule, email_alert, alert_timeout, clientAddress, xmrChannel, xmrPubKey, lastCommandSuccess, macAddress, lastChanged, lastWakeOnLanCommandSent, client_type, client_version, client_code, overrideConfig, newCmsAddress, newCmsKey, commercialLicence, lanIpAddress, syncGroupId, osVersion, osSdk, manufacturer, brand, model)
+              VALUES (:display, :auditingUntil, :defaultlayoutid, :license, :licensed, :lastAccessed, :inc_schedule, :email_alert, :alert_timeout, :clientAddress, :xmrChannel, :xmrPubKey, :lastCommandSuccess, :macAddress, :lastChanged, :lastWakeOnLanCommandSent, :clientType, :clientVersion, :clientCode, :overrideConfig, :newCmsAddress, :newCmsKey, :commercialLicence, :lanIpAddress, :syncGroupId, :osVersion, :osSdk, :manufacturer, :brand, :model)
         ', [
             'display' => $this->display,
             'auditingUntil' => 0,
@@ -806,7 +1038,14 @@ class Display implements \JsonSerializable
             'overrideConfig' => ($this->overrideConfig == '') ? null : json_encode($this->overrideConfig),
             'newCmsAddress' => null,
             'newCmsKey' => null,
-            'commercialLicence' => $this->commercialLicence
+            'commercialLicence' => $this->commercialLicence,
+            'lanIpAddress' => empty($this->lanIpAddress) ? null : $this->lanIpAddress,
+            'syncGroupId' => empty($this->syncGroupId) ? null : $this->syncGroupId,
+            'osVersion' => $this->osVersion,
+            'osSdk' => $this->osSdk,
+            'manufacturer' => $this->manufacturer,
+            'brand' => $this->brand,
+            'model' => $this->model,
         ]);
 
 
@@ -815,8 +1054,29 @@ class Display implements \JsonSerializable
         $displayGroup->tags = $this->tags;
 
         // this is added from xmds, by default new displays will end up in root folder.
-        $displayGroup->folderId = 1;
-        $displayGroup->permissionsFolderId = 1;
+        // Can be overridden per DISPLAY_DEFAULT_FOLDER setting
+        $folderId = $this->folderId ?? 1;
+
+        // If folderId is not set to Root Folder
+        // We need to check what permissionsFolderId should be set on the Display Group
+        if ($folderId !== 1) {
+            // just in case protect against no longer existing Folder.
+            try {
+                $folder = $this->folderFactory->getById($folderId, 0);
+
+                $displayGroup->folderId = $folder->getId();
+                $displayGroup->permissionsFolderId = $folder->getPermissionFolderIdOrThis();
+            } catch (NotFoundException $e) {
+                $this->getLog()->error('Display Default Folder no longer exists');
+
+                // if the Folder from settings no longer exists, default to Root Folder.
+                $displayGroup->folderId = 1;
+                $displayGroup->permissionsFolderId = 1;
+            }
+        } else {
+            $displayGroup->folderId = 1;
+            $displayGroup->permissionsFolderId = 1;
+        }
 
         $displayGroup->setDisplaySpecificDisplay($this);
 
@@ -830,12 +1090,22 @@ class Display implements \JsonSerializable
      * @throws GeneralException
      * @throws NotFoundException
      */
-    private function edit()
+    private function edit($options = [])
     {
         $this->getStore()->update('
             UPDATE display
                 SET display = :display,
                     defaultlayoutid = :defaultLayoutId,
+                    displayTypeId = :displayTypeId,
+                    venueId = :venueId,
+                    address = :address,
+                    isMobile = :isMobile,
+                    languages = :languages,
+                    screenSize = :screenSize,
+                    isOutdoor = :isOutdoor,
+                    `customId` = :customId,
+                    costPerPlay = :costPerPlay,
+                    impressionsPerPlay = :impressionsPerPlay,
                     inc_schedule = :incSchedule,
                     license = :license,
                     licensed = :licensed,
@@ -863,6 +1133,11 @@ class Display implements \JsonSerializable
                     screenShotRequested = :screenShotRequested,
                     storageAvailableSpace = :storageAvailableSpace,
                     storageTotalSpace = :storageTotalSpace,
+                    osVersion = :osVersion,
+                    osSdk = :osSdk,
+                    manufacturer = :manufacturer,
+                    brand = :brand,
+                    model = :model,
                     xmrChannel = :xmrChannel,
                     xmrPubKey = :xmrPubKey,
                     `lastCommandSuccess` = :lastCommandSuccess,
@@ -875,11 +1150,23 @@ class Display implements \JsonSerializable
                     `resolution` = :resolution,
                     `commercialLicence` = :commercialLicence,
                     `teamViewerSerial` = :teamViewerSerial,
-                    `webkeySerial` = :webkeySerial
+                    `webkeySerial` = :webkeySerial,
+                    `lanIpAddress` = :lanIpAddress,
+                    `syncGroupId` = :syncGroupId
              WHERE displayid = :displayId
         ', [
             'display' => $this->display,
             'defaultLayoutId' => $this->defaultLayoutId,
+            'displayTypeId' => $this->displayTypeId === 0 ? null : $this->displayTypeId,
+            'venueId' => $this->venueId === 0 ? null : $this->venueId,
+            'address' => $this->address,
+            'isMobile' => $this->isMobile,
+            'languages' => $this->languages,
+            'screenSize' => $this->screenSize,
+            'isOutdoor' => $this->isOutdoor,
+            'customId' => $this->customId,
+            'costPerPlay' => $this->costPerPlay,
+            'impressionsPerPlay' => $this->impressionsPerPlay,
             'incSchedule' => ($this->incSchedule == null) ? 0 : $this->incSchedule,
             'license' => $this->license,
             'licensed' => $this->licensed,
@@ -921,7 +1208,14 @@ class Display implements \JsonSerializable
             'commercialLicence' => $this->commercialLicence,
             'teamViewerSerial' => empty($this->teamViewerSerial) ? null : $this->teamViewerSerial,
             'webkeySerial' => empty($this->webkeySerial) ? null : $this->webkeySerial,
-            'displayId' => $this->displayId
+            'lanIpAddress' => empty($this->lanIpAddress) ? null : $this->lanIpAddress,
+            'syncGroupId' => empty($this->syncGroupId) ? null : $this->syncGroupId,
+            'displayId' => $this->displayId,
+            'osVersion' => $this->osVersion,
+            'osSdk' => $this->osSdk,
+            'manufacturer' => $this->manufacturer,
+            'brand' => $this->brand,
+            'model' => $this->model,
         ]);
 
         // Maintain the Display Group
@@ -930,6 +1224,11 @@ class Display implements \JsonSerializable
             || $this->hasPropertyChanged('tags')
             || $this->hasPropertyChanged('bandwidthLimit')
             || $this->hasPropertyChanged('folderId')
+            || $this->hasPropertyChanged('ref1')
+            || $this->hasPropertyChanged('ref2')
+            || $this->hasPropertyChanged('ref3')
+            || $this->hasPropertyChanged('ref4')
+            || $this->hasPropertyChanged('ref5')
         ) {
             $this->getLog()->debug('Display specific DisplayGroup properties need updating');
 
@@ -938,18 +1237,24 @@ class Display implements \JsonSerializable
             $displayGroup->displayGroup = $this->display;
             $displayGroup->description = $this->description;
             $displayGroup->bandwidthLimit = $this->bandwidthLimit;
+            $displayGroup->ref1 = $this->ref1;
+            $displayGroup->ref2 = $this->ref2;
+            $displayGroup->ref3 = $this->ref3;
+            $displayGroup->ref4 = $this->ref4;
+            $displayGroup->ref5 = $this->ref5;
 
             // Tags
             $saveTags = false;
             if ($this->hasPropertyChanged('tags')) {
                 $saveTags = true;
-                $displayGroup->replaceTags($this->tags);
+                $displayGroup->updateTagLinks($this->tags);
             }
 
             // If the folderId has changed, we should check this user has permissions to the new folderId
+            // it shouldn't ever be null, but just in case.
             $displayGroup->folderId = ($this->folderId == null) ? 1 : $this->folderId;
             if ($this->hasPropertyChanged('folderId')) {
-                $folder = $this->folderFactory->getById($displayGroup->folderId);
+                $folder = $this->folderFactory->getById($displayGroup->folderId, 0);
                 // We have permission, so assert the new folder's permission id
                 $displayGroup->permissionsFolderId = $folder->getPermissionFolderIdOrThis();
             }
@@ -962,10 +1267,16 @@ class Display implements \JsonSerializable
                 'manageDisplayLinks' => false,
                 'manageDynamicDisplayLinks' => false,
                 'allowNotify' => true,
-                'saveTags' => $saveTags
+                'saveTags' => $saveTags,
+                'setModifiedDt' => $options['setModifiedDt'],
             ]);
-        } else {
-            $this->store->update('UPDATE displaygroup SET `modifiedDt` = :modifiedDt WHERE displayGroupId = :displayGroupId', [
+        } else if ($options['setModifiedDt']) {
+            // Bump the modified date.
+            $this->store->update('
+                UPDATE displaygroup 
+                    SET `modifiedDt` = :modifiedDt
+                 WHERE displayGroupId = :displayGroupId
+            ', [
                 'modifiedDt' => Carbon::now()->format(DateFormatHelper::getSystemFormat()),
                 'displayGroupId' => $this->displayGroupId
             ]);
@@ -1056,7 +1367,7 @@ class Display implements \JsonSerializable
      * @return array
      * @throws NotFoundException
      */
-    private function setConfig($options = [])
+    private function setConfig(array $options = []): array
     {
         $options = array_merge([
             'displayOverride' => false
@@ -1066,7 +1377,11 @@ class Display implements \JsonSerializable
             $this->load();
 
             // Get the display profile
-            $displayProfile = $this->getDisplayProfile();
+            try {
+                $displayProfile = $this->getDisplayProfile();
+            } catch (NotFoundException) {
+                $displayProfile = $this->displayProfileFactory->getUnknownProfile($this->clientType);
+            }
 
             // Merge in any overrides we have on our display.
             $this->profileConfig = $displayProfile->getProfileConfig();
@@ -1082,29 +1397,35 @@ class Display implements \JsonSerializable
      * @param $override
      * @return array
      */
-    private function mergeConfigs($default, $override)
+    private function mergeConfigs($default, $override): array
     {
+        // No overrides, then nothing to do.
+        if (empty($override) || !is_array($override)) {
+            return $default;
+        }
 
+        // Merge the settings together
         foreach ($default as &$defaultItem) {
             for ($i = 0; $i < count($override); $i++) {
                 if ($defaultItem['name'] == $override[$i]['name']) {
-
                     // For special json fields, we need to decode, merge, encode and save instead
-                    if(in_array($defaultItem['name'], ['timers', 'pictureOptions', 'lockOptions']) && isset($defaultItem['value']) && isset($override[$i]['value'])) {
-
+                    if (in_array($defaultItem['name'], ['timers', 'pictureOptions', 'lockOptions'])
+                        && isset($defaultItem['value']) && isset($override[$i]['value'])
+                    ) {
                         // Decode values
                         $defaultItemValueDecoded = json_decode($defaultItem['value'], true);
                         $overrideValueDecoded = json_decode($override[$i]['value'], true);
 
                         // Merge values, encode and save
-                        $defaultItem['value'] = json_encode(array_merge($defaultItemValueDecoded, $overrideValueDecoded));
-                        break;
+                        $defaultItem['value'] = json_encode(array_merge(
+                            $defaultItemValueDecoded,
+                            $overrideValueDecoded
+                        ));
                     } else {
                         // merge
                         $defaultItem = array_merge($defaultItem, $override[$i]);
-                        break;
                     }
-                    
+                    break;
                 }
             }
         }
@@ -1228,4 +1549,18 @@ class Display implements \JsonSerializable
 
         return $this;
     }
+
+    /**
+     * Check if this Display is set as Lead Display on any Sync Group
+     * @return bool
+     */
+    public function isLead(): bool
+    {
+        $syncGroups = $this->getStore()->select(
+            'SELECT syncGroupId FROM `syncgroup` WHERE `syncgroup`.leadDisplayId = :displayId',
+            ['displayId' => $this->displayId]
+        );
+
+        return count($syncGroups) > 0;
+    }
 }
