@@ -1,8 +1,8 @@
 <?php
 /*
- * Copyright (c) 2022 Xibo Signage Ltd
+ * Copyright (C) 2025 Xibo Signage Ltd
  *
- * Xibo - Digital Signage - http://www.xibo.org.uk
+ * Xibo - Digital Signage - https://xibosignage.com
  *
  * This file is part of Xibo.
  *
@@ -21,6 +21,7 @@
  */
 namespace Xibo\Entity;
 
+use Carbon\Carbon;
 use Respect\Validation\Validator as v;
 use Xibo\Factory\CommandFactory;
 use Xibo\Factory\DisplayProfileFactory;
@@ -28,6 +29,7 @@ use Xibo\Service\ConfigServiceInterface;
 use Xibo\Service\LogServiceInterface;
 use Xibo\Storage\StorageServiceInterface;
 use Xibo\Support\Exception\InvalidArgumentException;
+use Xibo\Support\Exception\NotFoundException;
 
 
 /**
@@ -160,7 +162,7 @@ class DisplayProfile implements \JsonSerializable
      * @return mixed
      * @throws \Xibo\Support\Exception\NotFoundException
      */
-    public function getSetting($setting, $default = null, $fromDefault = false)
+    public function getSetting($setting, $default = null, $fromDefault = false): mixed
     {
         $this->load();
 
@@ -168,7 +170,7 @@ class DisplayProfile implements \JsonSerializable
 
         foreach ($configs as $config) {
             if ($config['name'] == $setting || $config['name'] == ucfirst($setting)) {
-                $default = array_key_exists('value', $config) ? $config['value'] : ((array_key_exists('default', $config)) ? $config['default'] : $default);
+                $default = $config['value'] ?? ($config['default'] ?? $default);
                 break;
             }
         }
@@ -245,7 +247,7 @@ class DisplayProfile implements \JsonSerializable
      * @param $override
      * @return array
      */
-    private function mergeConfigs($default, $override)
+    private function mergeConfigs($default, $override): array
     {
         foreach ($default as &$defaultItem) {
             for ($i = 0; $i < count($override); $i++) {
@@ -302,13 +304,15 @@ class DisplayProfile implements \JsonSerializable
             if ($alreadyAssigned->getId() == $command->getId()) {
                 $alreadyAssigned->commandString = $command->commandString;
                 $alreadyAssigned->validationString = $command->validationString;
+                $alreadyAssigned->createAlertOn = $command->createAlertOn;
                 $assigned = true;
                 break;
             }
         }
 
-        if (!$assigned)
+        if (!$assigned) {
             $this->commands[] = $command;
+        }
     }
 
     /**
@@ -343,8 +347,11 @@ class DisplayProfile implements \JsonSerializable
      * @param array $options
      * @throws \Xibo\Support\Exception\NotFoundException
      */
-    public function load($options = [])
+    public function load($options = []): void
     {
+        $this->getLog()->debug('load: Loading display profile, type: ' . $this->clientType
+            . ' id: ' . $this->displayProfileId);
+
         $options = array_merge([
             'loadConfig' => true,
             'loadCommands' => true
@@ -357,16 +364,19 @@ class DisplayProfile implements \JsonSerializable
         // Load in our default config from this class, based on the client type we are
         $this->configDefault = $this->displayProfileFactory->loadForType($this->getClientType());
 
-        $this->getLog()->debug('Config Default is: ' . json_encode($this->configDefault, JSON_PRETTY_PRINT) . ' for ' . $this->getClientType());
-
         // Get our combined config
         $this->configCombined = [];
 
         if ($options['loadConfig']) {
-            // Decode the config string (unless its already an array)
-            if (!is_array($this->config)) {
+            if (!is_array($this->config) && !empty($this->config)) {
                 $this->config = json_decode($this->config, true);
             }
+
+            // handle cases when config is empty
+            if (empty($this->config)) {
+                $this->config = [];
+            }
+
             $this->getLog()->debug('Config loaded: ' . json_encode($this->config, JSON_PRETTY_PRINT));
 
             // Populate our combined config accordingly
@@ -476,22 +486,41 @@ class DisplayProfile implements \JsonSerializable
         foreach ($this->commands as $command) {
             /* @var Command $command */
             $this->getStore()->update('
-              INSERT INTO `lkcommanddisplayprofile` (`commandId`, `displayProfileId`, `commandString`, `validationString`) VALUES
-                (:commandId, :displayProfileId, :commandString, :validationString) ON DUPLICATE KEY UPDATE commandString = :commandString2, validationString = :validationString2
+              INSERT INTO `lkcommanddisplayprofile` (
+                  `commandId`,
+                  `displayProfileId`,
+                  `commandString`,
+                  `validationString`,
+                  `createAlertOn`
+              )
+              VALUES (
+                  :commandId,
+                  :displayProfileId,
+                  :commandString,
+                  :validationString,
+                  :createAlertOn    
+              )
+              ON DUPLICATE KEY UPDATE 
+                  commandString = :commandString2,
+                  validationString = :validationString2,
+                  createAlertOn = :createAlertOn2
             ', [
                 'commandId' => $command->commandId,
                 'displayProfileId' => $this->displayProfileId,
                 'commandString' => $command->commandString,
                 'validationString' => $command->validationString,
+                'createAlertOn' => $command->createAlertOn,
                 'commandString2' => $command->commandString,
-                'validationString2' => $command->validationString
+                'validationString2' => $command->validationString,
+                'createAlertOn2' => $command->createAlertOn
             ]);
         }
 
         // Unlink
         $params = ['displayProfileId' => $this->displayProfileId];
 
-        $sql = 'DELETE FROM `lkcommanddisplayprofile` WHERE `displayProfileId` = :displayProfileId AND `commandId` NOT IN (0';
+        $sql = 'DELETE FROM `lkcommanddisplayprofile`
+                WHERE `displayProfileId` = :displayProfileId AND `commandId` NOT IN (0';
 
         $i = 0;
         foreach ($this->commands as $command) {
@@ -539,7 +568,7 @@ class DisplayProfile implements \JsonSerializable
     /**
      * @return array
      */
-    public function getProfileConfig()
+    public function getProfileConfig(): array
     {
         return $this->configCombined;
     }
@@ -549,7 +578,10 @@ class DisplayProfile implements \JsonSerializable
         if ($this->isCustom()) {
             return $this->displayProfileFactory->getCustomEditTemplate($this->getClientType());
         } else {
-            $this->getLog()->error('Attempting to get Custom Edit template for Display Profile ' . $this->getClientType() . ' that is not custom');
+            $this->getLog()->error(
+                'Attempting to get Custom Edit template for Display Profile ' .
+                $this->getClientType() . ' that is not custom'
+            );
             return null;
         }
     }
@@ -558,4 +590,22 @@ class DisplayProfile implements \JsonSerializable
     {
         return $this->displayProfileFactory->handleCustomFields($this, $sanitizedParams, $config, $display);
     }
+
+    /**
+     * Does this display profile has elevated log level?
+     * @return bool
+     * @throws NotFoundException
+     */
+    public function isElevatedLogging(): bool
+    {
+        $elevatedUntil = $this->getSetting('elevateLogsUntil', 0);
+
+        $this->getLog()->debug(sprintf(
+            'Testing whether this display profile has elevated log level. %d vs %d.',
+            $elevatedUntil,
+            Carbon::now()->format('U')
+        ));
+
+        return (!empty($elevatedUntil) && $elevatedUntil >= Carbon::now()->format('U'));
+    }
 }
