@@ -1,8 +1,8 @@
 <?php
 /*
- * Copyright (c) 2022 Xibo Signage Ltd
+ * Copyright (C) 2025 Xibo Signage Ltd
  *
- * Xibo - Digital Signage - http://www.xibo.org.uk
+ * Xibo - Digital Signage - https://xibosignage.com
  *
  * This file is part of Xibo.
  *
@@ -167,6 +167,16 @@ class DataSetFactory extends BaseFactory
         return $dataSets;
     }
 
+    /**
+     * @param int $folderId
+     * @return DataSet[]
+     * @throws NotFoundException
+     */
+    public function getByFolderId(int $folderId)
+    {
+        return $this->query(null, ['disableUserCheck' => 1, 'folderId' => $folderId]);
+    }
+
     /**
      * @param array $sortOrder
      * @param array $filterBy
@@ -192,6 +202,8 @@ class DataSetFactory extends BaseFactory
             dataset.`code`,
             dataset.`isLookup`,
             dataset.`isRemote`,
+            dataset.`isRealTime`,
+            dataset.`dataConnectorSource`,
             dataset.`method`,
             dataset.`uri`,
             dataset.`postData`,
@@ -252,9 +264,23 @@ class DataSetFactory extends BaseFactory
             $params['isRemote'] = $parsedFilter->getInt('isRemote');
         }
 
+        if ($parsedFilter->getInt('isRealTime') !== null) {
+            $body .= ' AND dataset.isRealTime = :isRealTime ';
+            $params['isRealTime'] = $parsedFilter->getInt('isRealTime');
+        }
+
         if ($parsedFilter->getString('dataSet') != null) {
             $terms = explode(',', $parsedFilter->getString('dataSet'));
-            $this->nameFilter('dataset', 'dataSet', $terms, $body, $params, ($parsedFilter->getCheckbox('useRegexForName') == 1));
+            $logicalOperator = $parsedFilter->getString('logicalOperatorName', ['default' => 'OR']);
+            $this->nameFilter(
+                'dataset',
+                'dataSet',
+                $terms,
+                $body,
+                $params,
+                ($parsedFilter->getCheckbox('useRegexForName') == 1),
+                $logicalOperator
+            );
         }
 
         if ($parsedFilter->getString('dataSetExact') != '') {
@@ -290,7 +316,18 @@ class DataSetFactory extends BaseFactory
 
         foreach ($this->getStore()->select($sql, $params) as $row) {
             $entries[] = $this->createEmpty()->hydrate($row, [
-                'intProperties' => ['isLookup', 'isRemote', 'clearRate', 'refreshRate', 'lastDataEdit', 'runsAfter', 'lastSync', 'lastClear', 'ignoreFirstRow']
+                'intProperties' => [
+                    'isLookup',
+                    'isRemote',
+                    'isRealTime',
+                    'clearRate',
+                    'refreshRate',
+                    'lastDataEdit',
+                    'runsAfter',
+                    'lastSync',
+                    'lastClear',
+                    'ignoreFirstRow',
+                ],
             ]);
         }
 
@@ -314,9 +351,10 @@ class DataSetFactory extends BaseFactory
      * @throws InvalidArgumentException
      * @throws NotFoundException
      */
-    public function callRemoteService(DataSet $dataSet, DataSet $dependant = null, $enableCaching = true)
+    public function callRemoteService(DataSet $dataSet, ?DataSet $dependant = null, $enableCaching = true)
     {
-        $this->getLog()->debug('Calling remote service for DataSet: ' . $dataSet->dataSet . ' and URL ' . $dataSet->uri);
+        $this->getLog()->debug('Calling remote service for DataSet: ' . $dataSet->dataSet
+            . ' and URL ' . $dataSet->uri);
 
         // Record our max memory
         $maxMemory = Environment::getMemoryLimitBytes() / 2;
@@ -328,7 +366,7 @@ class DataSetFactory extends BaseFactory
         $result->entries = [];
         $result->number = 0;
         $result->isEligibleToTruncate = false;
-        
+
         // Getting all dependant values if needed
         // just an empty array if we don't have a dependent
         $values = [
@@ -340,7 +378,7 @@ class DataSetFactory extends BaseFactory
 
             $values = $dependant->getData();
         }
-        
+
         // Fetching data for every field in the dependant dataSet
         foreach ($values as $options) {
             // Make some request params to provide to the HTTP client
@@ -383,7 +421,13 @@ class DataSetFactory extends BaseFactory
             if ($dataSet->method === 'POST') {
                 parse_str($this->replaceParams($dataSet->postData, $options), $requestParams['form_params']);
             } else {
-                parse_str(parse_url($resolvedUri, PHP_URL_QUERY), $queryParamsArray);
+                // Get the query params from the URL.
+                $queryParamsArray = [];
+                $dataSetPostData = [];
+                $parsedUrl = parse_url($resolvedUri, PHP_URL_QUERY);
+                if ($parsedUrl) {
+                    parse_str($parsedUrl, $queryParamsArray);
+                }
                 parse_str($this->replaceParams($dataSet->postData, $options), $dataSetPostData);
                 $requestParams['query'] = array_merge($queryParamsArray, $dataSetPostData);
             }
@@ -498,6 +542,14 @@ class DataSetFactory extends BaseFactory
                         array_shift($array);
                     }
 
+                    // Filter out rows that are entirely empty
+                    $array = array_filter($array, function($row) {
+                        // Check if the row is empty (all elements are empty or null)
+                        return array_filter($row, function($value) {
+                            return !empty($value);
+                        });
+                    });
+
                     $result->entries = $array;
                     $result->number = count($array);
                 }
@@ -509,7 +561,7 @@ class DataSetFactory extends BaseFactory
                 throw new InvalidArgumentException(__('Unable to get Data for %s because %s.', $dataSet->dataSet, $requestException->getMessage()), 'dataSetId');
             }
         }
-        
+
         return $result;
     }
 
@@ -737,11 +789,7 @@ class DataSetFactory extends BaseFactory
                             break;
                         default:
                             // Default value, assume it will be a string and filter it accordingly.
-                            $result[$column->heading] = filter_var(
-                                $value[1],
-                                FILTER_SANITIZE_STRING,
-                                FILTER_FLAG_NO_ENCODE_QUOTES
-                            );
+                            $result[$column->heading] = strip_tags($value[1]);
                     }
                 }
             } else {
