@@ -1,6 +1,6 @@
 <?php
 /*
- * Copyright (C) 2023 Xibo Signage Ltd
+ * Copyright (C) 2025 Xibo Signage Ltd
  *
  * Xibo - Digital Signage - https://xibosignage.com
  *
@@ -23,6 +23,7 @@ namespace Xibo\Factory;
 
 use Xibo\Entity\DisplayProfile;
 use Xibo\Service\ConfigServiceInterface;
+use Xibo\Support\Exception\GeneralException;
 use Xibo\Support\Exception\InvalidArgumentException;
 use Xibo\Support\Exception\NotFoundException;
 use Xibo\Support\Sanitizer\SanitizerInterface;
@@ -98,16 +99,15 @@ class DisplayProfileFactory extends BaseFactory
      * @return DisplayProfile
      * @throws NotFoundException
      */
-    public function getDefaultByType($type)
+    public function getDefaultByType(string $type): DisplayProfile
     {
         $profiles = $this->query(null, ['disableUserCheck' => 1, 'type' => $type, 'isDefault' => 1]);
 
         if (count($profiles) <= 0) {
-            throw new NotFoundException();
+            throw new NotFoundException(sprintf(__('No default display profile for %s'), $type));
         }
 
         $profile = $profiles[0];
-        /* @var DisplayProfile $profile */
         $profile->load();
         return $profile;
     }
@@ -117,13 +117,16 @@ class DisplayProfileFactory extends BaseFactory
      * @return DisplayProfile
      * @throws NotFoundException
      */
-    public function getUnknownProfile($clientType)
+    public function getUnknownProfile($clientType): DisplayProfile
     {
         $profile = $this->createEmpty();
         $profile->type = 'unknown';
         $profile->setClientType($clientType);
         $profile->isCustom = 0;
-        $profile->load();
+        $profile->config = [];
+        $profile->configDefault = [];
+        // We shoud not call load as it would be recursive due to unknown also being an unknown profile
+        // $profile->load();
         return $profile;
     }
 
@@ -169,6 +172,7 @@ class DisplayProfileFactory extends BaseFactory
                 ['name' => 'downloadEndWindow', 'default' => '00:00', 'type' => 'string'],
                 ['name' => 'dayPartId', 'default' => null],
                 ['name' => 'xmrNetworkAddress', 'default' => '', 'type' => 'string'],
+                ['name' => 'xmrWebSocketAddress', 'default' => '', 'type' => 'string'],
                 [
                     'name' => 'statsEnabled',
                     'default' => (int)$this->config->getSetting('DISPLAY_PROFILE_STATS_DEFAULT', 0),
@@ -187,6 +191,7 @@ class DisplayProfileFactory extends BaseFactory
                 ['name' => 'clientInfomationCtrlKey', 'default' => 0, 'type' => 'checkbox'],
                 ['name' => 'clientInformationKeyCode', 'default' => 'I', 'type' => 'string'],
                 ['name' => 'logLevel', 'default' => 'error', 'type' => 'string'],
+                ['name' => 'elevateLogsUntil', 'default' => 0, 'type' => 'int'],
                 ['name' => 'logToDiskLocation', 'default' => '', 'type' => 'string'],
                 ['name' => 'showInTaskbar', 'default' => 1, 'type' => 'checkbox'],
                 ['name' => 'cursorStartPosition', 'default' => 'Unchanged', 'type' => 'string'],
@@ -220,6 +225,7 @@ class DisplayProfileFactory extends BaseFactory
                 ['name' => 'downloadStartWindow', 'default' => '00:00'],
                 ['name' => 'downloadEndWindow', 'default' => '00:00'],
                 ['name' => 'xmrNetworkAddress', 'default' => ''],
+                ['name' => 'xmrWebSocketAddress', 'default' => ''],
                 [
                     'name' => 'statsEnabled',
                     'default' => (int)$this->config->getSetting('DISPLAY_PROFILE_STATS_DEFAULT', 0),
@@ -236,6 +242,7 @@ class DisplayProfileFactory extends BaseFactory
                 ['name' => 'storeHtmlOnInternal', 'default' => 0, 'type' => 'checkbox'],
                 ['name' => 'useSurfaceVideoView', 'default' => 1, 'type' => 'checkbox'],
                 ['name' => 'logLevel', 'default' => 'error'],
+                ['name' => 'elevateLogsUntil', 'default' => 0, 'type' => 'int'],
                 ['name' => 'versionMediaId', 'default' => null],
                 ['name' => 'startOnBoot', 'default' => 1, 'type' => 'checkbox'],
                 ['name' => 'actionBarMode', 'default' => 1],
@@ -254,6 +261,7 @@ class DisplayProfileFactory extends BaseFactory
                 ['name' => 'updateStartWindow', 'default' => '00:00'],
                 ['name' => 'updateEndWindow', 'default' => '00:00'],
                 ['name' => 'dayPartId', 'default' => null],
+                ['name' => 'restartWifiOnConnectionFailure', 'default' => 0, 'type' => 'checkbox'],
                 ['name' => 'webViewPluginState', 'default' => 'DEMAND'],
                 ['name' => 'hardwareAccelerateWebViewMode', 'default' => '2'],
                 ['name' => 'timeSyncFromCms', 'default' => 0, 'type' => 'checkbox'],
@@ -274,6 +282,7 @@ class DisplayProfileFactory extends BaseFactory
                 ['name' => 'downloadEndWindow', 'default' => '00:00'],
                 ['name' => 'dayPartId', 'default' => null],
                 ['name' => 'xmrNetworkAddress', 'default' => ''],
+                ['name' => 'xmrWebSocketAddress', 'default' => ''],
                 [
                     'name' => 'statsEnabled',
                     'default' => (int)$this->config->getSetting('DISPLAY_PROFILE_STATS_DEFAULT', 0),
@@ -289,6 +298,7 @@ class DisplayProfileFactory extends BaseFactory
                 ['name' => 'offsetX', 'default' => 0],
                 ['name' => 'offsetY', 'default' => 0],
                 ['name' => 'logLevel', 'default' => 'error'],
+                ['name' => 'elevateLogsUntil', 'default' => 0, 'type' => 'int'],
                 ['name' => 'enableShellCommands', 'default' => 0, 'type' => 'checkbox'],
                 ['name' => 'expireModifiedLayouts', 'default' => 0, 'type' => 'checkbox'],
                 ['name' => 'maxConcurrentDownloads', 'default' => 2],
@@ -312,6 +322,7 @@ class DisplayProfileFactory extends BaseFactory
                 ['name' => 'downloadEndWindow', 'default' => '00:00'],
                 ['name' => 'dayPartId', 'default' => null],
                 ['name' => 'xmrNetworkAddress', 'default' => ''],
+                ['name' => 'xmrWebSocketAddress', 'default' => ''],
                 [
                     'name' => 'statsEnabled',
                     'default' => (int)$this->config->getSetting('DISPLAY_PROFILE_STATS_DEFAULT', 0),
@@ -324,6 +335,7 @@ class DisplayProfileFactory extends BaseFactory
                 ],
                 ['name' => 'orientation', 'default' => 0],
                 ['name' => 'logLevel', 'default' => 'error'],
+                ['name' => 'elevateLogsUntil', 'default' => 0, 'type' => 'int'],
                 ['name' => 'versionMediaId', 'default' => null],
                 ['name' => 'actionBarMode', 'default' => 1],
                 ['name' => 'actionBarDisplayDuration', 'default' => 30],
@@ -339,6 +351,7 @@ class DisplayProfileFactory extends BaseFactory
                 ['name' => 'updateEndWindow', 'default' => '00:00'],
                 ['name' => 'embeddedServerAllowWan', 'default' => 0, 'type' => 'checkbox'],
                 ['name' => 'serverPort', 'default' => 9696],
+                ['name' => 'isUseMultipleVideoDecoders', 'default' => 'default', 'type' => 'string'],
             ],
             'sssp' => [
                 ['name' => 'emailAddress', 'default' => ''],
@@ -347,6 +360,7 @@ class DisplayProfileFactory extends BaseFactory
                 ['name' => 'downloadEndWindow', 'default' => '00:00'],
                 ['name' => 'dayPartId', 'default' => null],
                 ['name' => 'xmrNetworkAddress', 'default' => ''],
+                ['name' => 'xmrWebSocketAddress', 'default' => ''],
                 [
                     'name' => 'statsEnabled',
                     'default' => (int)$this->config->getSetting('DISPLAY_PROFILE_STATS_DEFAULT', 0),
@@ -359,6 +373,7 @@ class DisplayProfileFactory extends BaseFactory
                 ],
                 ['name' => 'orientation', 'default' => 0],
                 ['name' => 'logLevel', 'default' => 'error'],
+                ['name' => 'elevateLogsUntil', 'default' => 0, 'type' => 'int'],
                 ['name' => 'versionMediaId', 'default' => null],
                 ['name' => 'actionBarMode', 'default' => 1],
                 ['name' => 'actionBarDisplayDuration', 'default' => 30],
@@ -374,6 +389,33 @@ class DisplayProfileFactory extends BaseFactory
                 ['name' => 'updateEndWindow', 'default' => '00:00'],
                 ['name' => 'embeddedServerAllowWan', 'default' => 0, 'type' => 'checkbox'],
                 ['name' => 'serverPort', 'default' => 9696],
+                ['name' => 'isUseMultipleVideoDecoders', 'default' => 'default', 'type' => 'string'],
+            ],
+            'chromeOS' => [
+                ['name' => 'licenceCode', 'default' => ''],
+                ['name' => 'collectInterval', 'default' => 300],
+                ['name' => 'xmrNetworkAddress', 'default' => ''],
+                ['name' => 'xmrWebSocketAddress', 'default' => ''],
+                [
+                    'name' => 'statsEnabled',
+                    'default' => (int)$this->config->getSetting('DISPLAY_PROFILE_STATS_DEFAULT', 0),
+                    'type' => 'checkbox',
+                ],
+                [
+                    'name' => 'aggregationLevel',
+                    'default' => $this->config->getSetting('DISPLAY_PROFILE_AGGREGATION_LEVEL_DEFAULT'),
+                    'type' => 'string',
+                ],
+                ['name' => 'playerVersionId', 'default' => null],
+                ['name' => 'dayPartId', 'default' => null],
+                ['name' => 'logLevel', 'default' => 'error'],
+                ['name' => 'elevateLogsUntil', 'default' => 0, 'type' => 'int'],
+                ['name' => 'sendCurrentLayoutAsStatusUpdate', 'default' => 0, 'type' => 'checkbox'],
+                ['name' => 'screenShotRequestInterval', 'default' => 0, 'type' => 'int'],
+                [
+                    'name' => 'screenShotSize',
+                    'default' => (int)$this->config->getSetting('DISPLAY_PROFILE_SCREENSHOT_SIZE_DEFAULT', 200),
+                ],
             ]
         ];
 
@@ -392,7 +434,12 @@ class DisplayProfileFactory extends BaseFactory
         }
 
         if (!isset($config[$type])) {
-            $config[$type] = $this->getCustomProfileConfig($type);
+            try {
+                $config[$type] = $this->getCustomProfileConfig($type);
+            } catch (GeneralException $exception) {
+                $this->getLog()->error('loadForType: error with ' . $type . ', e = ' . $exception->getMessage());
+                $config[$type] = $this->getUnknownProfile($type)->getProfileConfig();
+            }
         }
 
         return $config[$type];
@@ -498,7 +545,11 @@ class DisplayProfileFactory extends BaseFactory
         return $profiles;
     }
 
-    public function getAvailableTypes()
+    /**
+     * Called by the Display Profile page and Add form
+     * @return array
+     */
+    public function getAvailableTypes(): array
     {
         // get distinct player types from the displayprofile table, this will include any custom profile types as well
         $dbTypes = $this->getStore()->select('SELECT DISTINCT type FROM `displayprofile` ORDER BY type', []);
