@@ -1,6 +1,6 @@
 <?php
 /*
- * Copyright (C) 2023 Xibo Signage Ltd
+ * Copyright (C) 2024 Xibo Signage Ltd
  *
  * Xibo - Digital Signage - https://xibosignage.com
  *
@@ -26,6 +26,7 @@ use Illuminate\Support\Str;
 use Xibo\Entity\Module;
 use Xibo\Widget\Definition\Asset;
 use Xibo\Widget\Definition\Element;
+use Xibo\Widget\Definition\ElementGroup;
 use Xibo\Widget\Definition\Extend;
 use Xibo\Widget\Definition\LegacyType;
 use Xibo\Widget\Definition\PlayerCompatibility;
@@ -76,6 +77,8 @@ trait ModuleXmlTrait
                     $stencil->height = doubleval($childNode->textContent);
                 } else if ($childNode->nodeName === 'gapBetweenHbs') {
                     $stencil->gapBetweenHbs = doubleval($childNode->textContent);
+                } else if ($childNode->nodeName === 'elementGroups') {
+                    $stencil->elementGroups = $this->parseElementGroups($childNode->childNodes);
                 }
             }
 
@@ -122,12 +125,21 @@ trait ModuleXmlTrait
                 $property->allowLibraryRefs = $node->getAttribute('allowLibraryRefs') === 'true';
                 $property->allowAssetRefs = $node->getAttribute('allowAssetRefs') === 'true';
                 $property->parseTranslations = $node->getAttribute('parseTranslations') === 'true';
-                $property->includeInXlf = $node->getAttribute('includeInXlf') === 'true';
+                $property->saveDefault = $node->getAttribute('saveDefault') === 'true';
+                $property->sendToElements = $node->getAttribute('sendToElements') === 'true';
                 $property->title = __($this->getFirstValueOrDefaultFromXmlNode($node, 'title'));
                 $property->helpText = __($this->getFirstValueOrDefaultFromXmlNode($node, 'helpText'));
-                $property->value = $this->getFirstValueOrDefaultFromXmlNode($node, 'value');
                 $property->dependsOn = $this->getFirstValueOrDefaultFromXmlNode($node, 'dependsOn');
 
+                // How should we default includeInXlf?
+                if ($module?->renderAs === 'native') {
+                    // Include by default
+                    $property->includeInXlf = $node->getAttribute('includeInXlf') !== 'false';
+                } else {
+                    // Exclude by default
+                    $property->includeInXlf = $node->getAttribute('includeInXlf') === 'true';
+                }
+
                 // Default value
                 $defaultValue = $this->getFirstValueOrDefaultFromXmlNode($node, 'default');
 
@@ -227,11 +239,24 @@ trait ModuleXmlTrait
                         /** @var \DOMElement $playerCompat */
                         $playerCompatibility = new PlayerCompatibility();
                         $playerCompatibility->message = $playerCompat->textContent;
-                        $playerCompatibility->windows = $playerCompat->getAttribute('windows');
-                        $playerCompatibility->android = $playerCompat->getAttribute('android');
-                        $playerCompatibility->linux = $playerCompat->getAttribute('linux');
-                        $playerCompatibility->webos = $playerCompat->getAttribute('webos');
-                        $playerCompatibility->tizen = $playerCompat->getAttribute('tizen');
+                        if ($playerCompat->hasAttribute('windows')) {
+                            $playerCompatibility->windows = $playerCompat->getAttribute('windows');
+                        }
+                        if ($playerCompat->hasAttribute('android')) {
+                            $playerCompatibility->android = $playerCompat->getAttribute('android');
+                        }
+                        if ($playerCompat->hasAttribute('linux')) {
+                            $playerCompatibility->linux = $playerCompat->getAttribute('linux');
+                        }
+                        if ($playerCompat->hasAttribute('webos')) {
+                            $playerCompatibility->webos = $playerCompat->getAttribute('webos');
+                        }
+                        if ($playerCompat->hasAttribute('tizen')) {
+                            $playerCompatibility->tizen = $playerCompat->getAttribute('tizen');
+                        }
+                        if ($playerCompat->hasAttribute('chromeos')) {
+                            $playerCompatibility->chromeos = $playerCompat->getAttribute('chromeos');
+                        }
                         $property->playerCompatibility = $playerCompatibility;
                     }
                 }
@@ -318,6 +343,7 @@ trait ModuleXmlTrait
             if ($elementNode instanceof \DOMElement) {
                 $element = new Element();
                 $element->id = $elementNode->getAttribute('id');
+                $element->elementGroupId = $elementNode->getAttribute('elementGroupId');
                 foreach ($elementNode->childNodes as $childNode) {
                     if ($childNode instanceof \DOMElement) {
                         if ($childNode->nodeName === 'top') {
@@ -351,6 +377,46 @@ trait ModuleXmlTrait
         return $elements;
     }
 
+    /**
+     * @param \DOMNodeList $elementGroupsNodes
+     * @return \Xibo\Widget\Definition\Property[]
+     */
+    private function parseElementGroups (\DOMNodeList $elementGroupsNodes): array
+    {
+        $elementGroups = [];
+        foreach ($elementGroupsNodes as $elementGroupsNode) {
+            /** @var \DOMNode $elementNode */
+            if ($elementGroupsNode instanceof \DOMElement) {
+                $elementGroup = new ElementGroup();
+                $elementGroup->id = $elementGroupsNode->getAttribute('id');
+                foreach ($elementGroupsNode->childNodes as $childNode) {
+                    if ($childNode instanceof \DOMElement) {
+                        if ($childNode->nodeName === 'top') {
+                            $elementGroup->top = doubleval($childNode->textContent);
+                        } else if ($childNode->nodeName === 'left') {
+                            $elementGroup->left = doubleval($childNode->textContent);
+                        } else if ($childNode->nodeName === 'width') {
+                            $elementGroup->width = doubleval($childNode->textContent);
+                        } else if ($childNode->nodeName === 'height') {
+                            $elementGroup->height = doubleval($childNode->textContent);
+                        } else if ($childNode->nodeName === 'layer') {
+                            $elementGroup->layer = intval($childNode->textContent);
+                        } else if ($childNode->nodeName === 'title') {
+                            $elementGroup->title = $childNode->textContent;
+                        } else if ($childNode->nodeName === 'slot') {
+                            $elementGroup->slot = intval($childNode->textContent);
+                        } else if ($childNode->nodeName === 'pinSlot') {
+                            $elementGroup->pinSlot = boolval($childNode->textContent);
+                        }
+                    }
+                }
+                $elementGroups[] = $elementGroup;
+            }
+        }
+
+        return $elementGroups;
+    }
+
     /**
      * @param \DOMNodeList $legacyTypeNodes
      * @return \Xibo\Widget\Definition\LegacyType[]
