@@ -1,6 +1,6 @@
 <?php
 /*
- * Copyright (C) 2023 Xibo Signage Ltd
+ * Copyright (C) 2024 Xibo Signage Ltd
  *
  * Xibo - Digital Signage - https://xibosignage.com
  *
@@ -78,7 +78,8 @@ class ScheduleFactory extends BaseFactory
         $userFactory,
         $scheduleReminderFactory,
         $scheduleExclusionFactory,
-        $user
+        $user,
+        private readonly ScheduleCriteriaFactory $scheduleCriteriaFactory
     ) {
         $this->setAclDependencies($user, $userFactory);
         $this->config = $config;
@@ -106,7 +107,8 @@ class ScheduleFactory extends BaseFactory
             $this->dayPartFactory,
             $this->userFactory,
             $this->scheduleReminderFactory,
-            $this->scheduleExclusionFactory
+            $this->scheduleExclusionFactory,
+            $this->scheduleCriteriaFactory
         );
     }
 
@@ -206,9 +208,9 @@ class ScheduleFactory extends BaseFactory
         // Firstly get all the scheduled layouts
         $SQL = '
             SELECT `schedule`.eventTypeId, 
-                layout.layoutId, 
-                `layout`.status, 
-                `layout`.duration,
+                `layoutLinks`.layoutId, 
+                `layoutLinks`.status, 
+                `layoutLinks`.duration,
                 `command`.code, 
                 schedule.fromDt, 
                 schedule.toDt,
@@ -244,7 +246,9 @@ class ScheduleFactory extends BaseFactory
                 `daypart`.isCustom,
                 `syncLayout`.layoutId AS syncLayoutId,
                 `syncLayout`.status AS syncLayoutStatus, 
-                `syncLayout`.duration AS syncLayoutDuration
+                `syncLayout`.duration AS syncLayoutDuration,
+                `schedule`.dataSetId,
+                `schedule`.dataSetParams
                FROM `schedule`
                 INNER JOIN `daypart`
                 ON `daypart`.dayPartId = `schedule`.dayPartId
@@ -267,12 +271,19 @@ class ScheduleFactory extends BaseFactory
         $SQL .= '    
                 LEFT OUTER JOIN `campaign`
                 ON `schedule`.CampaignID = campaign.CampaignID
-                LEFT OUTER JOIN `lkcampaignlayout`
-                ON lkcampaignlayout.CampaignID = campaign.CampaignID
-                LEFT OUTER JOIN `layout`
-                ON lkcampaignlayout.LayoutID = layout.LayoutID
-                  AND layout.retired = 0
-                  AND layout.parentId IS NULL
+                LEFT OUTER JOIN (
+                    SELECT `layout`.layoutId,
+                        `layout`.status, 
+                        `layout`.duration,
+                        `lkcampaignlayout`.campaignId,
+                        `lkcampaignlayout`.displayOrder
+                      FROM `layout`
+                        INNER JOIN `lkcampaignlayout`
+                        ON `lkcampaignlayout`.LayoutID = `layout`.layoutId  
+                     WHERE `layout`.retired = 0
+                        AND `layout`.parentId IS NULL
+                ) layoutLinks
+                ON `campaign`.CampaignID = `layoutLinks`.campaignId  
                 LEFT OUTER JOIN `command`
                 ON `command`.commandId = `schedule`.commandId
                 LEFT OUTER JOIN `schedule_sync`
@@ -312,7 +323,7 @@ class ScheduleFactory extends BaseFactory
             ORDER BY `schedule`.DisplayOrder,
                 CASE WHEN `campaign`.listPlayOrder = \'block\' THEN `schedule`.FromDT ELSE 0 END,
                 CASE WHEN `campaign`.listPlayOrder = \'block\' THEN `campaign`.campaignId ELSE 0 END,
-                IFNULL(`lkcampaignlayout`.DisplayOrder, 0),
+                IFNULL(`layoutLinks`.displayOrder, 0),
                 `schedule`.FromDT,
                 `schedule`.eventId
         ';
@@ -398,7 +409,10 @@ class ScheduleFactory extends BaseFactory
             `user`.userName as modifiedByName,
             `schedule`.createdOn,
             `schedule`.updatedOn,
-            `schedule`.name
+            `schedule`.name,
+            `schedule`.dataSetId,
+            `schedule`.dataSetParams,
+            `sc`.eventId AS criteria
         ';
 
         $body = ' FROM `schedule`
@@ -414,6 +428,9 @@ class ScheduleFactory extends BaseFactory
             ON `syncgroup`.syncGroupId = `schedule`.syncGroupId
             LEFT OUTER JOIN `user`
             ON `user`.userId = `schedule`.modifiedBy
+            LEFT OUTER JOIN (
+                SELECT DISTINCT `eventId` FROM schedule_criteria
+            ) AS sc ON `schedule`.eventId = sc.eventId
           WHERE 1 = 1';
 
         if ($parsedFilter->getInt('eventId') !== null) {
@@ -603,6 +620,7 @@ class ScheduleFactory extends BaseFactory
 
                 // not shared and not direct (new default)
                 // 1 - events scheduled on the selected display/groups
+                // 2 - events scheduled on display members of the selected display group
                 // 2 - events scheduled on a display group selected display is a member of
                 // 3 - events scheduled on a parent display group of selected display group
                 if (!$sharedSchedule && !$directSchedule) {
@@ -613,6 +631,16 @@ class ScheduleFactory extends BaseFactory
                         SELECT `lkscheduledisplaygroup`.eventId FROM `lkscheduledisplaygroup`
                             INNER JOIN `lkdgdg` ON `lkdgdg`.parentId = `lkscheduledisplaygroup`.displayGroupId 
                             INNER JOIN `lkdisplaydg` ON lkdisplaydg.DisplayGroupID = `lkdgdg`.childId
+                            INNER JOIN displaygroup ON lkdisplaydg.displayGroupId = displaygroup.displayGroupId
+                            WHERE `lkdisplaydg`.DisplayID IN (
+                                SELECT lkdisplaydg.displayId FROM lkdisplaydg 
+                                 WHERE lkdisplaydg.displayGroupId IN (' . $selectedDisplayGroupIds . ')
+                            ) AND displaygroup.isDisplaySpecific = 1 
+                        )
+                        OR `schedule`.eventID IN (
+                        SELECT `lkscheduledisplaygroup`.eventId FROM `lkscheduledisplaygroup`
+                            INNER JOIN
+                             `lkdisplaydg` ON lkdisplaydg.DisplayGroupID = `lkscheduledisplaygroup`.displayGroupId
                             WHERE `lkdisplaydg`.DisplayID IN (
                                 SELECT lkdisplaydg.displayId FROM lkdisplaydg 
                                 INNER JOIN displaygroup ON lkdisplaydg.displayGroupId = displaygroup.displayGroupId
@@ -621,7 +649,6 @@ class ScheduleFactory extends BaseFactory
                         )
                         OR `schedule`.eventID IN (
                                 SELECT `lkscheduledisplaygroup`.eventId FROM `lkscheduledisplaygroup`
-                                INNER JOIN `lkdgdg` ON `lkdgdg`.parentId = `lkscheduledisplaygroup`.displayGroupId
                                 WHERE `lkscheduledisplaygroup`.displayGroupId IN (
                                 SELECT lkdgdg.childId FROM lkdgdg 
                                 WHERE lkdgdg.parentId IN (' . $selectedDisplayGroupIds .')  AND lkdgdg.depth > 0)  
@@ -758,7 +785,8 @@ class ScheduleFactory extends BaseFactory
                     'recurrenceMonthlyRepeatsOn',
                     'isGeoAware',
                     'maxPlaysPerHour',
-                    'modifiedBy'
+                    'modifiedBy',
+                    'dataSetId',
                 ]
             ]);
         }
