@@ -1,6 +1,6 @@
 <?php
 /*
- * Copyright (C) 2023 Xibo Signage Ltd
+ * Copyright (C) 2025 Xibo Signage Ltd
  *
  * Xibo - Digital Signage - https://xibosignage.com
  *
@@ -30,7 +30,7 @@ use Phinx\Wrapper\TextWrapper;
  */
 class Environment
 {
-    public static $WEBSITE_VERSION_NAME = '4.0.5';
+    public static $WEBSITE_VERSION_NAME = '4.3.1';
     public static $XMDS_VERSION = '7';
     public static $XLF_VERSION = 4;
     public static $VERSION_REQUIRED = '8.1.0';
@@ -264,15 +264,6 @@ class Environment
         return true;
     }
 
-    /**
-     * @inheritdoc
-     */
-    public static function checkZmq()
-    {
-        return class_exists('ZMQSocket');
-    }
-
-
     public static function getMaxUploadSize()
     {
         return ini_get('upload_max_filesize');
@@ -328,6 +319,14 @@ class Environment
         return extension_loaded('simplexml');
     }
 
+    /**
+     * @return bool
+     */
+    public static function checkGnu()
+    {
+        return extension_loaded('gnupg');
+    }
+
     /**
      * @param $url
      * @return bool
