@@ -1,8 +1,8 @@
 <?php
 /*
- * Copyright (c) 2022 Xibo Signage Ltd
+ * Copyright (C) 2025 Xibo Signage Ltd
  *
- * Xibo - Digital Signage - http://www.xibo.org.uk
+ * Xibo - Digital Signage - https://xibosignage.com
  *
  * This file is part of Xibo.
  *
@@ -29,6 +29,7 @@ use Psr\Http\Message\ResponseInterface as Response;
 use Psr\Http\Message\ServerRequestInterface as Request;
 use Psr\Http\Server\MiddlewareInterface as Middleware;
 use Psr\Http\Server\RequestHandlerInterface as RequestHandler;
+use Respect\Validation\Factory;
 use Slim\App;
 use Slim\Routing\RouteContext;
 use Slim\Views\Twig;
@@ -113,7 +114,7 @@ class State implements Middleware
 
         // Handle correctly outputting cache headers for AJAX requests
         // IE cache busting
-        if ($this->isAjax($request) && $request->getMethod() == 'GET' && $request->getAttribute('name') == 'web') {
+        if ($this->isAjax($request) && $request->getMethod() == 'GET' && $request->getAttribute('_entryPoint') == 'web') {
             $response = $response->withHeader('Cache-control', 'no-cache')
                      ->withHeader('Cache-control', 'no-store')
                      ->withHeader('Pragma', 'no-cache')
@@ -177,7 +178,8 @@ class State implements Middleware
             '/playersoftware/{nonce}/sssp_config.xml',
             '/tfa',
             '/error',
-            '/notFound'
+            '/notFound',
+            '/public/thumbnail/{id}',
         ]));
 
         // Setup the translations for gettext
@@ -187,11 +189,19 @@ class State implements Middleware
         Carbon::setLocale(Translate::GetLocale(2));
 
         // Default timezone
-        date_default_timezone_set($container->get('configService')->getSetting("defaultTimezone"));
+        date_default_timezone_set($container->get('configService')->getSetting('defaultTimezone'));
 
         $container->set('session', function (ContainerInterface $container) use ($app) {
             if ($container->get('name') == 'web' || $container->get('name') == 'auth') {
-                return new Session($container->get('logService'));
+                $sessionHandler = new Session($container->get('logService'));
+
+                session_set_save_handler($sessionHandler, true);
+                register_shutdown_function('session_write_close');
+
+                // Start the session
+                session_cache_limiter(false);
+                session_start();
+                return $sessionHandler;
             } else {
                 return new NullSession();
             }
@@ -204,12 +214,13 @@ class State implements Middleware
         $mode = $container->get('configService')->getSetting('SERVER_MODE');
         $container->get('logService')->setMode($mode);
 
-
-        if ($container->get('name') == 'web' || $container->get('name') == 'xtr') {
+        // Inject some additional changes on a per-container basis
+        $containerName = $container->get('name');
+        if ($containerName == 'web' || $containerName == 'xtr' || $containerName == 'xmds') {
             /** @var Twig $view */
             $view = $container->get('view');
 
-            if ($container->get('name') == 'web') {
+            if ($containerName == 'web') {
                 $container->set('flash', function () {
                     return new \Slim\Flash\Messages();
                 });
@@ -222,8 +233,11 @@ class State implements Middleware
             $filter = new \Twig\TwigFilter('url_decode', 'urldecode');
             $twigEnvironment->addFilter($filter);
 
-            // set Twig auto reload
-            $twigEnvironment->enableAutoReload();
+            // Set Twig auto reload if needed
+            // XMDS only renders widget html cache, and shouldn't need auto reload.
+            if ($containerName !== 'xmds') {
+                $twigEnvironment->enableAutoReload();
+            }
         }
 
         // Configure logging
@@ -271,6 +285,13 @@ class State implements Middleware
             }
         }
 
+        // Add additional validation rules
+        Factory::setDefaultInstance(
+            (new Factory())
+                ->withRuleNamespace('Xibo\\Validation\\Rules')
+                ->withExceptionNamespace('Xibo\\Validation\\Exceptions')
+        );
+
         return $request;
     }
 
