@@ -1,6 +1,6 @@
 <?php
-/**
- * Copyright (C) 2019 Xibo Signage Ltd
+/*
+ * Copyright (C) 2022 Xibo Signage Ltd
  *
  * Xibo - Digital Signage - http://www.xibo.org.uk
  *
@@ -27,6 +27,7 @@ use MongoDB\BSON\ObjectId;
 use MongoDB\BSON\Regex;
 use MongoDB\BSON\UTCDateTime;
 use MongoDB\Client;
+use Xibo\Entity\Campaign;
 use Xibo\Factory\CampaignFactory;
 use Xibo\Factory\DisplayFactory;
 use Xibo\Factory\DisplayGroupFactory;
@@ -197,20 +198,15 @@ class MongoDbTimeSeriesStore implements TimeSeriesStoreInterface
             }
 
             $mediaName = $media->name; //dont remove used later
-            $mediaTags = $media->tags;
             $statData['mediaName'] = $mediaName;
 
-            if (isset($mediaTags)) {
-                $arrayOfTags = explode(',', $mediaTags);
-                for ($i=0; $i<count($arrayOfTags); $i++) {
-                    if (isset($arrayOfTags[$i]) && !empty($arrayOfTags[$i])) {
-                        $tag = explode('|', $arrayOfTags[$i]);
-                        $tagFilter['media'][$i]['tag'] = $tag[0];
-                        if (isset($tag[1])) {
-                            $tagFilter['media'][$i]['val'] = $tag[1];
-                        }
-                    }
+            $i = 0;
+            foreach ($media->tags as $tagLink) {
+                $tagFilter['media'][$i]['tag'] = $tagLink->tag;
+                if (isset($tagLink->value)) {
+                    $tagFilter['media'][$i]['val'] = $tagLink->value;
                 }
+                $i++;
             }
         }
 
@@ -246,7 +242,6 @@ class MongoDbTimeSeriesStore implements TimeSeriesStoreInterface
 
         // Layout data
         $layoutName = null;
-        $layoutTags = null;
 
         // For a type "event" we have layoutid 0 so is campaignId
         // otherwise we should try and resolve the campaignId
@@ -293,7 +288,15 @@ class MongoDbTimeSeriesStore implements TimeSeriesStoreInterface
 
             $campaignId = (int) $layout->campaignId;
             $layoutName = $layout->layout;
-            $layoutTags = $layout->tags;
+
+            $i = 0;
+            foreach ($layout->tags as $tagLink) {
+                $tagFilter['layout'][$i]['tag'] = $tagLink->tag;
+                if (isset($tagLink->value)) {
+                    $tagFilter['layout'][$i]['val'] = $tagLink->value;
+                }
+                $i++;
+            }
         }
 
         // Get layout Campaign ID
@@ -301,20 +304,6 @@ class MongoDbTimeSeriesStore implements TimeSeriesStoreInterface
 
         $statData['layoutName'] = $layoutName;
 
-        // Layout tags
-        if (isset($layoutTags)) {
-            $arrayOfTags = explode(',', $layoutTags);
-            for ($i=0; $i<count($arrayOfTags); $i++) {
-                if (isset($arrayOfTags[$i]) && !empty($arrayOfTags[$i])) {
-                    $tag = explode('|', $arrayOfTags[$i]);
-                    $tagFilter['layout'][$i]['tag'] = $tag[0];
-                    if (isset($tag[1])) {
-                        $tagFilter['layout'][$i]['val'] = $tag[1];
-                    }
-                }
-            }
-        }
-
 
         // Display
         $display = $statData['display'];
@@ -326,16 +315,13 @@ class MongoDbTimeSeriesStore implements TimeSeriesStoreInterface
         // Display name
         $statData['displayName'] = $display->display;
 
-        $arrayOfTags = array_filter(explode(',', $display->tags));
-
-        for ($i=0; $i<count($arrayOfTags); $i++) {
-            if (isset($arrayOfTags[$i]) && !empty($arrayOfTags[$i])) {
-                $tag = explode('|', $arrayOfTags[$i]);
-                $tagFilter['dg'][$i]['tag'] = $tag[0];
-                if (isset($tag[1])) {
-                    $tagFilter['dg'][$i]['val'] = $tag[1];
-                }
+        $i = 0;
+        foreach ($display->tags as $tagLink) {
+            $tagFilter['dg'][$i]['tag'] = $tagLink->tag;
+            if (isset($tagLink->value)) {
+                $tagFilter['dg'][$i]['val'] = $tagLink->value;
             }
+            $i++;
         }
 
         // Display tags
@@ -353,21 +339,27 @@ class MongoDbTimeSeriesStore implements TimeSeriesStoreInterface
             }
         }
 
-        $arrayOfTags = array_filter(explode(',', $displayGroup->tags));
-
-        for ($i=0; $i<count($arrayOfTags); $i++) {
-            if (isset($arrayOfTags[$i]) && !empty($arrayOfTags[$i])) {
-                $tag = explode('|', $arrayOfTags[$i]);
-                $tagFilter['dg'][$i]['tag'] = $tag[0];
-                if (isset($tag[1])) {
-                    $tagFilter['dg'][$i]['val'] = $tag[1];
-                }
+        $i = 0;
+        foreach ($displayGroup->tags as $tagLink) {
+            $tagFilter['dg'][$i]['tag'] = $tagLink->tag;
+            if (isset($tagLink->value)) {
+                $tagFilter['dg'][$i]['val'] = $tagLink->value;
             }
+            $i++;
         }
 
         // TagFilter array
         $statData['tagFilter'] = $tagFilter;
 
+        // Parent Campaign
+        if (array_key_exists('parentCampaign', $statData)) {
+            if ($statData['parentCampaign'] instanceof Campaign) {
+                $statData['parentCampaign'] = $statData['parentCampaign']->campaign;
+            } else {
+                $statData['parentCampaign'] = '';
+            }
+        }
+
         $this->stats[] = $statData;
     }
 
@@ -398,7 +390,7 @@ class MongoDbTimeSeriesStore implements TimeSeriesStoreInterface
         try {
             $cursor = $collectionPeriod->findOne(['name' => 'period']);
 
-            if (count($cursor) <= 0) {
+            if ($cursor === null) {
                 $this->log->error('Period collection does not exist in Mongo DB.');
                 // Period collection created
                 $collectionPeriod->insertOne(['name' => 'period']);
@@ -433,7 +425,7 @@ class MongoDbTimeSeriesStore implements TimeSeriesStoreInterface
     /**
      * @inheritdoc
      */
-    public function getStats($filterBy = [])
+    public function getStats($filterBy = [], $isBufferedQuery = false)
     {
         // do we consider that the fromDt and toDt will always be provided?
         $fromDt = $filterBy['fromDt'] ?? null;
@@ -457,6 +449,8 @@ class MongoDbTimeSeriesStore implements TimeSeriesStoreInterface
         $layoutIds = $filterBy['layoutIds'] ?? [];
         $mediaIds = $filterBy['mediaIds'] ?? [];
         $campaignId = $filterBy['campaignId'] ?? null;
+        $parentCampaignId = $filterBy['parentCampaignId'] ?? null;
+        $mustHaveParentCampaign = $filterBy['mustHaveParentCampaign'] ?? false;
         $eventTag = $filterBy['eventTag'] ?? null;
 
         // Limit
@@ -504,7 +498,7 @@ class MongoDbTimeSeriesStore implements TimeSeriesStoreInterface
             $match['$match']['displayId'] = ['$in' => $displayIds];
         }
 
-        // Campaign Filter
+        // Campaign/Layout Filter
         // ---------------
         // Use the Layout Factory to get all Layouts linked to the provided CampaignId
         if ($campaignId != null) {
@@ -553,6 +547,16 @@ class MongoDbTimeSeriesStore implements TimeSeriesStoreInterface
             $match['$match']['mediaId'] = ['$in' => $mediaIds];
         }
 
+        // Parent Campaign Filter
+        if ($parentCampaignId != null) {
+            $match['$match']['parentCampaignId'] = $parentCampaignId;
+        }
+
+        // Has Parent Campaign Filter
+        if ($mustHaveParentCampaign) {
+            $match['$match']['parentCampaignId'] = ['$exists' => true, '$ne' => 0];
+        }
+
         // Select collection
         $collection = $this->getClient()->selectCollection($this->config['database'], $this->table);
 
@@ -611,6 +615,10 @@ class MongoDbTimeSeriesStore implements TimeSeriesStoreInterface
                     'widgetId'=> 1,
                     'mediaId'=> 1,
                     'campaignId'=> 1,
+                    'parentCampaign'=> 1,
+                    'parentCampaignId'=> 1,
+                    'campaignStart'=> 1,
+                    'campaignEnd'=> 1,
                     'statDate'=> 1,
                     'engagements'=> 1,
                     'tagFilter' => 1
