@@ -312,6 +312,7 @@ class SeedDatabaseTask implements TaskInterface
         // all layouts name and file name
         $layoutNames = [
             'dataset test ' => 'export-dataset-test.zip',
+            'layout_with_8_items_dataset' => 'export-layout-with-8-items-dataset.zip',
             'Image test' => 'export-image-test.zip',
             'Layout for Schedule 1' => 'export-layout-for-schedule-1.zip',
             'List Campaign Layout 1' => 'export-list-campaign-layout-1.zip',
@@ -320,6 +321,9 @@ class SeedDatabaseTask implements TaskInterface
 
             // Layout for displaygroups.cy.js test
             'disp4_default_layout' => 'export-disp4-default-layout.zip',
+
+            // Layout editor tests
+            'Audio-Video-PDF' => 'export-audio-video-pdf.zip'
         ];
 
         // Get all layouts
@@ -908,7 +912,7 @@ class SeedDatabaseTask implements TaskInterface
                 $command->description = 'a command to test schedule';
                 $command->code = 'TIMEZONE';
                 $command->userId = $this->userFactory->getSystemUser()->getId();
-
+                $command->createAlertOn = 'never';
                 $command->save();
                 $this->store->commitIfNecessary();
             } catch (GeneralException $e) {
