@@ -1,6 +1,6 @@
 <?php
 /*
- * Copyright (C) 2023 Xibo Signage Ltd
+ * Copyright (C) 2024 Xibo Signage Ltd
  *
  * Xibo - Digital Signage - https://xibosignage.com
  *
@@ -27,6 +27,7 @@ use Xibo\Entity\Display;
 use Xibo\Entity\Module;
 use Xibo\Entity\Widget;
 use Xibo\Event\WidgetDataRequestEvent;
+use Xibo\Factory\WidgetDataFactory;
 use Xibo\Helper\DateFormatHelper;
 use Xibo\Support\Exception\GeneralException;
 use Xibo\Support\Exception\NotFoundException;
@@ -47,6 +48,9 @@ class WidgetSyncTask implements TaskInterface
     /** @var \Xibo\Factory\WidgetFactory */
     private $widgetFactory;
 
+    /** @var \Xibo\Factory\WidgetDataFactory */
+    private WidgetDataFactory $widgetDataFactory;
+
     /** @var \Xibo\Factory\MediaFactory */
     private $mediaFactory;
 
@@ -61,6 +65,7 @@ class WidgetSyncTask implements TaskInterface
     {
         $this->moduleFactory = $container->get('moduleFactory');
         $this->widgetFactory = $container->get('widgetFactory');
+        $this->widgetDataFactory = $container->get('widgetDataFactory');
         $this->mediaFactory = $container->get('mediaFactory');
         $this->displayFactory = $container->get('displayFactory');
         $this->eventDispatcher = $container->get('dispatcher');
@@ -125,7 +130,8 @@ class WidgetSyncTask implements TaskInterface
                         }
 
                         // Refresh the cache if needed.
-                        $isDisplaySpecific = str_contains($cacheKey, '%displayId%');
+                        $isDisplaySpecific = str_contains($cacheKey, '%displayId%')
+                            || str_contains($cacheKey, '%useDisplayLocation%');
 
                         // We're either assigning all media to all displays, or we're assigning then one by one
                         if ($isDisplaySpecific) {
@@ -133,20 +139,32 @@ class WidgetSyncTask implements TaskInterface
 
                             // We need to run the cache for every display this widget is assigned to.
                             foreach ($this->getDisplays($widget) as $display) {
-                                $mediaIds = $this->cache(
+                                $cacheData = $this->cache(
                                     $module,
                                     $widget,
                                     $widgetInterface,
                                     $display,
                                 );
-                                $this->linkDisplays($widget->widgetId, [$display], $mediaIds);
+
+                                $this->linkDisplays(
+                                    $widget->widgetId,
+                                    [$display],
+                                    $cacheData['mediaIds'],
+                                    $cacheData['fromCache']
+                                );
                             }
                         } else {
                             $this->getLogger()->debug('widgetSyncTask: cache is not display specific');
 
                             // Just a single run will do it.
-                            $mediaIds = $this->cache($module, $widget, $widgetInterface, null);
-                            $this->linkDisplays($widget->widgetId, $this->getDisplays($widget), $mediaIds);
+                            $cacheData = $this->cache($module, $widget, $widgetInterface, null);
+
+                            $this->linkDisplays(
+                                $widget->widgetId,
+                                $this->getDisplays($widget),
+                                $cacheData['mediaIds'],
+                                $cacheData['fromCache']
+                            );
                         }
 
                         // Record end time and aggregate for final total
@@ -177,8 +195,12 @@ class WidgetSyncTask implements TaskInterface
     }
 
     /**
-     * @return int[] mediaIds
-     * @throws \Xibo\Support\Exception\GeneralException
+     * @param Module $module
+     * @param Widget $widget
+     * @param WidgetProviderInterface|null $widgetInterface
+     * @param Display|null $display
+     * @return array
+     * @throws GeneralException
      */
     private function cache(
         Module $module,
@@ -188,11 +210,17 @@ class WidgetSyncTask implements TaskInterface
     ): array {
         $this->getLogger()->debug('cache: ' . $widget->widgetId . ' for display: ' . ($display?->displayId ?? 0));
 
-        $mediaIds = [];
-
         // Each time we call this we use a new provider
         $dataProvider = $module->createDataProvider($widget);
         $dataProvider->setMediaFactory($this->mediaFactory);
+
+        // Set our provider up for the display
+        $dataProvider->setDisplayProperties(
+            $display?->latitude ?: $this->getConfig()->getSetting('DEFAULT_LAT'),
+            $display?->longitude ?: $this->getConfig()->getSetting('DEFAULT_LONG'),
+            $display?->displayId ?? 0
+        );
+
         $widgetDataProviderCache = $this->moduleFactory->createWidgetDataProviderCache();
 
         // Get the cache key
@@ -204,16 +232,6 @@ class WidgetSyncTask implements TaskInterface
             $widgetInterface
         );
 
-        // Set our provider up for the displays
-        if ($display !== null) {
-            $dataProvider->setDisplayProperties($display->latitude, $display->longitude, $display->displayId);
-        } else {
-            $dataProvider->setDisplayProperties(
-                $this->getConfig()->getSetting('DEFAULT_LAT'),
-                $this->getConfig()->getSetting('DEFAULT_LONG')
-            );
-        }
-
         // Get the data modified date
         $dataModifiedDt = null;
         if ($widgetInterface !== null) {
@@ -224,6 +242,35 @@ class WidgetSyncTask implements TaskInterface
             }
         }
 
+        // Will we use fallback data if available?
+        $showFallback = $widget->getOptionValue('showFallback', 'never');
+        if ($showFallback !== 'never') {
+            // What data type are we dealing with?
+            try {
+                $dataTypeFields = [];
+                foreach ($this->moduleFactory->getDataTypeById($module->dataType)->fields as $field) {
+                    $dataTypeFields[$field->id] = $field->type;
+                }
+
+                // Potentially we will, so get the modifiedDt of this fallback data.
+                $fallbackModifiedDt = $this->widgetDataFactory->getModifiedDtForWidget($widget->widgetId);
+
+                if ($fallbackModifiedDt !== null) {
+                    $this->getLogger()->debug('cache: fallback modifiedDt is ' . $fallbackModifiedDt->toAtomString());
+
+                    $dataModifiedDt = max($dataModifiedDt, $fallbackModifiedDt);
+                }
+            } catch (NotFoundException) {
+                $this->getLogger()->info('cache: widget will fallback set where the module does not support it');
+                $dataTypeFields = null;
+            }
+        } else {
+            $dataTypeFields = null;
+        }
+
+        // Is this data from cache?
+        $fromCache = false;
+
         if (!$widgetDataProviderCache->decorateWithCache($dataProvider, $cacheKey, $dataModifiedDt)
             || $widgetDataProviderCache->isCacheMissOrOld()
         ) {
@@ -231,6 +278,7 @@ class WidgetSyncTask implements TaskInterface
 
             $dataProvider->clearData();
             $dataProvider->clearMeta();
+            $dataProvider->addOrUpdateMeta('showFallback', $showFallback);
 
             try {
                 if ($widgetInterface !== null) {
@@ -246,25 +294,66 @@ class WidgetSyncTask implements TaskInterface
                     );
                 }
 
+                // Before caching images, check to see if the data provider is handled
+                $isFallback = false;
+                if ($showFallback !== 'never'
+                    && $dataTypeFields !== null
+                    && (
+                        count($dataProvider->getErrors()) > 0
+                        || count($dataProvider->getData()) <= 0
+                        || $showFallback === 'always'
+                    )
+                ) {
+                    // Error or no data.
+                    // Pull in the fallback data
+                    foreach ($this->widgetDataFactory->getByWidgetId($dataProvider->getWidgetId()) as $item) {
+                        // Handle any special data types in the fallback data
+                        foreach ($item->data as $itemId => $itemData) {
+                            if (!empty($itemData)
+                                && array_key_exists($itemId, $dataTypeFields)
+                                && $dataTypeFields[$itemId] === 'image'
+                            ) {
+                                $item->data[$itemId] = $dataProvider->addLibraryFile($itemData);
+                            }
+                        }
+
+                        $dataProvider->addItem($item->data);
+
+                        // Indicate we've been handled by fallback data
+                        $isFallback = true;
+                    }
+
+                    if ($isFallback) {
+                        $dataProvider->addOrUpdateMeta('includesFallback', true);
+                    }
+                }
+
+                // Remove fallback data from the cache if no-longer needed
+                if (!$isFallback) {
+                    $dataProvider->addOrUpdateMeta('includesFallback', false);
+                }
+
                 // Do we have images?
-                $media = $dataProvider->getImages();
-                if (count($media) > 0) {
+                // They could be library images (i.e. they already exist) or downloads
+                $mediaIds = $dataProvider->getImageIds();
+                if (count($mediaIds) > 0) {
                     // Process the downloads.
-                    $this->mediaFactory->processDownloads(function ($media) use ($widget, &$mediaIds) {
+                    $this->mediaFactory->processDownloads(function ($media) {
                         /** @var \Xibo\Entity\Media $media */
                         // Success
                         // We don't need to do anything else, references to mediaId will be built when we decorate
                         // the HTML.
                         $this->getLogger()->debug('cache: Successfully downloaded ' . $media->mediaId);
-
-                        if (!in_array($media->mediaId, $mediaIds)) {
-                            $mediaIds[] = $media->mediaId;
-                        }
+                    }, function ($media) use (&$mediaIds) {
+                        /** @var \Xibo\Entity\Media $media */
+                        // Error
+                        // Remove it
+                        unset($mediaIds[$media->mediaId]);
                     });
                 }
 
                 // Save to cache
-                if ($dataProvider->isHandled()) {
+                if ($dataProvider->isHandled() || $isFallback) {
                     $widgetDataProviderCache->saveToCache($dataProvider);
                 }
             } finally {
@@ -273,11 +362,16 @@ class WidgetSyncTask implements TaskInterface
         } else {
             $this->getLogger()->debug('cache: Cache still valid, key: ' . $cacheKey);
 
+            $fromCache = true;
+
             // Get the existing mediaIds so that we can maintain the links to displays.
             $mediaIds = $widgetDataProviderCache->getCachedMediaIds();
         }
 
-        return $mediaIds;
+        return [
+            'mediaIds' => $mediaIds,
+            'fromCache' => $fromCache
+        ];
     }
 
     /**
@@ -311,9 +405,10 @@ class WidgetSyncTask implements TaskInterface
      * @param int $widgetId
      * @param Display[] $displays
      * @param int[] $mediaIds
+     * @param bool $fromCache
      * @return void
      */
-    private function linkDisplays(int $widgetId, array $displays, array $mediaIds): void
+    private function linkDisplays(int $widgetId, array $displays, array $mediaIds, bool $fromCache): void
     {
         $this->getLogger()->debug('linkDisplays: ' . count($displays) . ' displays, ' . count($mediaIds) . ' media');
 
@@ -356,8 +451,11 @@ class WidgetSyncTask implements TaskInterface
                     $this->displayFactory->getDisplayNotifyService()->collectLater()
                         ->notifyByDisplayId($display->displayId);
                 }
-                $this->displayFactory->getDisplayNotifyService()
-                    ->notifyDataUpdate($display, $widgetId);
+
+                if (!$fromCache) {
+                    $this->displayFactory->getDisplayNotifyService()
+                        ->notifyDataUpdate($display, $widgetId);
+                }
             } else {
                 $this->displayFactory->getDisplayNotifyService()->collectNow()
                     ->notifyByDisplayId($display->displayId);
