@@ -1,8 +1,8 @@
 <?php
 /*
- * Copyright (c) 2022 Xibo Signage Ltd
+ * Copyright (C) 2025 Xibo Signage Ltd
  *
- * Xibo - Digital Signage - http://www.xibo.org.uk
+ * Xibo - Digital Signage - https://xibosignage.com
  *
  * This file is part of Xibo.
  *
@@ -25,6 +25,7 @@ use Carbon\Carbon;
 use Intervention\Image\ImageManagerStatic as Img;
 use Xibo\Entity\Bandwidth;
 use Xibo\Entity\Display;
+use Xibo\Event\XmdsDependencyRequestEvent;
 use Xibo\Helper\DateFormatHelper;
 use Xibo\Helper\Random;
 use Xibo\Support\Exception\GeneralException;
@@ -33,6 +34,7 @@ use Xibo\Support\Exception\NotFoundException;
 /**
  * Class Soap4
  * @package Xibo\Xmds
+ * @phpcs:disable PSR1.Methods.CamelCapsMethodName.NotCamelCaps
  */
 class Soap4 extends Soap
 {
@@ -67,7 +69,7 @@ class Soap4 extends Soap
             'operatingSystem' => $operatingSystem,
             'macAddress' => $macAddress,
             'xmrChannel' => $xmrChannel,
-            'xmrPubKey' => $xmrPubKey
+            'xmrPubKey' => $xmrPubKey,
         ]);
 
         // Sanitize
@@ -79,14 +81,17 @@ class Soap4 extends Soap
         $clientCode = $sanitized->getInt('clientCode');
         $macAddress = $sanitized->getString('macAddress');
         $clientAddress = $this->getIp();
+        $operatingSystem = $sanitized->getString('operatingSystem');
 
         // Check the serverKey matches
-        if ($serverKey != $this->getConfig()->getSetting('SERVER_KEY'))
+        if ($serverKey != $this->getConfig()->getSetting('SERVER_KEY')) {
             throw new \SoapFault('Sender', 'The Server key you entered does not match with the server key at this address');
+        }
 
         // Check the Length of the hardwareKey
-        if (strlen($hardwareKey) > 40)
+        if (strlen($hardwareKey) > 40) {
             throw new \SoapFault('Sender', 'The Hardware Key you sent was too long. Only 40 characters are allowed (SHA1).');
+        }
 
         // Return an XML formatted string
         $return = new \DOMDocument('1.0');
@@ -98,10 +103,13 @@ class Soap4 extends Soap
             $display = $this->displayFactory->getByLicence($hardwareKey);
             $this->display = $display;
 
-            $this->logProcessor->setDisplay($display->displayId, ($display->isAuditing()));
+            $this->logProcessor->setDisplay($display->displayId, $display->isAuditing());
 
             // Audit in
-            $this->getLog()->debug('serverKey: ' . $serverKey . ', hardwareKey: ' . $hardwareKey . ', displayName: ' . $displayName . ', macAddress: ' . $macAddress);
+            $this->getLog()->debug(
+                'serverKey: ' . $serverKey . ', hardwareKey: ' . $hardwareKey .
+                ', displayName: ' . $displayName . ', macAddress: ' . $macAddress
+            );
 
             // Now
             $dateNow = Carbon::now();
@@ -116,7 +124,6 @@ class Soap4 extends Soap
                 $displayElement->setAttribute('status', 2);
                 $displayElement->setAttribute('code', 'WAITING');
                 $displayElement->setAttribute('message', 'Display is awaiting licensing approval from an Administrator.');
-
             } else {
                 // It is licensed
                 $displayElement->setAttribute('status', 0);
@@ -131,12 +138,6 @@ class Soap4 extends Soap
                     // Upper case the setting name for windows
                     $settingName = ($clientType == 'windows') ? ucfirst($arrayItem['name']) : $arrayItem['name'];
 
-                    $node = $return->createElement($settingName, (isset($arrayItem['value']) ? $arrayItem['value'] : $arrayItem['default']));
-
-                    if (isset($arrayItem['type'])) {
-                        $node->setAttribute('type', $arrayItem['type']);
-                    }
-
                     // Patch download and update windows to make sure they are unix time stamps
                     // XMDS schema 4 sent down unix time
                     // https://github.com/xibosignage/xibo/issues/1791
@@ -154,7 +155,13 @@ class Soap4 extends Soap
                         }
                     }
 
-                    $node = $return->createElement($arrayItem['name'], (isset($arrayItem['value']) ? $arrayItem['value'] : $arrayItem['default']));
+                    // Apply an offset to the collectInterval
+                    // https://github.com/xibosignage/xibo/issues/3530
+                    if (strtolower($arrayItem['name']) == 'collectinterval') {
+                        $arrayItem['value'] = $this->collectionIntervalWithOffset($arrayItem['value']);
+                    }
+
+                    $node = $return->createElement($arrayItem['name'], $arrayItem['value'] ?? $arrayItem['default']);
                     $node->setAttribute('type', $arrayItem['type']);
                     $displayElement->appendChild($node);
                 }
@@ -162,21 +169,21 @@ class Soap4 extends Soap
                 // Player upgrades
                 $version = '';
                 try {
-                    $upgradeMediaId = $this->display->getSetting('versionMediaId', null, ['displayOverride' => true]);
+                    $versionId = $this->display->getSetting('versionMediaId', null, ['displayOverride' => true]);
 
-                    if ($clientType != 'windows' && $upgradeMediaId != null) {
-                        $version = $this->playerVersionFactory->getByMediaId($upgradeMediaId);
+                    if ($clientType != 'windows' && $versionId != null) {
+                        $version = $this->playerVersionFactory->getById($versionId);
 
                         if ($clientType == 'android') {
                             $version = json_encode([
-                                'id' => $upgradeMediaId,
-                                'file' => $version->storedAs,
+                                'id' => $versionId,
+                                'file' => $version->fileName,
                                 'code' => $version->code
                             ]);
                         } elseif ($clientType == 'lg') {
                             $version = json_encode([
-                                'id' => $upgradeMediaId,
-                                'file' => $version->storedAs,
+                                'id' => $versionId,
+                                'file' => $version->fileName,
                                 'code' => $version->code
                             ]);
                         } elseif ($clientType == 'sssp') {
@@ -188,8 +195,8 @@ class Soap4 extends Soap
                             $this->getPool()->saveDeferred($cache);
 
                             $version = json_encode([
-                                'id' => $upgradeMediaId,
-                                'file' => $version->storedAs,
+                                'id' => $versionId,
+                                'file' => $version->fileName,
                                 'code' => $version->code,
                                 'url' => str_replace('/xmds.php', '', Wsdl::getRoot()) . '/playersoftware/' . $nonce
                             ]);
@@ -226,9 +233,7 @@ class Soap4 extends Soap
                     $displayElement->setAttribute('localDate', $dateNow->format(DateFormatHelper::getSystemFormat()));
                 }
             }
-
         } catch (NotFoundException $e) {
-
             // Add a new display
             try {
                 $display = $this->displayFactory->createEmpty();
@@ -244,17 +249,17 @@ class Soap4 extends Soap
                 if (!$display->isDisplaySlotAvailable()) {
                     $display->licensed = 0;
                 }
-            }
-            catch (\InvalidArgumentException $e) {
+            } catch (\InvalidArgumentException $e) {
                 throw new \SoapFault('Sender', $e->getMessage());
             }
 
             $displayElement->setAttribute('status', 1);
             $displayElement->setAttribute('code', 'ADDED');
-            if ($display->licensed == 0)
+            if ($display->licensed == 0) {
                 $displayElement->setAttribute('message', 'Display added and is awaiting licensing approval from an Administrator.');
-            else
+            } else {
                 $displayElement->setAttribute('message', 'Display is active and ready to start.');
+            }
         }
 
         // Send Notification if required
@@ -267,7 +272,18 @@ class Soap4 extends Soap
         $display->clientType = $clientType;
         $display->clientVersion = $clientVersion;
         $display->clientCode = $clientCode;
-        //$display->operatingSystem = $operatingSystem;
+
+        // Parse operatingSystem JSON data
+        $operatingSystemJson = json_decode($operatingSystem, false);
+
+        // Newer version of players will return a JSON value, but for older version, it will return a string.
+        // In case the json decode fails, use the operatingSystem string value as the default value for the osVersion.
+        $display->osVersion = $operatingSystemJson->version ?? $operatingSystem;
+        $display->osSdk = $operatingSystemJson->sdk ?? null;
+        $display->manufacturer = $operatingSystemJson->manufacturer ?? null;
+        $display->brand = $operatingSystemJson->brand ?? null;
+        $display->model = $operatingSystemJson->model ?? null;
+
         $display->save(['validate' => false, 'audit' => false]);
 
         // Log Bandwidth
@@ -276,9 +292,6 @@ class Soap4 extends Soap
 
         // Audit our return
         $this->getLog()->debug($returnXml);
-        
-        // Phone Home?
-        $this->phoneHome();
 
         return $returnXml;
     }
@@ -311,7 +324,7 @@ class Soap4 extends Soap
      * @throws \Xibo\Support\Exception\InvalidArgumentException
      * @throws \Xibo\Support\Exception\NotFoundException
      */
-    function GetFile($serverKey, $hardwareKey, $fileId, $fileType, $chunkOffset, $chunkSize)
+    function GetFile($serverKey, $hardwareKey, $fileId, $fileType, $chunkOffset, $chunkSize, $isDependency = false)
     {
         $this->logProcessor->setRoute('GetFile');
 
@@ -323,37 +336,88 @@ class Soap4 extends Soap
             'chunkOffset' => $chunkOffset,
             'chunkSize' => $chunkSize
         ]);
+
         // Sanitize
         $serverKey = $sanitizer->getString('serverKey');
         $hardwareKey = $sanitizer->getString('hardwareKey');
-        $fileId = $sanitizer->getInt('fileId');
+        if ($isDependency) {
+            $fileId = $sanitizer->getString('fileId');
+        } else {
+            $fileId = $sanitizer->getInt('fileId');
+        }
         $fileType = $sanitizer->getString('fileType');
         $chunkOffset = $sanitizer->getDouble('chunkOffset');
         $chunkSize = $sanitizer->getDouble('chunkSize');
 
-        $libraryLocation = $this->getConfig()->getSetting("LIBRARY_LOCATION");
+        $libraryLocation = $this->getConfig()->getSetting('LIBRARY_LOCATION');
 
         // Check the serverKey matches
         if ($serverKey != $this->getConfig()->getSetting('SERVER_KEY')) {
-            throw new \SoapFault('Sender', 'The Server key you entered does not match with the server key at this address');
+            throw new \SoapFault(
+                'Sender',
+                'The Server key you entered does not match with the server key at this address'
+            );
         }
 
         // Authenticate this request...
         if (!$this->authDisplay($hardwareKey)) {
-            throw new \SoapFault('Receiver', "This Display is not authorised.");
+            throw new \SoapFault('Receiver', 'This Display is not authorised.');
         }
 
         // Now that we authenticated the Display, make sure we are sticking to our bandwidth limit
         if (!$this->checkBandwidth($this->display->displayId)) {
-            throw new \SoapFault('Receiver', "Bandwidth Limit exceeded");
+            throw new \SoapFault('Receiver', 'Bandwidth Limit exceeded');
         }
 
-        if ($this->display->isAuditing()) {
-            $this->getLog()->debug('hardwareKey: ' . $hardwareKey . ', fileId: ' . $fileId . ', fileType: ' . $fileType . ', chunkOffset: ' . $chunkOffset . ', chunkSize: ' . $chunkSize);
-        }
+
+        $this->getLog()->debug(
+            'hardwareKey: ' . $hardwareKey . ', fileId: ' . $fileId . ', fileType: ' . $fileType .
+            ', chunkOffset: ' . $chunkOffset . ', chunkSize: ' . $chunkSize
+        );
+
 
         try {
-            if ($fileType == "layout") {
+            if ($isDependency || ($fileType == 'media' && $fileId < 0)) {
+                // Validate the nonce
+                // If we are an older player downloading as media using a faux fileId, then this lookup
+                // should be performed against the `itemId`
+                $requiredFile = $this->requiredFileFactory->getByDisplayAndDependency(
+                    $this->display->displayId,
+                    $fileType,
+                    $fileId,
+                    !($fileType == 'media' && $fileId < 0)
+                );
+
+                // File is valid, see if we can return it.
+                $event = new XmdsDependencyRequestEvent($requiredFile);
+                $this->getDispatcher()->dispatch($event, 'xmds.dependency.request');
+
+                // Get the path
+                $path = $event->getRelativePath();
+                if (empty($path)) {
+                    throw new NotFoundException(__('File not found'));
+                }
+
+                $path = $libraryLocation . $path;
+
+                $f = fopen($path, 'r');
+                if (!$f) {
+                    throw new NotFoundException(__('Unable to get file pointer'));
+                }
+
+                fseek($f, $chunkOffset);
+                $file = fread($f, $chunkSize);
+
+                // Store file size for bandwidth log
+                $chunkSize = strlen($file);
+
+                if ($chunkSize === 0) {
+                    throw new NotFoundException(__('Empty file'));
+                }
+
+                $requiredFile->bytesRequested = $requiredFile->bytesRequested + $chunkSize;
+                $requiredFile->save();
+            } else if ($fileType == 'layout') {
                 // Validate the nonce
                 $requiredFile = $this->requiredFileFactory->getByDisplayAndLayout($this->display->displayId, $fileId);
 
@@ -370,20 +434,26 @@ class Soap4 extends Soap
 
                 $requiredFile->bytesRequested = $requiredFile->bytesRequested + $chunkSize;
                 $requiredFile->save();
-
-            } else if ($fileType == "media") {
-                // Validate the nonce
-                $requiredFile = $this->requiredFileFactory->getByDisplayAndMedia($this->display->displayId, $fileId);
+            } else if ($fileType == 'media') {
+                // A normal media file.
+                $requiredFile = $this->requiredFileFactory->getByDisplayAndMedia(
+                    $this->display->displayId,
+                    $fileId
+                );
 
                 $media = $this->mediaFactory->getById($fileId);
                 $this->getLog()->debug(json_encode($media));
 
-                if (!file_exists($libraryLocation . $media->storedAs))
-                    throw new NotFoundException(__('Media exists but file missing from library. ') . $libraryLocation);
+                if (!file_exists($libraryLocation . $media->storedAs)) {
+                    throw new NotFoundException(__('Media exists but file missing from library.'));
+                }
 
                 // Return the Chunk size specified
-                if (!$f = fopen($libraryLocation . $media->storedAs, 'r'))
+                $f = fopen($libraryLocation . $media->storedAs, 'r');
+
+                if (!$f) {
                     throw new NotFoundException(__('Unable to get file pointer'));
+                }
 
                 fseek($f, $chunkOffset);
 
@@ -392,23 +462,27 @@ class Soap4 extends Soap
                 // Store file size for bandwidth log
                 $chunkSize = strlen($file);
 
-                if ($chunkSize === 0)
+                if ($chunkSize === 0) {
                     throw new NotFoundException(__('Empty file'));
+                }
 
                 $requiredFile->bytesRequested = $requiredFile->bytesRequested + $chunkSize;
                 $requiredFile->save();
-
             } else {
                 throw new NotFoundException(__('Unknown FileType Requested.'));
             }
-        }
-        catch (NotFoundException $e) {
-            $this->getLog()->error('Not found FileId: ' . $fileId . '. FileType: ' . $fileType . '. ' . $e->getMessage());
+        } catch (NotFoundException $e) {
+            $this->getLog()->error('Not found FileId: ' . $fileId . '. FileType: '
+                . $fileType . '. ' . $e->getMessage());
             throw new \SoapFault('Receiver', 'Requested an invalid file.');
         }
 
         // Log Bandwidth
-        $this->logBandwidth($this->display->displayId, Bandwidth::$GETFILE, $chunkSize);
+        if ($isDependency) {
+            $this->logBandwidth($this->display->displayId, Bandwidth::$GET_DEPENDENCY, $chunkSize);
+        } else {
+            $this->logBandwidth($this->display->displayId, Bandwidth::$GETFILE, $chunkSize);
+        }
 
         return $file;
     }
@@ -538,9 +612,7 @@ class Soap4 extends Soap
         }
 
         // Important to keep this logging in place (status screen notification gets logged)
-        if ($this->display->isAuditing()) {
-            $this->getLog()->debug($status);
-        }
+        $this->getLog()->debug($status);
 
         $this->logBandwidth($this->display->displayId, Bandwidth::$NOTIFYSTATUS, strlen($status));
 
@@ -551,13 +623,17 @@ class Soap4 extends Soap
         $this->display->storageTotalSpace = $sanitizedStatus->getInt('totalSpace', ['default' => $this->display->storageTotalSpace]);
         $this->display->lastCommandSuccess = $sanitizedStatus->getCheckbox('lastCommandSuccess');
         $this->display->deviceName = $sanitizedStatus->getString('deviceName', ['default' => $this->display->deviceName]);
-        $commercialLicenceString = $sanitizedStatus->getString('licenceResult',['default' => null]);
+        $this->display->lanIpAddress = $sanitizedStatus->getString('lanIpAddress', ['default' => $this->display->lanIpAddress]);
+        $commercialLicenceString = $sanitizedStatus->getString('licenceResult', ['default' => null]);
 
         // Commercial Licence Check,  0 - Not licensed, 1 - licensed, 2 - trial licence, 3 - not applicable
         if (!empty($commercialLicenceString)) {
-            if ($commercialLicenceString === 'Licensed fully') {
+            if ($commercialLicenceString === 'Licensed fully'
+                || $commercialLicenceString === 'licensed'
+                || $commercialLicenceString === 'full'
+            ) {
                 $commercialLicence = 1;
-            } elseif ($commercialLicenceString === 'Trial') {
+            } elseif ($commercialLicenceString === 'Trial' || $commercialLicenceString === 'trial') {
                 $commercialLicence = 2;
             } else {
                 $commercialLicence = 0;
@@ -640,8 +716,9 @@ class Soap4 extends Soap
 
         // Touch the display record
         try {
-            if (count($this->display->getChangedProperties()) > 0)
+            if (count($this->display->getChangedProperties()) > 0) {
                 $this->display->save(Display::$saveOptionsMinimum);
+            }
         } catch (GeneralException $xiboException) {
             $this->getLog()->error($xiboException->getMessage());
             throw new \SoapFault('Receiver', 'Unable to save status update');
@@ -694,41 +771,35 @@ class Soap4 extends Soap
             throw new \SoapFault('Receiver', "Bandwidth Limit exceeded");
         }
 
-        if ($this->display->isAuditing()) {
-            $this->getLog()->debug('Received Screen shot');
-        }
+        $this->getLog()->debug('Received Screen shot');
 
         // Open this displays screen shot file and save this.
         $location = $this->getConfig()->getSetting('LIBRARY_LOCATION') . 'screenshots/' . $this->display->displayId . '_screenshot.' . $screenShotFmt;
 
-        foreach(array('imagick', 'gd') as $imgDriver) {
+        foreach (array('imagick', 'gd') as $imgDriver) {
             Img::configure(array('driver' => $imgDriver));
             try {
                 $screenShotImg = Img::make($screenShot);
             } catch (\Exception $e) {
-                if ($this->display->isAuditing())
-                    $this->getLog()->debug($imgDriver . " - " . $e->getMessage());
+                $this->getLog()->debug($imgDriver . ' - ' . $e->getMessage());
             }
-            if($screenShotImg !== false) {
-                if ($this->display->isAuditing())
-                    $this->getLog()->debug("Use " . $imgDriver);
+            if ($screenShotImg !== false) {
+                $this->getLog()->debug('Use ' . $imgDriver);
                 break;
             }
         }
 
         if ($screenShotImg !== false) {
-            $imgMime = $screenShotImg->mime(); 
+            $imgMime = $screenShotImg->mime();
 
-            if($imgMime != $screenShotMime) {
+            if ($imgMime != $screenShotMime) {
                 $needConversion = true;
                 try {
-                    if ($this->display->isAuditing())
-                        $this->getLog()->debug("converting: '" . $imgMime . "' to '" . $screenShotMime . "'");
+                    $this->getLog()->debug("converting: '" . $imgMime . "' to '" . $screenShotMime . "'");
                     $screenShot = (string) $screenShotImg->encode($screenShotFmt);
                     $converted = true;
                 } catch (\Exception $e) {
-                    if ($this->display->isAuditing())
-                        $this->getLog()->debug($e->getMessage());
+                    $this->getLog()->debug($e->getMessage());
                 }
             }
         }
