@@ -1,8 +1,8 @@
 <?php
 /*
- * Copyright (C) 2023 Xibo Signage Ltd
+ * Copyright (C) 2024 Xibo Signage Ltd
  *
- * Xibo - Digital Signage - http://www.xibo.org.uk
+ * Xibo - Digital Signage - https://xibosignage.com
  *
  * This file is part of Xibo.
  *
@@ -44,27 +44,24 @@ $app->group('', function(RouteCollectorProxy $group) {
 $app->get('/icondashboard', ['\Xibo\Controller\IconDashboard', 'displayPage'])
     ->setName('icondashboard.view');
 
-$app->group('', function(RouteCollectorProxy $group) {
+$app->group('', function (RouteCollectorProxy $group) {
     $group->get('/mediamanager', ['\Xibo\Controller\MediaManager', 'displayPage'])
         ->setName('mediamanager.view');
     $group->get('/mediamanager/data', ['\Xibo\Controller\MediaManager', 'grid'])
         ->setName('mediamanager.search');
 })->add(new FeatureAuth($app->getContainer(), ['dashboard.media.manager']));
 
-$app->group('', function(RouteCollectorProxy $group) {
+$app->group('', function (RouteCollectorProxy $group) {
     $group->get('/playlistdashboard', ['\Xibo\Controller\PlaylistDashboard', 'displayPage'])
         ->setName('playlistdashboard.view');
     $group->get('/playlistdashboard/data', ['\Xibo\Controller\PlaylistDashboard', 'grid'])
         ->setName('playlistdashboard.search');
     $group->get('/playlistdashboard/{id}', ['\Xibo\Controller\PlaylistDashboard', 'show'])
         ->setName('playlistdashboard.show');
-    $group->get('/playlistdashboard/widget/form/delete/{id}', ['\Xibo\Controller\PlaylistDashboard', 'deletePlaylistWidgetForm'])
-        ->setName('playlist.module.widget.delete.form');
-
-    //TODO: why is this commented out?
-    //$group->map('/playlistdashboard/library', '\Xibo\Controller\PlaylistDashboard:upload')->via('HEAD');
-    $group->post('/playlistdashboard/library', ['\Xibo\Controller\PlaylistDashboard', 'upload'])
-        ->setName('playlistdashboard.library.add');
+    $group->get('/playlistdashboard/widget/form/delete/{id}', [
+        '\Xibo\Controller\PlaylistDashboard',
+        'deletePlaylistWidgetForm',
+    ])->setName('playlist.module.widget.delete.form');
 })->add(new FeatureAuth($app->getContainer(), ['dashboard.playlist']));
 
 // Login Form
@@ -88,7 +85,11 @@ $app->get('/schedule/view', ['\Xibo\Controller\Schedule','displayPage'])
     ->add(new FeatureAuth($app->getContainer(), ['schedule.view']))
     ->setName('schedule.view');
 
-$app->get('/schedule/form/add', ['\Xibo\Controller\Schedule','addForm'])
+$app->get('/schedule/grid/view', ['\Xibo\Controller\Schedule','gridPage'])
+    ->add(new FeatureAuth($app->getContainer(), ['schedule.view']))
+    ->setName('schedule.grid.view');
+
+$app->get('/schedule/form/add[/{from}/{id}]', ['\Xibo\Controller\Schedule','addForm'])
     ->add(new FeatureAuth($app->getContainer(), ['schedule.add']))
     ->setName('schedule.add.form');
 
@@ -103,10 +104,6 @@ $app->group('', function(\Slim\Routing\RouteCollectorProxy $group) {
         ->setName('schedule.recurrence.delete.form');
 })->add(new FeatureAuth($app->getContainer(), ['schedule.modify']));
 
-$app->get('/schedule/form/now/{from}/{id}', ['\Xibo\Controller\Schedule','scheduleNowForm'])
-    ->add(new FeatureAuth($app->getContainer(), ['schedule.now']))
-    ->setName('schedule.now.form');
-
 //
 // notification
 //
@@ -137,13 +134,17 @@ $app->get('/layout/view', ['\Xibo\Controller\Layout', 'displayPage'])
     ->setName('layout.view');
 
 $app->group('', function(\Slim\Routing\RouteCollectorProxy $group) {
-    $group->get('/layout/preview/{id}', ['\Xibo\Controller\Preview', 'show'])->setName('layout.preview');
     $group->get('/layout/xlf/{id}', ['\Xibo\Controller\Preview', 'getXlf'])->setName('layout.getXlf');
     $group->get('/layout/background/{id}', ['\Xibo\Controller\Layout', 'downloadBackground'])->setName('layout.download.background');
     $group->get('/layout/thumbnail/{id}', ['\Xibo\Controller\Layout', 'downloadThumbnail'])->setName('layout.download.thumbnail');
+    $group->get('/layout/playerBundle', ['\Xibo\Controller\Preview', 'playerBundle'])->setName('layout.preview.bundle');
     $group->get('/connector/widget/preview', ['\Xibo\Controller\Connector', 'connectorPreview'])->setName('layout.preview.connector');
 })->addMiddleware(new FeatureAuth($app->getContainer(), ['layout.view', 'template.view']));
 
+$app->get('/layout/preview/{id}', ['\Xibo\Controller\Preview', 'show'])
+    ->addMiddleware(new FeatureAuth($app->getContainer(), ['layout.view', 'template.view', 'campaign.view']))
+    ->setName('layout.preview');
+
 // forms
 $app->get('/layout/form/add', ['\Xibo\Controller\Layout','addForm'])
     ->addMiddleware(new FeatureAuth($app->getContainer(), ['layout.add']))
@@ -155,6 +156,7 @@ $app->group('', function(\Slim\Routing\RouteCollectorProxy $group) {
     $group->get('/layout/form/background/{id}', ['\Xibo\Controller\Layout', 'editBackgroundForm'])->setName('layout.background.form');
     $group->get('/layout/form/copy/{id}', ['\Xibo\Controller\Layout', 'copyForm'])->setName('layout.copy.form');
     $group->get('/layout/form/delete/{id}', ['\Xibo\Controller\Layout', 'deleteForm'])->setName('layout.delete.form');
+    $group->get('/layout/form/clear/{id}', ['\Xibo\Controller\Layout', 'clearForm'])->setName('layout.clear.form');
     $group->get('/layout/form/checkout/{id}', ['\Xibo\Controller\Layout', 'checkoutForm'])->setName('layout.checkout.form');
     $group->get('/layout/form/publish/{id}', ['\Xibo\Controller\Layout', 'publishForm'])->setName('layout.publish.form');
     $group->get('/layout/form/discard/{id}', ['\Xibo\Controller\Layout', 'discardForm'])->setName('layout.discard.form');
@@ -183,27 +185,27 @@ $app->get('/region/preview/{id}', ['\Xibo\Controller\Region','preview'])
     ->setName('region.preview');
 
 $app->group('', function(\Slim\Routing\RouteCollectorProxy $group) {
-    $group->get('/region/form/edit/{id}', ['\Xibo\Controller\Region', 'editForm'])->setName('region.edit.form');
-    $group->get('/region/form/delete/{id}', ['\Xibo\Controller\Region', 'deleteForm'])->setName('region.delete.form');
+    $group->get('/region/{id}', ['\Xibo\Controller\Region', 'get'])->setName('region.get');
 
     // Designer
     $group->get('/playlist/form/library/assign/{id}', ['\Xibo\Controller\Playlist','libraryAssignForm'])->setName('playlist.library.assign.form');
 
     // Outputs
-    $group->get('/playlist/widget/tab/{tab}/{id}', ['\Xibo\Controller\Module','getTab'])->setName('module.widget.tab.form');
-    $group->get('/playlist/widget/resource/{regionId}/{id}', ['\Xibo\Controller\Module','getResource'])->setName('module.getResource');
-    $group->get('/playlist/widget/form/templateimage/{type}/{templateId}', ['\Xibo\Controller\Module','getTemplateImage'])->setName('module.getTemplateImage');
+    $group->get('/playlist/widget/resource/{regionId}[/{id}]', [
+        '\Xibo\Controller\Widget', 'getResource'
+    ])->setName('module.getResource');
+
+    $group->get('/playlist/widget/data/{regionId}/{id}', [
+        '\Xibo\Controller\Widget', 'getData'
+    ])->setName('module.getData');
 })->addMiddleware(new FeatureAuth($app->getContainer(), ['layout.modify']));
 
 $app->group('', function (\Slim\Routing\RouteCollectorProxy $group) {
-    // Module functions
-    $group->get('/playlist/widget/form/edit/{id}', ['\Xibo\Controller\Module','editWidgetForm'])->setName('module.widget.edit.form');
-    $group->get('/playlist/widget/form/delete/{id}', ['\Xibo\Controller\Module','deleteWidgetForm'])->setName('module.widget.delete.form');
-    $group->get('/playlist/widget/form/transition/edit/{type}/{id}', ['\Xibo\Controller\Module','editWidgetTransitionForm'])->setName('module.widget.transition.edit.form');
-    $group->get('/playlist/widget/form/audio/{id}', ['\Xibo\Controller\Module','widgetAudioForm'])->setName('module.widget.audio.form');
-    $group->get('/playlist/widget/form/expiry/{id}', ['\Xibo\Controller\Module','widgetExpiryForm'])->setName('module.widget.expiry.form');
-    $group->get('/playlist/widget/dataset', ['\Xibo\Controller\Module','getDataSets'])->setName('module.widget.dataset.search');
-    $group->get('/playlist/widget/menuboard', ['\Xibo\Controller\Module','getMenuBoards'])->setName('module.widget.menuboard.search');
+    // Widget functions
+    $group->get('/playlist/widget/{id}', ['\Xibo\Controller\Widget','getWidget'])->setName('module.widget.get');
+    $group->get('/playlist/widget/form/transition/edit/{type}/{id}', ['\Xibo\Controller\Widget','editWidgetTransitionForm'])->setName('module.widget.transition.edit.form');
+    $group->get('/playlist/widget/form/audio/{id}', ['\Xibo\Controller\Widget','widgetAudioForm'])->setName('module.widget.audio.form');
+    $group->get('/playlist/widget/form/expiry/{id}', ['\Xibo\Controller\Widget','widgetExpiryForm'])->setName('module.widget.expiry.form');
 })->addMiddleware(new FeatureAuth($app->getContainer(), ['layout.modify', 'playlist.modify']));
 
 //
@@ -225,8 +227,6 @@ $app->group('', function(\Slim\Routing\RouteCollectorProxy $group) {
     $group->get('/playlist/form/{id}/selectfolder', ['\Xibo\Controller\Playlist','selectFolderForm'])->setName('playlist.selectfolder.form');
 })->addMiddleware(new FeatureAuth($app->getContainer(), ['playlist.modify']));
 
-// What permissions do we need to be able to see the timeline form?
-// this can be accessed from the designer and the playlist page
 $app->get('/playlist/form/timeline/{id}', ['\Xibo\Controller\Playlist','timelineForm'])->setName('playlist.timeline.form');
 
 $app->get('/playlist/form/usage/{id}', ['\Xibo\Controller\Playlist','usageForm'])
@@ -239,6 +239,9 @@ $app->get('/playlist/form/usage/{id}', ['\Xibo\Controller\Playlist','usageForm']
 $app->get('/library/search', ['\Xibo\Controller\Library','search'])
     ->setName('library.search.all');
 
+$app->get('/library/connector/list', ['\Xibo\Controller\Library','providersList'])
+    ->setName('library.search.providers');
+
 $app->get('/library/view', ['\Xibo\Controller\Library','displayPage'])
     ->addMiddleware(new FeatureAuth($app->getContainer(), ['library.view']))
     ->setName('library.view');
@@ -264,9 +267,6 @@ $app->get('/library/form/usage/{id}', ['\Xibo\Controller\Library','usageForm'])
     ->addMiddleware(new FeatureAuth($app->getContainer(), ['schedule.view', 'layout.view']))
     ->setName('library.usage.form');
 
-$app->get('/library/fontcss', ['\Xibo\Controller\Library','fontCss'])->setName('library.font.css');
-$app->get('/library/fontlist', ['\Xibo\Controller\Library','fontList'])->setName('library.font.list');
-
 //
 // display
 //
@@ -371,13 +371,14 @@ $app->get('/campaign/form/{id}/selectfolder', ['\Xibo\Controller\Campaign','sele
     ->setName('campaign.selectfolder.form');
 
 $app->get('/campaign/{id}/preview', ['\Xibo\Controller\Campaign','preview'])
-    ->addMiddleware(new FeatureAuth($app->getContainer(), ['campaign.view']))
-    ->addMiddleware(new FeatureAuth($app->getContainer(), ['layout.view']))
+    ->addMiddleware(new FeatureAuth($app->getContainer(), ['campaign.view', 'layout.view']))
     ->setName('campaign.preview');
 
 //
 // template
 //
+$app->get('/template/connector/list', ['\Xibo\Controller\Template','providersList'])
+    ->setName('template.search.providers');
 $app->get('/template/search', ['\Xibo\Controller\Template', 'search'])->setName('template.search.all');
 $app->get('/template/view', ['\Xibo\Controller\Template','displayPage'])
     ->addMiddleware(new FeatureAuth($app->getContainer(), ['template.view']))
@@ -423,6 +424,11 @@ $app->group('', function(\Slim\Routing\RouteCollectorProxy $group) {
     $group->get('/dataset/form/import/{id}', ['\Xibo\Controller\DataSet', 'importForm'])->setName('dataSet.import.form');
     $group->get('/dataset/form/cache/clear/{id}', ['\Xibo\Controller\DataSet', 'clearCacheForm'])->setName('dataSet.clear.cache.form');
     $group->post('/dataset/cache/clear/{id}', ['\Xibo\Controller\DataSet', 'clearCache'])->setName('dataSet.clear.cache');
+    $group->get('/dataset/form/{id}/selectfolder', ['\Xibo\Controller\DataSet', 'selectFolderForm'])->setName('dataSet.selectfolder.form');
+
+    $group->get('/dataset/dataConnector/{id}', ['\Xibo\Controller\DataSet', 'dataConnectorView'])->setName('dataSet.dataConnector.view');
+    $group->get('/dataset/dataConnector/request/{id}', ['\Xibo\Controller\DataSet', 'dataConnectorRequest'])->setName('dataSet.dataConnector.request');
+    $group->get('/dataset/dataConnector/test/{id}', ['\Xibo\Controller\DataSet', 'dataConnectorTest'])->setName('dataSet.dataConnector.test');
 
     // columns
     $group->get('/dataset/{id}/column/view', ['\Xibo\Controller\DataSetColumn','displayPage'])->setName('dataSet.column.view');
@@ -557,25 +563,59 @@ $app->group('', function(\Slim\Routing\RouteCollectorProxy $group) {
 
 $app->get('/application/authorize', ['\Xibo\Controller\Applications','authorizeRequest'])->setName('application.authorize.request');
 $app->post('/application/authorize', ['\Xibo\Controller\Applications','authorize'])->setName('application.authorize');
-$app->delete('/application/revoke/{id}/{userId}', ['\Xibo\Controller\Applications', 'revokeAccess'])->setName('application.revoke');
 
 //
 // module
 //
-$app->group('', function(\Slim\Routing\RouteCollectorProxy $group) {
+$app->group('', function (\Slim\Routing\RouteCollectorProxy $group) {
     $group->get('/module/view', ['\Xibo\Controller\Module','displayPage'])->setName('module.view');
-    $group->post('/module/inst/{name}', ['\Xibo\Controller\Module','install'])->setName('module.install');
-    $group->delete('/module/uninst/{id}', ['\Xibo\Controller\Module','uninstall'])->setName('module.uninstall');
-    $group->get('/module/form/inst/{name}', ['\Xibo\Controller\Module','installForm'])->setName('module.install.form');
-    $group->get('/module/form/uninst/{id}', ['\Xibo\Controller\Module','uninstallForm'])->setName('module.uninstall.form');
-    $group->get('/module/form/instlist', ['\Xibo\Controller\Module','installListForm'])->setName('module.install.list.form');
-    $group->get('/module/form/verify', ['\Xibo\Controller\Module','verifyForm'])->setName('module.verify.form');
-    $group->get('/module/form/clear-cache/{id}', ['\Xibo\Controller\Module','clearCacheForm'])->setName('module.clear.cache.form');
-    $group->get('/module/form/settings/{id}', ['\Xibo\Controller\Module','settingsForm'])->setName('module.settings.form');
-    $group->get('/module/form/{id}/custom/{name}', ['\Xibo\Controller\Module','customFormRender'])->setName('module.custom.form');
-    $group->map(['GET','POST'], '/module/{id}/custom/{name}', ['\Xibo\Controller\Module','customFormExecute'])->setName('module.custom');
+    $group->get('/module/form/clear-cache/{id}', ['\Xibo\Controller\Module','clearCacheForm'])
+        ->setName('module.clear.cache.form');
+
+    $group->get('/module/form/settings/{id}', ['\Xibo\Controller\Module','settingsForm'])
+        ->setName('module.settings.form');
 })->addMiddleware(new FeatureAuth($app->getContainer(), ['module.view']));
 
+//
+// Developer
+//
+$app->group('', function (\Slim\Routing\RouteCollectorProxy $group) {
+    $group->get('/developer/template/datatypes', ['\Xibo\Controller\Developer', 'getAvailableDataTypes'])
+        ->setName('developer.templates.datatypes.search');
+    $group->get('/developer/template/view', ['\Xibo\Controller\Developer', 'displayTemplatePage'])
+        ->setName('developer.templates.view');
+    $group->get('/developer/template', ['\Xibo\Controller\Developer', 'templateGrid'])
+        ->setName('developer.templates.search');
+
+    $group->get('/developer/template/{id}', ['\Xibo\Controller\Developer', 'displayTemplateEditPage'])
+        ->setName('developer.templates.view.edit');
+
+    $group->get('/developer/template/form/add', ['\Xibo\Controller\Developer', 'templateAddForm'])
+        ->setName('developer.templates.form.add');
+
+    $group->get('/developer/template/form/edit/{id}', ['\Xibo\Controller\Developer', 'templateEditForm'])
+        ->setName('developer.templates.form.edit');
+
+    $group->get('/developer/template/form/delete/{id}', ['\Xibo\Controller\Developer', 'templateDeleteForm'])
+        ->setName('developer.templates.form.delete');
+
+    $group->get('/developer/template/form/copy/{id}', ['\Xibo\Controller\Developer', 'templateCopyForm'])
+        ->setName('developer.templates.form.copy');
+
+    $group->post('/developer/template', ['\Xibo\Controller\Developer', 'templateAdd'])
+        ->setName('developer.templates.add');
+    $group->put('/developer/template/{id}', ['\Xibo\Controller\Developer', 'templateEdit'])
+        ->setName('developer.templates.edit');
+    $group->delete('/developer/template/{id}', ['\Xibo\Controller\Developer', 'templateDelete'])
+        ->setName('developer.templates.delete');
+    $group->get('/developer/template/{id}/export', ['\Xibo\Controller\Developer', 'templateExport'])
+        ->setName('developer.templates.export');
+    $group->post('/developer/template/import', ['\Xibo\Controller\Developer', 'templateImport'])
+        ->setName('developer.templates.import');
+    $group->post('/developer/template/{id}/copy', ['\Xibo\Controller\Developer', 'templateCopy'])
+        ->setName('developer.templates.copy');
+})->addMiddleware(new FeatureAuth($app->getContainer(), ['developer.edit']));
+
 //
 // transition
 //
@@ -604,16 +644,6 @@ $app->get('/fault/view', ['\Xibo\Controller\Fault','displayPage'])
 //
 $app->get('/license/view', ['\Xibo\Controller\Login','about'])->setName('license.view');
 
-//
-// help
-//
-$app->group('', function(\Slim\Routing\RouteCollectorProxy $group) {
-    $group->get('/help/view', ['\Xibo\Controller\Help','displayPage'])->setName('help.view');
-    $group->get('/help/form/add', ['\Xibo\Controller\Help','addForm'])->setName('help.add.form');
-    $group->get('/help/form/edit/{id}', ['\Xibo\Controller\Help','editForm'])->setName('help.edit.form');
-    $group->get('/help/form/delete/{id}', ['\Xibo\Controller\Help','deleteForm'])->setName('help.delete.form');
-})->addMiddleware(new FeatureAuth($app->getContainer(), ['help.view']));
-
 //
 // Reporting
 //
@@ -723,15 +753,9 @@ $app->group('', function(\Slim\Routing\RouteCollectorProxy $group) {
     $group->get('/tag/form/add', ['\Xibo\Controller\Tag','addForm'])->setName('tag.add.form');
     $group->get('/tag/form/edit/{id}', ['\Xibo\Controller\Tag','editForm'])->setName('tag.edit.form');
     $group->get('/tag/form/delete/{id}', ['\Xibo\Controller\Tag','deleteForm'])->setName('tag.delete.form');
+    $group->get('/tag/form/usage/{id}', ['\Xibo\Controller\Tag', 'usageForm'])->setName('tag.usage.form');
 })->addMiddleware(new FeatureAuth($app->getContainer(), ['tag.view']));
 
-// Actions
-$app->group('', function(\Slim\Routing\RouteCollectorProxy $group) {
-    $group->get('/action/form/add/{source}/{id}', ['\Xibo\Controller\Action', 'addForm'])->setName('action.add.form');
-    $group->get('/action/form/edit/{id}', ['\Xibo\Controller\Action', 'editForm'])->setName('action.edit.form');
-    $group->get('/action/form/delete/{id}', ['\Xibo\Controller\Action', 'deleteForm'])->setName('action.delete.form');
-})->addMiddleware(new FeatureAuth($app->getContainer(), ['layout.modify', 'playlist.modify']));
-
 // Menu Boards
 $app->group('', function (\Slim\Routing\RouteCollectorProxy $group) {
     $group->get('/menuboard/view', ['\Xibo\Controller\MenuBoard','displayPage'])->setName('menuBoard.view');
@@ -754,3 +778,23 @@ $app->group('', function (\Slim\Routing\RouteCollectorProxy $group) {
 $app->group('', function (RouteCollectorProxy $group) {
     $group->get('/folders/form/{folderId}/move', ['\Xibo\Controller\Folder', 'moveForm'])->setName('folders.move.form');
 })->addMiddleware(new FeatureAuth($app->getContainer(), ['folder.modify']));
+
+$app->get('/fonts/fontcss', ['\Xibo\Controller\Font','fontCss'])->setName('library.font.css');
+
+$app->group('', function (RouteCollectorProxy $group) {
+    $group->get('/fonts/view', ['\Xibo\Controller\Font', 'displayPage'])->setName('font.view');
+    $group->get('/fonts/{id}/form/delete', ['\Xibo\Controller\Font', 'deleteForm'])->setName('font.form.delete');
+})->addMiddleware(new FeatureAuth($app->getContainer(), ['font.view']));
+
+$app->group('', function (RouteCollectorProxy $group) {
+    $group->get('/syncgroup/view', ['\Xibo\Controller\SyncGroup', 'displayPage'])->setName('syncgroup.view');
+    $group->get('/syncgroup/form/add', ['\Xibo\Controller\SyncGroup', 'addForm'])->setName('syncgroup.form.add');
+    $group->get('/syncgroup/form/{id}/members', ['\Xibo\Controller\SyncGroup', 'membersForm'])->setName('syncgroup.form.members');
+    $group->get('/syncgroup/form/{id}/edit', ['\Xibo\Controller\SyncGroup', 'editForm'])->setName('syncgroup.form.edit');
+    $group->get('/syncgroup/form/{id}/delete', ['\Xibo\Controller\SyncGroup', 'deleteForm'])->setName('syncgroup.form.delete');
+})->addMiddleware(new FeatureAuth($app->getContainer(), ['display.syncView']));
+
+$app->group('', function (RouteCollectorProxy $group) {
+    $group->get('/schedule/form/sync', ['\Xibo\Controller\Schedule', 'syncForm'])->setName('schedule.add.sync.form');
+    $group->get('/schedule/form/{id}/sync', ['\Xibo\Controller\Schedule', 'syncEditForm'])->setName('schedule.edit.sync.form');
+})->addMiddleware(new FeatureAuth($app->getContainer(), ['schedule.sync']));
